import java.io.*;
import java.net.*;
import java.applet.Applet;

class Project {
  // constants
  final String sNIL = "NIL";
  final int iNIL = -1;
  final int DAYSECONDS = 86400; // 3600*24

  // implementation
  String projectName;
  Link projectNameLink;
  String projectTitle;
  String phaseName;
  String startLabel;
  String endLabel;

  int startDate;
  int todayDate;
  int endDate;
  int dateMin = iNIL;
  int dateMax = iNIL;

  TimeFrame timeFrame = null;
  Task[] taskList = null;

  // --------------------------------------------------------------
  //                          Constructor
  // --------------------------------------------------------------
  Project()
  {
    timeFrame = new TimeFrame();
    projectNameLink = new Link();
  }

  // --------------------------------------------------------------
  //                        Private Methods
  // --------------------------------------------------------------
  String parseString(String sLine)
  {
    int a,b;

    // empty string
    if(sLine.equals(""))  
      return "";

    a = sLine.indexOf('"');
    b = sLine.lastIndexOf('"');
    return sLine.substring(a+1,b);
  }

  String parseYear(String sLine)
  {
    int a,b;

    a = sLine.indexOf('<');
    b = sLine.indexOf('>');

    // error
    if(a==-1 || b==-1 || b<a)
      return null;

    return sLine.substring(a+1,b);
  }

  String readString(DataInputStream dstream) throws IOException
  {
    String sLine;

    try {
      sLine = dstream.readLine();
    }
    catch (IOException e) {
      throw e;
    }

    return parseString(sLine);
  }

  int readInt(DataInputStream dstream) throws IOException
  {
    String sLine;

    try {
      sLine = dstream.readLine();
    }
    catch (IOException e) {
      throw e;
    }

    return Integer.parseInt(sLine);
  }

  int seconds2Days(int seconds)
  {
    return(seconds / DAYSECONDS);
  }

  int days2Seconds(int days)
  {
    return(days * DAYSECONDS);
  }

  int readUDate(DataInputStream dstream) throws IOException
  {
    String sLine;
    int result;

    try {
      sLine = dstream.readLine();
    }
    catch (IOException e) {
      throw e;
    }

    // NIL
    if(sLine.equals(sNIL))
      return iNIL;

    result = seconds2Days(Integer.parseInt(sLine));

    // update max & min dates
    if(dateMax==iNIL && dateMin==iNIL) {
      dateMax = result;
      dateMin = result;
    }
    else {
      if(dateMax < result) dateMax = result;
      if(dateMin > result) dateMin = result;
    }

    return(result);
  }

  // --------------------------------------------------------------
  //                        Public Methods
  // --------------------------------------------------------------
  //public boolean readDataSource(URL documentBase, String dataSource) {
  public boolean readDataSource(InputStream istream) {
  //   InputStream istream = null;
  //   URL   datasourceURL = null;
     boolean result;

  if (istream == null)
    return false;

  //   if ((dataSource == null)    || 
  //       (dataSource.equals("")) || 
  //       (dataSource.charAt(0) < 'A')) { 
  //     return false;
  //   }

  //   // absolute URL 
  //   try {
  //      datasourceURL = new URL(dataSource);
  //   }
  //   catch( MalformedURLException e )
  //   {
  //      datasourceURL = null;
  //   }

  //   if (datasourceURL == null) {
  //     // document based URL
  //     try {
  //        datasourceURL = new URL(documentBase,
  //                                dataSource);
  //     }
  //     catch( Exception ex)
  //     {
  //        datasourceURL = null;
  //     }
  //   }

  //   try {
  //      istream = datasourceURL.openStream();
  //   }
  //   catch(Exception e)
  //   {          
  //       System.err.println("Unable to open URL data source.");
  //       return false;
  //   }

     result = parseInputStream(istream );

     try {
       istream.close();
     }
     catch (IOException e) {
       System.err.println("Unable to close URL data source.");
       return false;
     }


/*
     // absolute URL 
     try {
        datasourceURL = new URL(dataSource);
     }
     catch( MalformedURLException e )
     {
//       // document based URL
//       try {
//          Applet p = (Applet)getParent();
//          datasourceURL = new URL(p.getDocumentBase(),
//                                  dataSource);
//       }
//       catch( Exception ex)
//       {
//          datasourceURL = null;
//       }

       datasourceURL = null;
     }

     try {
        istream = datasourceURL.openStream();
     }
     catch(Exception e)
     {          
         System.err.println("Unable to open URL data source.");
         return false;
     }

     result = parseInputStream(istream );

     try {
       istream.close();
     }
     catch (IOException e) {
       System.err.println("Unable to close URL data source.");
       return false;
     }
*/

     return result;
  } // method:readDataSource

  public String getProjectName()
  {
    return(projectName);
  }

  public Link getProjectNameLink()
  {
    return(projectNameLink);
  }

  public String getProjectTitle()
  {
    return(projectTitle);
  }

  public String getPhaseName()
  {
    return(phaseName);
  }

  public String getStartLabel()
  {
    return(startLabel);
  }

  public String getEndLabel()
  {
    return(endLabel);
  }
 
  public int getStartDate()
  {
    return(startDate);
  }

  public int getTodayDate()
  {
    return(todayDate);
  }

  public int getEndDate()
  {
    return(endDate);
  }

  public Task[] getTasks()
  {
    return(taskList);
  }

  public TimeFrame getTimeFrame()
  {
    return(timeFrame);
  }

  public int getDateMax()
  {
    return(dateMax);
  }

  public int getDateMin()
  {
    return(dateMin);
  }

  public boolean parseInputStream(InputStream istream) {
    DataInputStream dStream = new DataInputStream(istream);

    try {
      int count;
      String sAux;

      projectName = readString(dStream);      // PROJECT NAME
      sAux = readString(dStream);             // URL (link) 
      projectNameLink.setURL(sAux);

      if(projectNameLink.getURL() != null) {  // TARGET FRAME
        sAux = readString(dStream);           // field presente only if
        projectNameLink.setFrame(sAux);       // url is given
      }
      else
        projectNameLink.setFrame("");

      projectTitle = readString(dStream);     // PROJECT TITLE
      phaseName = readString(dStream);        // PHASE NAME
      startLabel = readString(dStream);       // START DATE LABEL
      endLabel = readString(dStream);         // END DATE LABEL
      startDate = readUDate(dStream);         // STARTING DATE
      todayDate = readUDate(dStream);         // TODAY
      endDate = readUDate(dStream);           // ENDING DATE

      count = readInt(dStream);               // NUMBER OF TASKS
      taskList = new Task[count];
      int i;
      for(i=0; i<count; i++) {            // TASKS (phases/activities)
         int iAux;

         taskList[i] = new Task();

         sAux = readString(dStream);             // CAPTION
         taskList[i].setCaption(sAux);
         sAux = readString(dStream);
         taskList[i].setResponsavel(sAux);        // Responsavel
         sAux = readString(dStream);             // URL (link) 
         taskList[i].setCaptionURL(sAux);

         if(taskList[i].getCaptionURL() != null) {   // TARGET FRAME
           sAux = readString(dStream);               // field presente only if
           taskList[i].setCaptionFrame(sAux);        // url is given
         }
         else
           taskList[i].setCaptionFrame("");


         iAux = readUDate(dStream);              // ESTIMATED:STARTING DATE 
         taskList[i].setEstimatedStart(iAux);
         
         sAux = readString(dStream);						// ESTIMATED:STARTING DATE LABEL
         taskList[i].setEstimatedStartLabel(sAux);
         
         iAux = readUDate(dStream);              // ESTIMATED:ENDING DATE 
         taskList[i].setEstimatedEnd(iAux);
         
         sAux = readString(dStream);						// ESTIMATED:ENDING DATE LABEL
         taskList[i].setEstimatedEndLabel(sAux);
         
         iAux = readUDate(dStream);              // CURRENT:STARTING DATE 
         taskList[i].setCurrentStart(iAux);
         
         sAux = readString(dStream);						// CURRENT:STARTING DATE LABEL
         taskList[i].setCurrentStartLabel(sAux);
         
         iAux = readUDate(dStream);              // CURRENT:ENDING DATE 
         taskList[i].setCurrentEnd(iAux);
         
         sAux = readString(dStream);						//  CURRENT:ENDING DATE LABEL
         taskList[i].setCurrentEndLabel(sAux);
         
         sAux = readString(dStream);             // URL (link) 
         taskList[i].setCurrentURL(sAux);

         if(taskList[i].getCurrentURL() != null) {   // TARGET FRAME
           sAux = readString(dStream);               // field presente only if
           taskList[i].setCurrentFrame(sAux);        // url is given
         }
         else
           taskList[i].setCurrentFrame("");

         sAux = readString(dStream);             // STATUS
         taskList[i].setStatus(sAux);
      }

      YearSet yset;
      yset = new YearSet();
      count = readInt(dStream);               // NUMBER OF MONTHS
      String sLine = null;
      for(i=0; i<count; i++) {            // MONTHS
         int iAux;
 
         if(sLine == null) {                     // CAPTION
           // read caption
           sAux = readString(dStream);
         }
         else {
           // parse caption
           sAux = parseString(sLine);
         }

         iAux = readUDate(dStream);              // POSITION
         yset.addMonth(sAux,iAux);

         sLine = dStream.readLine();
         sAux = parseYear(sLine);
         if(sAux != null) {                      // [YEAR]
           // it's a year
            
           yset.setYear(sAux);
           timeFrame.addStripe(yset);
           if((i+1)<count)
             yset = new YearSet();
           sLine = null;
         }
      }
    }
    catch (ArrayIndexOutOfBoundsException e) {
      System.err.println("Array index out of bounds.");
      return false;
    }
    catch (Exception e) {
      System.err.println("Parsing data source failed.");
      return false;
    }

    return true;
  } // method:parseInputStream
}

