import java.awt.*;
import java.net.*;
import java.io.*;

public class Share
{
  final int iNIL = -1;

  int cursorX0;
  int cursorX1;
  int cursorY0;
  int cursorY1;

  String data = null;

  // .................................................................

  public boolean readProject(URL documentBase, String dataSource)
  {
    InputStream istream = null;
    StringBuffer buffer;
    URL datasourceURL;

    if ((dataSource == null)    || 
        (dataSource.equals("")) || 
        (dataSource.charAt(0) < 'A')) { 
      return false;
    }

    // absolute URL 
    try {
       datasourceURL = new URL(dataSource);
    }
    catch( MalformedURLException e )
    {
       datasourceURL = null;
    }

    if (datasourceURL == null) {
      // document based URL
      try {
         datasourceURL = new URL(documentBase,
                                 dataSource);
      }
      catch( Exception ex)
      {
         datasourceURL = null;
      }
    }

    try {
       istream = datasourceURL.openStream();
    }
    catch(Exception e)
    {          
        System.err.println("Unable to open URL data source.");
        return false;
    }

    DataInputStream dstream = new DataInputStream(istream);
    buffer = new StringBuffer();

    // store data file
    while(true) {
      try {
        String newline = readLine(dstream);
        if(newline != null) {
          buffer.append(newline);
          buffer.append("\n");
        }
        else {
          if(buffer != null) {
            data = buffer.toString();
            return true;
          }
          else {
            data = null;
            return false;
          }
        }
      }
      catch(IOException e) {
      }
    }
  }

  // .................................................................

  public String getData()
  {
    return data;
  }

  // .................................................................

  String readLine(DataInputStream dstream) throws IOException
  {
    try {
      return dstream.readLine();
    }
    catch(IOException e) {
      throw e;
    }
  }

  // .................................................................

  public void drawProject(Graphics g, 
                          Dimension d, 
  //                        URL documentBase, 
  //                        String dataSource, 
                          InputStream dataStream,
                          HotSpot hotspot)
  {
    int DATE_START;
    int DATE_TODAY;
    int DATE_END;
    final int x0 = 10;
    final int y0 = 60;               // initial y position for Title
    final int CAPTION_WIDTH = 80;
    final int CAPTION_HEIGHT = 40;
    final int USER_START = x0 +  CAPTION_WIDTH;
    final int USER_MARGIN = 5;
    final int leftBorderWidth = 47;
    final int rightBorderWidth = 47;
    int monthMin, monthMax;
    Task t[] = null;
    TimeFrame tf = null;
    TextGroup tg = null;
    TimeFlag tflag = null;
    CoordinateMapper cm = null;
    Project project = null;
    VisualProject vp = new VisualProject();
    int imageWidth = d.width;
    int imageHeight = d.height;

    //---Fonts available
    FontList fList = null;

    // Constants used in method drawProject
    final  int projectTitleHeight = 70;
    final  int timeFlagHeight = 30;
    final  int timebarHeight = 50;
    final  int legendHeight = 100;

    boolean debug=false;
    Color old;
    int ypos = y0;
    int ypos_tflag;
    int newHeight;
    String fontMatch;
    int taskHeight = 0;
    int taskWidth = 0;

    // determine fonts available
    fList = new FontList();

    // read data File
    project = new Project();
    if(project.readDataSource(dataStream)==false) {
      System.err.println("Read failed");
      return;
    }

    t = project.getTasks();
    tf = project.getTimeFrame();
    monthMin = project.getDateMin();
    monthMax = project.getDateMax();

    DATE_START = project.getStartDate();
    DATE_TODAY = project.getTodayDate();
    DATE_END = project.getEndDate();

    if(DATE_START < monthMin) monthMin = DATE_START;
    if(DATE_END > monthMax) monthMax = DATE_END;

    tf.setLeftBorderWidth(leftBorderWidth);
    tf.setRightBorderWidth(rightBorderWidth);

    taskHeight = vp.taskFrameHeight(t);
    taskWidth = vp.taskFrameWidth(t);

    newHeight = projectTitleHeight + timeFlagHeight + timebarHeight +
                taskHeight + legendHeight;

    cm = new CoordinateMapper(monthMin, monthMax,
                              x0 + taskWidth + leftBorderWidth, 
                              imageWidth - USER_MARGIN - rightBorderWidth);

    cursorX0 = cm.user2Screen(monthMin) - leftBorderWidth;
    cursorX1 = cm.user2Screen(monthMax) + rightBorderWidth;

    if(debug)
      g.setColor(Color.red);

    // set iNIL dates to "today"
    for(int i=0; i<t.length; i++)
      setNILs(t[i], DATE_TODAY);

    // DRAW PROJECT TITLE
    // title
    tg = new TextGroup();
    fontMatch = fList.get("Chicago");
    tg.add(project.getProjectName(),30,CustomColor.x9999CC,Font.BOLD,fontMatch);
    tg.add(project.getProjectTitle(),16);
    tg.setFrameAlign(-1);
    tg.setLabelAlign(-1);
    tg.draw(g,x0+2,ypos+2);

    // add hotspot
    if(hotspot != null) {
      int x = x0+2;
      int y = ypos+2;
      int width = tg.width(project.getProjectName());
      int height = tg.height(project.getProjectName());
      hotspot.add(x,y,width,height,project.getProjectNameLink());
    }

    if(debug)
      g.drawLine(1,ypos,10,ypos);

    // phase
    tg = new TextGroup();
    fontMatch = fList.get("Helvetica");
    tg.add(project.getPhaseName(),20,CustomColor.x9999CC,Font.BOLD,fontMatch);
    tg.add(project.getStartLabel(),12);
    tg.add(project.getEndLabel(),12);
    tg.setFrameAlign(1);
    tg.setLabelAlign(-1);
    tg.draw(g,imageWidth-USER_MARGIN,ypos);

    ypos += projectTitleHeight;
    if(debug)
      g.drawLine(1,ypos,10,ypos);

    ypos_tflag = ypos;
    ypos += timeFlagHeight;

    cursorY0 = ypos;

    vp.drawTimeBar(g,ypos,timebarHeight,timebarHeight+taskHeight,
                   monthMin,monthMax,
                   cm, tf);
    if(debug)
      g.drawLine(1,ypos,10,ypos);

    // DRAW TASKS
    vp.drawTasks(g,x0,ypos+timebarHeight,imageWidth-USER_MARGIN-x0,
                 taskWidth, CAPTION_HEIGHT,
                 cm, t, hotspot);
    if(debug)
      g.drawLine(1,ypos,10,ypos);

    // DRAW TIME FLAG (TODAY)
    tflag = new TimeFlag(cm.user2Screen(DATE_TODAY), ypos_tflag, 
                         timeFlagHeight, timebarHeight+taskHeight, "hoje", 
                         CustomColor.xFF3300);
    vp.drawTimeFlag(g,tflag,cm);

    if(debug) {
      g.drawLine(1,ypos,10,ypos);
      g.drawLine(1,ypos_tflag,10,ypos_tflag);
      g.drawLine(1,ypos_tflag+7,10,ypos_tflag+7);
      g.drawLine(1,ypos_tflag+7+5,10,ypos_tflag+7+5);
      g.drawLine(1,ypos_tflag+7+15,10,ypos_tflag+7+15);
      g.drawLine(1,ypos_tflag+timeFlagHeight,10,ypos_tflag+timeFlagHeight);
      g.drawLine(1,ypos_tflag+timeFlagHeight+timebarHeight,10,ypos_tflag+timeFlagHeight+timebarHeight);
      g.drawLine(1,ypos_tflag+timeFlagHeight+timebarHeight+taskHeight,10,ypos_tflag+timeFlagHeight+timebarHeight+taskHeight);
    }
    
    // DRAW FRAME BORDER
    old = g.getColor();
    g.setColor(CustomColor.x666666);
    // -- VERTICAL's
    g.drawLine(x0,                       ypos+timebarHeight,
               x0,                       ypos+timebarHeight+taskHeight);
    g.drawLine(x0+taskWidth,             ypos,
               x0+taskWidth,             ypos+timebarHeight+taskHeight);
    g.drawLine(imageWidth-USER_MARGIN, ypos,
               imageWidth-USER_MARGIN, ypos+timebarHeight+taskHeight);
    // -- HORIZONTAL's
    g.drawLine(x0,                        ypos+timebarHeight,
               imageWidth-USER_MARGIN,    ypos+timebarHeight);
    g.drawLine(x0+taskWidth,              ypos,
               imageWidth-USER_MARGIN,    ypos);
    g.drawLine(x0,                        ypos+timebarHeight+taskHeight,
               imageWidth-USER_MARGIN,    ypos+timebarHeight+taskHeight);

    g.setColor(old);

    ypos += timebarHeight + taskHeight;

    cursorY1 =  ypos;

    // DRAW LEGEND
    vp.drawLegend(g,x0,ypos,imageWidth-USER_MARGIN);  
    if(debug)
      g.drawLine(1,ypos,10,ypos);
  }

  // .................................................................

  public int cursorX0()
  {
    return cursorX0;
  }
  
  // .................................................................

  public int cursorX1()
  {
    return cursorX1;
  }
  
  // .................................................................

  public int cursorY0()
  {
    return cursorY0;
  }
  
  // .................................................................

  public int cursorY1()
  {
    return cursorY1;
  }

  // .................................................................

  void setNILs(Task t, int today)
  {
    if(t.getCurrentEnd() == iNIL)
      t.setCurrentEnd(today);
  }

  
}
