-- ------------------------------------------------------------------------------
-- CADASTRA.LUA
-- Responsavel pelo CADASTRAMENTO e EDICAO de projetos
-- ------------------------------------------------------------------------------
-- CARREGA BIBLIOTECAS
-- ------------------------------------------------------------------------------

-- ------------------------------------------------------------------------------
-- As operacoes de Cadastramento e Revisao encontram-se indisponiveis
-- ------------------------------------------------------------------------------
-- if cgi.botao == 'Grava Novo Projeto' then      
--   cgilua.redirect('../operacao_indisponivel.html?tipo_servico=cadastra')  
--   exit()
-- end   
-- ------------------------------------------------------------------------------
    

assert(dofile( 'acessabd.lua' ))
assert(dofile( 'global.lua' ))
assert(dofile( 'constantes.lua' ))
assert(dofile( 'func_calculoFinanceiro.lua' ))
assert(dofile( 'func_atualiza_proj_dt_prev_ativ.lua' ))

if ESTADO_BD ~= 'aberto' then
   AbreBD()
end

-- ---------------------------------------------------------------------------------------------------
-- OBS: O campo PROJ_PAE era utilizado para indicar se o projeto pertence ou no ao PLANO EMERGENCIAL
--      Atualmente essa informao no est sendo mais utilizada, entretanto o campo foi mantido para
--      manter a compatibilidade com projetos mais antigos que possuiam essa informao
-- ---------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------
-- Guarda o valor original de CGI.PROJ_COD para posteriormente atualizar PROJ_SIGLA
-- ---------------------------------------------------------------------------------------------------
if temConteudo( cgi.proj_cod ) then
   proj_codigo = cgi.proj_cod
end   

-- ----------------------------------------------------------------------------------------------------
-- Inicializa o valor de flag_valor_custo_dif com 0. Esse flag eh utilizado para comparar os valores 
-- fornecidos em custo anual (tabela CUSTO) e os jah cadastrados em custo mensal (CUSTO_PREVISTO) - 
-- caso entradas para esse ultimo sejam registradas, verificando se eles sao diferentes
-- ----------------------------------------------------------------------------------------------------
flag_valor_custo_dif = 0  

-- ----------------------------------------------------------------------------------------
-- Esta funcao inicializa o valor do CGI referente ao TOTAL do custo previsto anual 
-- Eh utilizada tanto no CADASTRO quanto na EDICAO dos projetos
-- ----------------------------------------------------------------------------------------
function InicializaCustoPrevistoTotal()
   local ano
   if not( temConteudo( cgi.custo_total ) ) then
      ano = tonumber( _ANO_INI_CUSTO_PEGASO_ )
      cgi.custo_total = 0
      while ano <= tonumber( _ANO_FIM_CUSTO_PEGASO_ ) do 
         if temConteudo(cgi['custo_previsto_'..ano]) then
            cgi.custo_total = tonumber(cgi.custo_total) + ( tonumber(cgi['custo_previsto_'..ano]) or 0 )
         end   
         ano = ano + 1
      end     
   else
      cgi.custo_total = gsub(cgi.custo_total,",",".")
   end 
end   

-- ----------------------------------------------------------------------------------------
-- Esta funcao inicializa os valores dos CGIs referentes ao custo previsto anual 
-- Eh utilizada tanto no CADASTRO quanto na EDICAO dos projetos
-- ----------------------------------------------------------------------------------------
function InicializaCustoPrevistoAnual()
   local ano
   ano = tonumber( _ANO_INI_CUSTO_PEGASO_ )
   while ano <= tonumber( _ANO_FIM_CUSTO_PEGASO_ ) do 
      if temConteudo( cgi['custo_previsto_'..ano] ) then
      cgi['custo_previsto_'..ano] = gsub(cgi['custo_previsto_'..ano], ',', '.')
      else
         cgi['custo_previsto_'..ano] = 0
      end   
      ano = ano + 1
   end 
end      

-- ----------------------------------------------------------------------------------------
-- Esta funcao efetua as insercoes/atualizacoes na tabela CUSTO
-- Recebe os valores referentes ao codigo do projeto e ao FLAG flag_valor_custo_dif
-- Eh utilizada tanto no CADASTRO quanto na EDICAO dos projetos
-- ----------------------------------------------------------------------------------------
function AtualizaCustoPrevistoAnual(cod_proj, flag)
   local ano
   ano = tonumber( _ANO_INI_CUSTO_PEGASO_ )    
   while ano <= tonumber( _ANO_FIM_CUSTO_PEGASO_ ) do                   	
      if ( temConteudo(cgi['custo_previsto_'..ano]) ) and flag ~= 1 then               
      	 custo = ConsultaBD("custo","*","proj_cod = "..cod_proj.." and custo_ano = "..ano,nil)
      	 cgi['custo_previsto_'..ano] = gsub(cgi['custo_previsto_'..ano],",",".")   
         if custo and custo.custo_prev_ano then                              	
            AlteraBD("custo", "custo_prev_ano = "..cgi["custo_previsto_"..ano].." ",
                     "proj_cod = "..cod_proj.." AND custo_ano = "..ano.." ")         	                                                     	
         else
            EscreveBD("custo", "proj_cod,custo_ano,custo_prev_ano", " "..cod_proj..", "..ano..", "..
                      cgi["custo_previsto_"..ano].." ")                  
         end              
      end
      ano = ano + 1           
   end  
end       

-- ----------------------------------------------------------------------------------------
-- Esta funcao efetua as atualizacoes na tabela PROJ_OT. 
-- Recebe valores referentes a OT OPERACIONAL, CLASSIFICACAO, SUB-CLASSIFICACAO e suas OTs
-- Eh utilizada tanto no CADASTRO quanto na EDICAO dos projetos 
-- Armazena os valores referentes a CLASSIFICACAO e SUBCLASSIFICACAO mesmo que nao tenha
-- sido informado o valor da OT OPERACIONAL ou da OT ESPECIFICA
-- ----------------------------------------------------------------------------------------
function AtualizaOtOperacional(proj_cod, classificacao, ot_classificacao, subclass, ot_subclass, ot_oper)
   local ot_sub = '' 
   
   -- --------------------------------
   -- OT_CLASS (OT da classificao) 
   -- --------------------------------
   ot_class = ConsultaBD2("classificacao a , ot b","b.ot_cod as ot_class, b.ot_chave ","a.clas_cod = "..classificacao.." and b.ot_chave = '"..ot_classificacao.."' ",nil )
 
   -- --------------------------------------------------------
   -- MONTAGEM DOS CAMPOS PARA INCLUSO EM proj_ot
   -- --------------------------------------------------------                     
   -- --------------------------------------------------------
   -- Classificacao e sua OT 
   -- --------------------------------------------------------            
   if temConteudo(classificacao) then 
      escreve_campo = "proj_cod, clas_cod, ot_cod"
      escreve_valor = proj_cod..", "..classificacao..", "..ot_class[1].ot_class
   end   
   -- -------------------------------------------------------------------
   -- Se tem sub-classificacao e for diferente de branco, nulo ou vazio
   -- -------------------------------------------------------------------
   if temConteudo(subclass) then
      subclassificacao = ConsultaBD("ot a, classificacao b","b.clas_cod as subclas, a.ot_cod as ot_subclas, a.ot_chave ",
                                    "a.ot_chave = '"..ot_subclass.."' and a.clas_cod = b.clas_cod",nil )                  
      if subclassificacao and subclassificacao.ot_chave then
         ot_sub = subclassificacao.ot_chave
      end    
      -- --------------------------------------------
      -- Sub-Classificacao e sua OT 
      -- --------------------------------------------
      if (subclassificacao and temConteudo(subclassificacao.subclas) ) then
         escreve_campo = escreve_campo..", proj_ot_subcl_cod, proj_ot_ot_subclas "
         escreve_valor = escreve_valor..", "..subclass..", "..subclassificacao.ot_subclas             
      end                         
   else  -- if temConteudo(subclassificacao) 
      if ot_class[1] and ot_class[1].ot_chave then
         ot_sub = ot_class[1].ot_chave
      end   
   end                                 
   -- --------------------------------------------
   -- OT Operacional
   -- --------------------------------------------            
   if ( temConteudo(ot_oper) ) then
      escreve_campo = escreve_campo..", ot_oper_cod "
      escreve_valor = escreve_valor..", "..ot_oper
   end	                        

   ApagaBD("proj_ot", "proj_cod = "..proj_cod.." ")
   EscreveBD("proj_ot", escreve_campo, escreve_valor)  
   
   return (ot_sub)
end -- fim function AtualizaOtOperacional()


-- ----------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------
-- CADASTRA.LUA  (CADASTRO/EDICAO)
-- ----------------------------------------------------------------------------------------------------
-- ----------------------------------------------------------------------------------------------------

if cgi.botao == 'Grava' or cgi.botao == 'Grava Novo Projeto' or cgi.botao == 'Detalhar_Custo' or 
   cgi.botao == 'Atividades' or cgi.botao == 'Obs' or cgi.botao == 'Dutos' or
   cgi.botao == 'Formaliza' then

   -- ---------------------------------------------------------------------------------------
   -- Inicializa o valor do CGI referente ao TOTAL do Custo Previsto Anual (cgi.custo_total) 
   -- --------------------------------------------------------------------------------------- 
   InicializaCustoPrevistoTotal()

   -- ------------------------------------------------------------------
   -- Inicializa os valores dos CGIs referentes ao Custo Previsto Anual 
   -- ------------------------------------------------------------------            
   InicializaCustoPrevistoAnual()      
  

   -- ----------------------------------
   -- CARREGA A BIBLIOTECA VALIDAFORM
   -- ----------------------------------
   assert(dofile("../validaform/validaform.lua"))
   assert(dofile("form_cadastra.lua"))     
     
   cgi.msg = form_cadastra:critica()  
   -- TORNANDO OS VALORES DE CUSTO PREVISTO OBRIGATRIOS DENTRO DO INTERVALO DEFINIDO PELAS
   -- DATAS DE INCIO E FIM PREVISTAS DO PROJETO
   if cgi.botao ~= 'Detalhar_Custo' then
      if not( temConteudo(cgi.msg_dt_ini_prevista) ) and not( temConteudo(cgi.msg_dt_fim_prevista) ) then
         ano_ini = strsub(cgi.dt_ini_prevista,7,10)   
         ano_fim = strsub(cgi.dt_fim_prevista,7,10)
         ano_custo_fim = tonumber(_ANO_FIM_CUSTO_PEGASO_)
         i = tonumber(ano_ini)
         while i <= tonumber(ano_fim) do
            if (not temConteudo(cgi['custo_previsto_'..i]) or tonumber(cgi['custo_previsto_'..i]) == 0) and i <= ano_custo_fim then 
               cgi['msg_custo_previsto_'..i] = 'Preencha o campo.'

               if not cgi.msg then
                  cgi.msg = 'O formulrio foi preenchido incorretamente. Siga as instrues junto aos campos que devem ser revistos.'
               end
            end
            i = i + 1
         end      
      end	
   end
   
   if cgi.coordenador and cgi.coordenador ~= '' then
      confere_coord = "SELECT pes_chave FROM pessoal WHERE pes_chave = '"..cgi.coordenador.."'"
      DBExec(confere_coord)
      resultado_coord = DBRow()
      if not resultado_coord or not resultado_coord.pes_chave then
         cgi.msg_coordenador = 'O usurio no consta nos cadastros.'
         if not cgi.msg then
            cgi.msg = 'O formulrio foi preenchido incorretamente. Siga as instrues junto aos campos que devem ser revistos.'         
         end
      end
   end  
   
   -- ----------------------------------------------------------------------------
   -- Verifica se o valor de custo anual  diferente do valor do custo mensal. 
   -- ---------------------------------------------------------------------------- 
   if cgi.tipo_servico == 'edita' then     
      consulta_custo_previsto = "SELECT proj_cod FROM custo_previsto WHERE proj_cod = " .. cgi.proj_cod
      DBExec(consulta_custo_previsto)
      resultado_custo_previsto = DBRow()      
      if ((resultado_custo_previsto) and (resultado_custo_previsto.proj_cod)) then
         i = tonumber( _ANO_INI_CUSTO_PEGASO_ )
         while ( i <= tonumber( _ANO_FIM_CUSTO_PEGASO_ ) ) do
            local consulta = "SELECT SUM(valor_real) as valor_real FROM custo_previsto WHERE year(dt_prevista) = "..i..
                             " AND proj_cod = "..cgi.proj_cod
            DBExec(consulta)
            local resultado = DBRow()
            -- -----------------------------------------------------------------------------------------
            -- Verifica se o somatrio de custo mensal para um dado ano em um projeto no  NULL (nil)
            -- -----------------------------------------------------------------------------------------             
            if resultado and resultado.valor_real == nil then            	
            	  som_custo_mensal = 0
            else               
                som_custo_mensal = resultado.valor_real
            end 	
            cgi['custo_previsto_'..i] = gsub(cgi['custo_previsto_'..i],',','.')  


            ----------------------------------------------------------------------
            --Remove o custo previsto, se a data de inicio previsto do projeto, --
            --for superior a data do custo incluido na tabela.                  --
            ----------------------------------------------------------------------
            if (tonumber(cgi['custo_previsto_'..i]) ~= tonumber(som_custo_mensal)) and 
               (tonumber(strsub(cgi.dt_ini_prevista,7,10)) > i) then

            local limpa = "DELETE FROM custo_previsto WHERE year(dt_prevista) = "..i..
                             " AND proj_cod = "..cgi.proj_cod
            DBExec(limpa)
            elseif (tonumber(cgi['custo_previsto_'..i]) ~= tonumber(som_custo_mensal)) then
            	flag_valor_custo_dif = 1
            end
            i = i + 1
         end      
         if cgi.botao ~= "Detalhar_Custo" then            	
            if (flag_valor_custo_dif == 1) then
               cgi.msg_custo_anual_dif = 'Os valores de custo anual esto diferentes dos valores detalhados em custo mensal.<BR>V em "<i>detalhar custos</i>" para corrigir esses valores.'
               if not (cgi.msg) then
               	  cgi.msg = 'O formulrio foi preenchido incorretamente. Siga as instrues junto aos campos que devem ser revistos.'
               end  
            end         
         end      
      end
      -- -----------------------------------------------------------------------------------------------------------------
      -- Caso o projeto tenha atividades, verifica se a data de inicio prevista informada para o projeto eh anterior
      -- ou igual a menor data de inicio previsto dentre as suas atividades e se a data de termino prevista informada
      -- para o projeto eh posterior ou igual a maior data de termino previsto dentre as suas atividade
      -- -----------------------------------------------------------------------------------------------------------------
      if temConteudo(cgi.proj_cod) and temConteudo(cgi.dt_ini_prevista) and temConteudo(cgi.dt_fim_prevista) then 
      	 cgi.dt_ini_prevista = formataData(cgi.dt_ini_prevista)
         cgi.dt_fim_prevista = formataData(cgi.dt_fim_prevista)
         dataini = strsub(cgi.dt_ini_prevista,7,10)..strsub(cgi.dt_ini_prevista,4,5)..strsub(cgi.dt_ini_prevista,1,2)
         datafim = strsub(cgi.dt_fim_prevista,7,10)..strsub(cgi.dt_fim_prevista,4,5)..strsub(cgi.dt_fim_prevista,1,2)  	  
         
         local result
         DBExec("SELECT MIN(ativ_dt_ini_prev) AS dt_min FROM atividade WHERE proj_cod = "..cgi.proj_cod)
         result = DBRow()
         
         if result and result.dt_min and tonumber(dataini) > tonumber(result.dt_min) then
            dtIniPrev =  strsub(result.dt_min,7,8)..'/'..strsub(result.dt_min,5,6)..'/'..strsub(result.dt_min,1,4)           
            cgi.msg_dt_ini_prevista = 'A data de incio prevista para o projeto no est de acordo com a(s) data(s) de '..
                                      'incio prevista(s) informada(s) para a(s) atividade(s). A data informada deve ser '.. 
                                      'inferior ou igual a '..dtIniPrev..', que corresponde a menor data de incio previsto '..
                                      'de uma atividade do projeto.'  
            if not (cgi.msg) then
               cgi.msg = 'O formulrio foi preenchido incorretamente. Siga as instrues junto aos campos que devem ser revistos.'
            end                                       
         end 
         
         DBExec("SELECT MAX(ativ_dt_fim_prev) AS dt_max FROM atividade WHERE proj_cod = "..cgi.proj_cod)
         result = DBRow()
         if result and result.dt_max and tonumber(datafim) < tonumber(result.dt_max)then
            dtFimPrev =  strsub(result.dt_max,7,8)..'/'..strsub(result.dt_max,5,6)..'/'..strsub(result.dt_max,1,4)
            cgi.msg_dt_fim_prevista = 'A data de fim prevista para o projeto no est de acordo com a(s) data(s) de '..
                   'fim prevista(s) informada(s) para a(s) atividade(s). A data informada deve ser superior ou igual '..
                   'a '..dtFimPrev..', que corresponde a maior data de trmino previsto de uma atividade do projeto.'
            if not (cgi.msg) then
               cgi.msg = 'O formulrio foi preenchido incorretamente. Siga as instrues junto aos campos que devem ser revistos.'
            end                                                          
         end    
      end
      -- -----------------------------------------------------------------------------------------------------------------
   end
   
   -- ----------------------------------------------------------------------  
   -- HOUVE ERRO NO PREENCHIMENTO DO FORMULRIO DE CADASTRO
   -- ----------------------------------------------------------------------  
   if cgi.msg then     
      
      if ESTADO_BD == 'aberto' then
         FechaBD()
      end                   
          
      cgilua.redirect('../cadastra.html?'..cgilua.encodecgi())   -- VOLTA PARA A PGINA html--

   elseif temConteudo(cgi.ot_oper) and temConteudo(cgi.ot_especifica) then
      cgi.msg ='Escolha OT Operacional ou OT Especfica. Essa escolha  exclusiva.'      
      
      if ESTADO_BD == 'aberto' then
         FechaBD()
      end             
      
      cgilua.redirect('../cadastra.html?'..cgilua.encodecgi())    -- VOLTA PARA A PGINA html--
         
   else -- NAO HOUVE ERRO NA VALIDACAO DO FORMULARIO

      cgi.dt_ini_prevista = formataData(cgi.dt_ini_prevista)
      cgi.dt_fim_prevista = formataData(cgi.dt_fim_prevista)
      dataini = strsub(cgi.dt_ini_prevista,7,10)..strsub(cgi.dt_ini_prevista,4,5)..strsub(cgi.dt_ini_prevista,1,2)
      datafim = strsub(cgi.dt_fim_prevista,7,10)..strsub(cgi.dt_fim_prevista,4,5)..strsub(cgi.dt_fim_prevista,1,2)
      anoIni  = tonumber( strsub(dataini,1,4) ) 
      anoFim  = tonumber( strsub(datafim,1,4) )
      datacad = DataAtual()
 
      if temConteudo(cgi.proj_cod) and not strfind(cgi.proj_cod,",") and cgi.botao ~= 'Grava Novo Projeto' then
         
         -- -----------------------
         -- inicio UPDATE (EDIO)
         -- -----------------------
         
         orgao = ConsultaBD('orgao','*'," org_cod = '"..cgi.orgao.."' ",nil)      

         if cgi.unidade and cgi.unidade ~= '' and cgi.unidade ~= ' ' then
            unidade = ConsultaBD('unidade','*'," unid_cod = '"..cgi.unidade.."' ",nil)      
            resp = unidade.pes_cod_resp
            cgi.resp = resp
            cgi.unid_cod = cgi.unidade
         else
            resp = orgao.pes_cod_resp
            cgi.resp = resp
         end

         if unidade and unidade ~= '' and unidade ~= ' ' then

            campos =       " org_cod = "..cgi.orgao..", "..
                           " proj_nome = '"..cgi.projeto.."', "..
                           " proj_prior = "..cgi.prioridade..", "..
                           " proj_coord = '"..cgi.coordenador.."', "..
                           " proj_colab = '"..getuser().."', "..    
                           " proj_freq = "..cgi.freq..", "..
                           " proj_dt_ini_prev = '"..dataini.."', "..
                           " proj_dt_fim_prev = '"..datafim.."', "..
                           " unid_cod = "..cgi.unidade..", "..
                           " pes_cod = "..resp..", "..                                                  
                           " stat_cod = 1, "..
                           " proj_pae = 0, "..
                           " proj_paeid = "..cgi.plano_emerg_duto..", "..
                           " proj_meta = "..cgi.metas..", "..
                           " proj_num_ote = '"..cgi.ot_especifica.."'"



            if cgi.gerencia ~= "NULL" and temConteudo(cgi.gerencia) then
               campos = campos..", ger_cod = "..tonumber(cgi.gerencia)
            elseif cgi.gerencia == "" or cgi.gerencia == "NULL" then
               campos = campos..", ger_cod = NULL"
            end                       


            if cgi.setor ~= "NULL" and temConteudo(cgi.setor) then
               campos = campos..", set_cod = "..tonumber(cgi.setor)
            elseif cgi.setor == "" or cgi.setor == "NULL" then
               campos = campos..", set_cod = NULL"
            end       


            if cgi.descricao then
               campos = campos..", proj_descr = '"..forca_plic(cgi.descricao).."'"
            end       


            if temConteudo(cgi.plano_rec) then
               campos = campos ..", proj_plano_rec = "..cgi.plano_rec
            end
            if temConteudo(cgi.class_acomp) then
               campos = campos ..", proj_class = "..cgi.class_acomp
            end
            if temConteudo(cgi.cod_usuario) then
               campos = campos ..", proj_cod_usuario = '"..cgi.cod_usuario.."'"
            end                                                     
            if temConteudo(cgi.ot_especifica) then               
               -- ---------------------------------------------------------------------------------------------------
               -- A existencia da OT ESPECIFICA implica na nao existencia da OT OPERACIONAL, logo, eh apagada na ta-
               -- bela PROJ_OT (informacoes referentes a OT OPERACIONAL) qq entrada referente ao PROJETO em questao
               -- Esse registro vai ser refeito pela execucao mais adiante da funcao AtualizaOtOperacional           
               -- ---------------------------------------------------------------------------------------------------
               cons = ConsultaBD("proj_ot","*"," proj_cod = "..cgi.proj_cod.." ", nil )
               if cons and temConteudo(cons.proj_cod) then
                  ApagaBD("proj_ot", " proj_cod = "..cons.proj_cod.." ")
                  cgi.ot_oper = ''
               end
            end

            
            if cgi.estado_un then
               campos = campos..", proj_est_un = '"..cgi.estado_un.."'"
            end
            
            if cgi.funcao_un then
               campos = campos..", proj_funcao_un = '"..cgi.funcao_un.."'"
            end

            if cgi.tipo_un then
               campos = campos..", proj_tipo_un = '"..cgi.tipo_un.."'"
            end

            AlteraBD("projeto", campos, " proj_cod = "..cgi.proj_cod.." ")  

                                                                          
         else
            
            campos = " org_cod = "..cgi.orgao..", "..
                     " proj_nome = '"..cgi.projeto.."', "..
                     " proj_prior = "..cgi.prioridade..", "..
                     " proj_coord = '"..cgi.coordenador.."', "..
                     " proj_colab = '"..getuser().."', "..
                     " proj_freq = "..cgi.freq..", "..
                     " proj_dt_ini_prev = '"..dataini.."', "..
                     " proj_dt_fim_prev = '"..datafim.."', "..
                     " pes_cod = "..resp..", "..                     
                     " stat_cod = 1, "..
                     " proj_pae = 0, "..
                     " proj_paeid = "..cgi.plano_emerg_duto..", "..
                     " proj_meta = "..cgi.metas..", "..
                     " proj_num_ote = '"..cgi.ot_especifica.."'"                                             



            if cgi.descricao then
               campos = campos..", proj_descr = '"..forca_plic(cgi.descricao).."'"
            end       

            if cgi.gerencia ~= "NULL" and temConteudo(cgi.gerencia) then
               campos = campos..", ger_cod = "..tonumber(cgi.gerencia)
            elseif cgi.gerencia == "" or cgi.gerencia == "NULL" then
               campos = campos..", ger_cod = NULL"
            end                       

            if cgi.unidade ~= "NULL" and temConteudo(cgi.unidade) then
               campos = campos..", unid_cod = "..tonumber(cgi.unidade)
            elseif cgi.unidade == "" or cgi.unidade == "NULL" then
               campos = campos..", unid_cod = NULL"
            end                       

            if cgi.setor ~= "NULL" and temConteudo(cgi.setor) then
               campos = campos..", set_cod = "..tonumber(cgi.setor)
            elseif cgi.setor == "" or cgi.setor == "NULL" then
               campos = campos..", set_cod = NULL"
            end       


            if temConteudo(cgi.plano_rec) then
               campos = campos ..", proj_plano_rec = "..cgi.plano_rec
            end
            if temConteudo(cgi.class_acomp) then
               campos = campos ..", proj_class = "..cgi.class_acomp
            end
            if temConteudo(cgi.cod_usuario) then
               campos = campos ..", proj_cod_usuario = '"..cgi.cod_usuario.."'"
            end
            if temConteudo(cgi.ot_especifica) then               
               -- ---------------------------------------------------------------------------------------------------
               -- A existencia da OT ESPECIFICA implica na nao existencia da OT OPERACIONAL, logo, eh apagada na ta-
               -- bela PROJ_OT (informacoes referentes a OT OPERACIONAL) qq entrada referente ao PROJETO em questao
               -- Esse registro vai ser refeito pela execucao mais adiante da funcao AtualizaOtOperacional           
               -- ---------------------------------------------------------------------------------------------------
               cons = ConsultaBD("proj_ot","*"," proj_cod = "..cgi.proj_cod.." ", nil )
               if cons and temConteudo(cons.proj_cod) then
                  ApagaBD("proj_ot", " proj_cod = "..cons.proj_cod.." ")
                  cgi.ot_oper = ''
               end
            end

            if cgi.estado_un then
               campos = campos..", proj_est_un = '"..cgi.estado_un.."'"
            end
            
            if cgi.funcao_un then
               campos = campos..", proj_funcao_un = '"..cgi.funcao_un.."'"
            end

            if cgi.tipo_un then
               campos = campos..", proj_tipo_un = '"..cgi.tipo_un.."'"
            end

            AlteraBD("projeto", campos, " proj_cod = "..cgi.proj_cod.." ")
                                           
         end
         
         -- -------------------------------------------------------------------------------------
         -- OT OPERACIONAL
         -- Atualiza os dados referentes a CLASSIFICACAO, SUB e OT OPERACIONAL na tabela PROJ_OT
         -- -------------------------------------------------------------------------------------
         cgi.ot_sub = AtualizaOtOperacional(cgi.proj_cod, cgi.classificacao, cgi.ot_class, cgi.subclassificacao, cgi.ot_subclass, cgi.ot_oper)            
          
         -- ----------------------------------------------------------------------------------------
         -- Esta funcao efetua as insercoes/atualizacoes na tabela CUSTO
         -- Recebe os valores referentes ao codigo do projeto e ao FLAG flag_valor_custo_dif
         -- ----------------------------------------------------------------------------------------
         AtualizaCustoPrevistoAnual(cgi.proj_cod, flag_valor_custo_dif)
 
         -- -----------------------------------------------------------------------------
         -- CUSTO PREVISTO COM DETALHAMENTO MENSAL 
         -- Remove os valores inconsistentes com as datas de inicio e termino previstos
         -- OBS: dataini e datafim estao no formato AAAAMMDD                  
         -- -----------------------------------------------------------------------------         
         if dataini and datafim then         	
            sql_delete = " DELETE FROM custo_previsto WHERE proj_cod = "..cgi.proj_cod.." AND ( dt_prevista < '"..
                           dataini.."' OR dt_prevista > '"..datafim.."' ) "                                       
            DBExec(sql_delete)
         end   
         
         -- -----------------------------------------------------------------------------
         -- CUSTO PREVISTO ANUAL
         -- Remove os valores inconsistentes com as datas de inicio e termino previstos
         -- OBS: anoIni e anoFim estao no formato AAAA                  
         -- -----------------------------------------------------------------------------                 
         if dataini and datafim then        
            AlteraBD("custo", "custo_prev_ano = 0 ",
                     "proj_cod = "..cgi.proj_cod.." AND (custo_ano < "..anoIni.." OR custo_ano > "..anoFim..") ")                                      
         end   
        
         -- Apos a edicao de um projeto atualiza o FLAG referente as datas previstas das atividades estarem
         -- no intervalo compreendido pelas datas previstas do projeto ( proj_dt_prev_ativ - PROJETO )
         -- Uma das condicoes para formalizar um projeto
         atualizaDatasPrevistas(cgi.proj_cod)
         
         -- FIM EDICAO                                                        
        
      else   --  if temConteudo(cgi.proj_cod) and not strfind(cgi.proj_cod,",") and cgi.botao ~= 'Grava Novo Projeto'
               
         -- -----------------------------
         -- NOVO PROJETO (CADASTRAMENTO)
         -- -----------------------------
          
         resp_cad = retornaPessoaCod(getuser())
         orgao = ConsultaBD('orgao','*'," org_cod = '"..cgi.orgao.."' ",nil)      
   
         if temConteudo(cgi.unidade) then
            unidade = ConsultaBD('unidade','*'," unid_cod = '"..cgi.unidade.."' ",nil)      
            resp = unidade.pes_cod_resp
            cgi.resp = resp
            cgi.unid_cod = cgi.unidade
         else
            resp = orgao.pes_cod_resp
            cgi.resp = resp
            cgi.unidade = nil
         end   
         -------------------------------------
         -- Verifica se o projeto jah existe 
         -------------------------------------
         if cgi.versao then
            versao = cgi.versao
         else
            versao = 0
         end
         ------------------------------------------------------------------------------
         -- toda inclusao de novo projeto considera o mesmo como sendo a ultima versao
         ------------------------------------------------------------------------------ 
         last_versao='s'
         
         --------------------------------------------------------------------------------
         -- Atribuindo o valor de PROJ_SIGLA (inicialmente PROJ_SIGLA recebe 0)
         -- Quando o projeto eh gravado, esse valor eh atualizado para o codigo do 
         -- projeto PAI, nesse caso, o codigo do proprio projeto
         ---------------------------------------------------------------------------------         
         proj_sigla = 0
      
         proj_unico = ConsultaBD("projeto","*"," org_cod = "..cgi.orgao.." AND  proj_nome = '"..cgi.projeto.."' ","proj_versao")
         if temConteudo(proj_unico.proj_nome) then
            cgi.msg_projeto = 'Nome de Projeto j Cadastrado.'                        
            
            if ESTADO_BD == 'aberto' then
               FechaBD()
            end                         
            
            cgilua.redirect('../cadastra.html?'..cgilua.encodecgi())    -- VOLTA PARA A PGINA html --
          
         elseif temConteudo(unidade) then

            if temConteudo(cgi.gerencia) then  
               -- ------------------------------------------------------------------------------------------
               -- APRESENTA CAMPO UNIDADE, GER_COD e SET_COD                
               -- Verifica se existem campos nulos antes de inserir no BD para evitar erros no ACESSABD.LUA                         
               -- ------------------------------------------------------------------------------------------
               campos  = "org_cod, proj_nome, proj_prior, proj_dt_ini_prev, proj_dt_fim_prev, proj_dt_incl, unid_cod, "..
                         "ger_cod, pes_cod, proj_versao, stat_cod, proj_pae, proj_paeid,last_versao, proj_sigla, "..
                         "proj_resp_cad, proj_coord, proj_colab, proj_freq, proj_meta, proj_num_ote"
               valores = cgi.orgao..", '"..cgi.projeto.."', "..cgi.prioridade..", '"..dataini.."','"..datafim.."','"..
                         datacad.."',"..cgi.unidade..","..cgi.gerencia..","..resp..", "..versao..", 1, 0,"..
                         cgi.plano_emerg_duto..",'"..last_versao.."', '"..proj_sigla.."',"..
                         resp_cad..", '"..cgi.coordenador.."', '"..getuser().."', "..cgi.freq..", "..cgi.metas..",'"..
                         cgi.ot_especifica.."' "
               if cgi.descricao and cgi.descricao ~= 'NULL' then
                  campos  = campos .. ", proj_descr"
                  valores = valores .. ", '" .. cgi.descricao .. "'"
               end
               if cgi.setor and cgi.setor ~= 'NULL' and cgi.setor ~= '' then 	  
                  campos  = campos .. ", set_cod"
                  valores = valores .. ", " .. cgi.setor
               end
               if cgi.cod_usuario and cgi.cod_usuario ~= '' then
                  campos = campos .. ", proj_cod_usuario"
                  valores = valores .. ", '" .. cgi.cod_usuario .. "'"
               end
               if cgi.plano_rec and cgi.plano_rec ~= '' then
                  campos = campos .. ", proj_plano_rec"
                  valores = valores .. ", " .. cgi.plano_rec
               end
               if cgi.class_acomp and cgi.class_acomp ~= '' then
                  campos = campos .. ", proj_class"
                  valores = valores .. ", " .. cgi.class_acomp
               end               


               if cgi.estado_un and cgi.estado_un ~= "" then
                  campos = campos..", proj_est_un"
                  valores = valores..", '"..cgi.estado_un.."'"
               end
            
               if cgi.funcao_un and cgi.funcao_un ~= "" then
                  campos = campos..", proj_funcao_un"
                  valores = valores..", '"..cgi.funcao_un.."'"
               end

               if cgi.tipo_un and cgi.tipo_un ~= "" then
                  campos = campos..", proj_tipo_un"
                  valores = valores..", '"..cgi.tipo_un.."'"
               end

               EscreveBD("projeto", campos, valores)        
         
            else
               -- ----------------------------------------------------------------------------------------
               -- APRESENTA CAMPO UNIDADE                           
               -- Verifica se existem campos nulos antes de inserir BD para evitar erros no ACESSABD.LUA
               -- ----------------------------------------------------------------------------------------
               campos  = "org_cod, proj_nome, proj_prior, proj_dt_ini_prev, proj_dt_fim_prev, proj_dt_incl, unid_cod, "..
                         "pes_cod, proj_versao, stat_cod, proj_pae, proj_paeid, last_versao, proj_sigla, "..
                         "proj_resp_cad, proj_coord, proj_colab, proj_freq, proj_meta, proj_num_ote"
               valores = cgi.orgao..", '"..cgi.projeto.."', "..cgi.prioridade..", '"..dataini.."','"..datafim.."','"..
                         datacad.."',"..cgi.unidade..","..resp..", "..versao..",1,0,"..
                         cgi.plano_emerg_duto..",'"..last_versao.."', '"..proj_sigla.."',"..resp_cad..", '"..
                         cgi.coordenador.."', '"..getuser().."', "..cgi.freq..", "..cgi.metas..",'"..cgi.ot_especifica.."' "
               if cgi.descricao and cgi.descricao ~= 'NULL' then
                  campos  = campos .. ", proj_descr"
                  valores = valores .. ", '" .. cgi.descricao .."'"
               end
               if cgi.cod_usuario and cgi.cod_usuario ~= '' then
                  campos = campos .. ", proj_cod_usuario"   
                  valores = valores .. ", '" .. cgi.cod_usuario .. "'"
               end
               if cgi.plano_rec and cgi.plano_rec ~= '' then
                  campos = campos .. ", proj_plano_rec"
                  valores = valores .. ", '" .. cgi.plano_rec .. "'"
               end
               if cgi.class_acomp and cgi.class_acomp ~= '' then
                  campos = campos .. ", proj_class"
                  valores = valores .. ", '" .. cgi.class_acomp .. "'"
               end               

               if cgi.estado_un and cgi.estado_un ~= "" then
                  campos = campos..", proj_est_un"
                  valores = valores..", '"..cgi.estado_un.."'"
               end
            
               if cgi.funcao_un and cgi.funcao_un ~= "" then
                  campos = campos..", proj_funcao_un"
                  valores = valores..", '"..cgi.funcao_un.."'"
               end

               if cgi.tipo_un and cgi.tipo_un ~= "" then
                  campos = campos..", proj_tipo_un"
                  valores = valores..", '"..cgi.tipo_un.."'"
               end

               EscreveBD("projeto", campos, valores)           
            end
         else
            projeto = ConsultaBD("projeto","*"," org_cod = "..cgi.orgao.." AND proj_nome = '"..cgi.projeto.."' ","proj_versao")
            -- ------------------------------------------------------------------------------------------
            -- Verifica se existem campos nulos antes de inserir no BD para evitar erros no ACESSABD.LUA  
            -- ------------------------------------------------------------------------------------------
            campos  = "org_cod, proj_nome, proj_prior, proj_dt_ini_prev, proj_dt_fim_prev, proj_dt_incl, pes_cod, "..
                      "proj_versao, stat_cod, proj_pae, proj_paeid, last_versao, proj_sigla, proj_resp_cad, "..
                      "proj_coord, proj_colab, proj_freq, proj_meta, proj_num_ote"
            valores = cgi.orgao..", '"..cgi.projeto.."', "..cgi.prioridade..", '"..dataini.."','"..datafim.."','"..
                      datacad.."', "..resp..","..versao..",1,0,'"..cgi.plano_emerg_duto.."','"..
                      last_versao.."','"..proj_sigla.."',"..resp_cad..", '"..cgi.coordenador.."', '"..getuser()..
                      "', "..cgi.freq..", "..cgi.metas..",'"..cgi.ot_especifica.."' " 
            if cgi.descricao and cgi.descricao ~= 'NULL' then
               campos  = campos .. ", proj_descr"
               valores = valores .. ", '" .. cgi.descricao .."'"
            end                    
            if cgi.cod_usuario and cgi.cod_usuario ~= '' then
               campos = campos .. ", proj_cod_usuario"
               valores = valores .. ", '" .. cgi.cod_usuario .. "'"
            end
            if cgi.plano_rec and cgi.plano_rec ~= '' then
               campos = campos .. ", proj_plano_rec"
               valores = valores .. ", " .. cgi.plano_rec
            end
            if cgi.class_acomp and cgi.class_acomp ~= '' then
               campos = campos .. ", proj_class"
               valores = valores .. ", " .. cgi.class_acomp
            end            


            if cgi.estado_un and cgi.estado_un ~= "" then
               campos = campos..", proj_est_un"
               valores = valores..", '"..cgi.estado_un.."'"
            end
            
            if cgi.funcao_un and cgi.funcao_un ~= "" then
               campos = campos..", proj_funcao_un"
               valores = valores..", '"..cgi.funcao_un.."'"
            end

            if cgi.tipo_un and cgi.tipo_un ~= "" then
               campos = campos..", proj_tipo_un"
               valores = valores..", '"..cgi.tipo_un.."'"
            end
            EscreveBD("projeto", campos, valores)            
        
         end      
   
         projeto = ConsultaBD("projeto","*"," org_cod = "..cgi.orgao.." AND proj_nome = '"..cgi.projeto.."' AND proj_versao = "..versao.." ","proj_versao")
         cod_proj = projeto.proj_cod
         cgi.proj_cod = projeto.proj_cod

         -- -------------------------------------------------------------------------------------
         -- OT OPERACIONAL
         -- Atualiza os dados referentes a CLASSIFICACAO, SUB e OT OPERACIONAL na tabela PROJ_OT
         -- -------------------------------------------------------------------------------------
         cgi.ot_sub = AtualizaOtOperacional(cod_proj, cgi.classificacao, cgi.ot_class, cgi.subclassificacao, cgi.ot_subclass, cgi.ot_oper)                         
         
         -- ----------------------------------------------------------------------------------------
         -- Esta funcao efetua as insercoes/atualizacoes na tabela CUSTO
         -- Recebe os valores referentes ao codigo do projeto e ao FLAG flag_valor_custo_dif
         -- ----------------------------------------------------------------------------------------
         AtualizaCustoPrevistoAnual(cod_proj, flag_valor_custo_dif)
          
      end

      -- -----------------------------------------------------------------------------
      -- INSERO NA TABELA TRANSPETRO 
      -- Faz primeiro um DELETE para que se alguma entrada para o projeto jah existir 
      -- na tabela TRANSPETRO, essa entrada serah removida, evitando duplicacoes
      -- -----------------------------------------------------------------------------          
      delete = "DELETE FROM transpetro WHERE proj_cod = "..cgi.proj_cod
      DBExec(delete)
      insert = "INSERT INTO transpetro (proj_cod,trans_tipo,trans_eng,trans_cbi,trans_cac,trans_cad,trans_ui,trans_prog,trans_proj) "..
               "VALUES ("..cgi.proj_cod..",'"..troca_nil_por_vazio(cgi.tipo).."','"..troca_nil_por_vazio(cgi.engenharia).."','"..
               troca_nil_por_vazio(cgi.cbi).."','"..troca_nil_por_vazio(cgi.cac).."','"..troca_nil_por_vazio(cgi.cad).."','"..
               troca_nil_por_vazio(cgi.ui).."','"..troca_nil_por_vazio(cgi.programa).."','"..troca_nil_por_vazio(cgi.proj).."')"
      DBExec(insert)     
  
      if cgi.botao == 'Grava' or cgi.botao == 'Grava Novo Projeto' then  
         cgi.form_rev = nil --Se este flag for igual a 1 o botao de "formalizar" nao aparece.
         prox_pag = 'cadastra'
      elseif cgi.botao == 'Detalhar_Custo' then
         prox_pag = 'custo_previsto'   
      elseif cgi.botao == 'Atividades' then
         prox_pag = 'atividades'        
      elseif cgi.botao == 'Obs' then
         prox_pag = 'incl_obs'
      elseif cgi.botao == 'Dutos' then
         prox_pag = 'dutos'
      elseif cgi.botao == 'Formaliza' then
         prox_pag = 'formaliza'         
      end
      
      -- -------------------------------------------------------------------
      -- DELETA INFORMAES SOBRE DUTOS SE O USURIO TIVER SELECIONADO NO
      -- -------------------------------------------------------------------      
      if tonumber(cgi.plano_emerg_duto) == 0 then
         DBExec( "DELETE FROM dutos WHERE proj_cod = "..cgi.proj_cod )     
      end
          
      deleta_colab = "DELETE FROM colaboradores WHERE proj_cod = "..cgi.proj_cod
      DBExec(deleta_colab)
      proj_cod_colab = tonumber(cgi.proj_cod)

      tab_colab = {}
      tab_colab = Separa_Variaveis(cgi.recebe_colaboradores)  
      
      recebe_colaboradores = cgi.recebe_colaboradores       

      i = 1
      while (tab_colab[i]) do
      	 -- ---------------------------------------------------------------------------------------------
      	 -- Soh insere na tabela de colaboradores se o valor nao corresponder ao coordenador do projeto
      	 -- ---------------------------------------------------------------------------------------------
      	 if ( tonumber(tab_colab[i]) ~= tonumber(retornaPessoaCod(cgi.coordenador)) ) then      	              	  	  	 	
            insert_colab = "INSERT INTO colaboradores (proj_cod, colaborador) VALUES ("..proj_cod_colab..", "..
                           "'"..tab_colab[i].."')"
            DBExec(insert_colab)
         else 
            -- ---------------------------------------------------------------------
            -- Remove o colaborador no caso colab1,COLAB,colabn
            -- ---------------------------------------------------------------------          
            recebe_colaboradores = gsub(recebe_colaboradores, tab_colab[i]..",", "")    
            -- ---------------------------------------------------------------------
            -- Remove o colaborador no caso colab1,colabn,COLAB
            -- ---------------------------------------------------------------------                      
            recebe_colaboradores = gsub(recebe_colaboradores, ","..tab_colab[i], "")         
         end                
         i = i + 1
      end
      
      cgi.recebe_colaboradores = recebe_colaboradores              
      
      -- Atualiza os valores de custo ESPERADO e REALIZADO na tabela de projeto
      atualizaFinanceiro(cgi.proj_cod)
      
      -- -----------------------------------------------------------------------------
      -- Se for CADASTRAMENTO, atualiza o valor de PROJ_SIGLA na tabela PROJETO
      -- PROJ_CODIGO: valor original de CGI.PROJ_COD
      -- CGI.PROJ_COD: caso nao tenha sido passado como parametro, recebe PROJ_COD
      -- -----------------------------------------------------------------------------
      if not( temConteudo( proj_codigo ) ) then         
         if temConteudo( cgi.proj_cod ) then
            DBExec("UPDATE projeto SET proj_sigla = '"..cgi.proj_cod.."' WHERE proj_cod = "..cgi.proj_cod)
         end  
      end     
      
      if ESTADO_BD == 'aberto' then
         FechaBD()
      end                   
       
      if prox_pag == 'custo_previsto' then
      	 -- -------------------------------------------------------------------------------------------
      	 -- Evita passagem desnecessaria de CGIs para a proxima pagina
         -- -------------------------------------------------------------------------------------------
      	 cgilua.redirect("../"..prox_pag..".html?"..cgilua.encodetable({tipo_servico = cgi.tipo_servico,              
      	                 projeto = cgi.projeto, proj_cod = cgi.proj_cod, dt_ini_prevista = cgi.dt_ini_prevista,
      	                 dt_fim_prevista = cgi.dt_fim_prevista, custo_previsto_2000 = cgi.custo_previsto_2000, 
      	                 custo_previsto_2001 = cgi.custo_previsto_2001, custo_previsto_2002 = cgi.custo_previsto_2002, 
      	                 custo_previsto_2003 = cgi.custo_previsto_2003, custo_total = cgi.custo_total,
                         estado_un = cgi.estado_un, tipo_un = cgi.tipo_un, funcao_un = cgi.funcao_un}))
      elseif prox_pag == 'formaliza' then
         cgilua.redirect(prox_pag..".lua?tipo_servico=edita&acao=Formaliza&cod="..cgi.proj_cod)
      elseif prox_pag == 'incl_obs' then
         cgilua.redirect("../"..prox_pag..".html?tipo_servico=edita&acao=Obs&proj_cod="..cgi.proj_cod.."&proj_nome="..cgi.projeto)         
      else
         cgilua.redirect("../"..prox_pag..".html?"..cgilua.encodecgi())
      end   
                        
   end    -- if/else cgi.msg -- 

else -- do CGI.BOTAO
   
   if ESTADO_BD == 'aberto' then
      FechaBD()
   end      
   
   cgilua.redirect('../cadastra.html?'..cgilua.encodecgi())  -- VOLTA PARA A PGINA html
end
