-- -------------------------------------------------------------------------------------------
-- GLOBAL.LUA
-- -------------------------------------------------------------------------------------------

-- -------------------------------------------------------------- --
-- --Verifica se a pagina corrente tem acesso ao diretorio "lua", --
-- --se no tiver inclui "lua/" para que o acesso seja valido.    --
-- -------------------------------------------------------------- --
   local caminho = cgilua.script_vdir
   if strfind(caminho,'lua') then
      path = ""
   else
      path = "lua/"
   end

-- -----------------------------
-- -----------------------------
-- FORMATACAO DE VALORES REAIS
-- -----------------------------
-- -----------------------------

-- -------------------------------------------------------------------------------------------
-- Esta funcao coloca em tabelas de custos pontos de 3 em 3 casas decimais e aceita centavos.
-- Ex do formato: 1234569087,90  -> 1.234.569.087,90
-- -------------------------------------------------------------------------------------------
   function ponto_centavo (valor)
      local v,int,dec,ini,vv,fim,i,resto, menos      
      if ( temConteudo(valor) )then
         -- ----------------------------------------------------------------
         -- Tratamento para valores negativos que passaram a ser permitidos  
         -- como estorno de valores REALIZADOS
         -- ----------------------------------------------------------------
         menos = ''
         if strfind(valor,"-") then
            valor = gsub(valor, '-', '')
            menos = '- '
         end         
         -- ----------------------------------------------------------------
         v     = strfind (valor,",")
         int   = strsub (valor ,1, (v-1))
         dec   = strsub (valor, v , strlen(valor))
         qtd   = strlen (int)
         resto = mod(qtd,3)
         qtd   = floor(qtd/3)           
         if resto ~= 0 then
            vv = strsub(int,1,resto)          
         else
            vv = ''
         end           
         i = 1
         ini = resto + 1
         fim = v - 1 
         while i <= qtd do
           if temConteudo(vv) then
         	   vv = vv..'.'.. strsub(int,ini, (ini + 2))      	   
         	else
         	   vv = strsub(int,ini, (ini + 2)) 
         	end   
         	ini = ini + 3
         	i = i + 1
         end	      
         fim = menos..vv..dec      
         return fim
      else
         return(valor)
      end      
   end

   function formata_valor(valor)
      if temConteudo(valor) then
         valor = format("%.2f",valor)
         valor = gsub( valor, "%.", "," )
      end
      return valor
   end

   function formata_valor2(valor)
      if strfind(valor,',') then
         valor = gsub( valor,',00','' )
      elseif strfind(valor,'.') then
         valor = tonumber(valor)
      end
      return valor
   end

-- -----------------------------------------------------------
-- Utilizada nos calculos da realizacao fisica das atividades
-- Essa funcao retorna somente a parte inteira de uma string
-- created by: sandro
-- -----------------------------------------------------------
   function formata_percentual(valor)
      -- %d inteiro com sinal
      if temConteudo(valor) then
         valor = format("%d", valor )
      end
      return valor
   end

-- -----------------------------------------------------------
-- Utilizada para arredondamento de valores decimais
-- -----------------------------------------------------------
   function arredonda(valor)  
      if temConteudo(valor) then  
         valor = gsub(valor,',', '.')
         valor = format('%.1f',valor)
         ini = strfind(valor,'%.')
         if ini then 
            decimal = tonumber(strsub(valor, ini+1, ini+1))
            if decimal >= 5 then
               valor = ceil(valor)
            elseif decimal < 5 then
               valor = floor(valor)     
            end
         end
      else
         valor = 0         
      end
      return valor
   end


-- ---------------------
-- ---------------------
-- TRATAMENTO DE DATAS
-- ---------------------
-- ---------------------

-- -------------------------------------------------------------------------------------------------------------
-- Em algumas situacoes, qdo sao retornadas por CGI ou do BD, as datas perdem o '0' que precede os dias e meses
-- ( EX:  1/ 1/2002, 2002 1 1 ) e isso causa problemas quando do tratamento das datas, quer para exibicao, quer  
-- para insercao no BD. Essa funcao substitui os espacos em branco em uma data por 0
-- EX: 11/ 1/2002  -> 11/01/2002
-- -------------------------------------------------------------------------------------------------------------
   function formataData(data)
      if temConteudo(data) then
         data = gsub(data,' ','0')
         data = strsub(data,1,10)
      end
      return data
   end

-- -------------------------------------------
-- Formata uma data dd/mm/aaaa para aaaammdd
-- EX: 11/01/2002  -> 20020111
-- -------------------------------------------
   function inclui_data(data) 
      if temConteudo(data) then
         data = gsub(data,' ','0')
         data = strsub(data,7,10)..strsub(data,4,5)..strsub(data,1,2)
      end
      return data
   end

-- -------------------------------------------
-- Formata uma data aaaammdd para dd/mm/aaaa
-- EX: 20020111 -> 11/01/2002
-- -------------------------------------------
   function formataData2(data)
      if temConteudo(data) then
         data = gsub(data,' ','0')
         data = strsub(data,7,8)..'/'..strsub(data,5,6)..'/'..strsub(data,1,4)
      end
      return data			
   end

-- -------------------------------------------------------------------
-- Parametros: fmes = mes no formato m ou mm
--             ano  = ano no formato aaaa
-- Retorna:    descritivo do mes com 3 letras (Jan, Fev,...) mais /aa
-- -------------------------------------------------------------------
   function BuscaMesAno(fmes, ano)
      local tab_meses = {'Jan/','Fev/','Mar/','Abr/','Mai/','Jun/','Jul/','Ago/','Set/','Out/','Nov/','Dez/'}
      return tab_meses[tonumber(fmes)]..strsub(ano,3,4)
   end

-- -----------------------------------------------
-- Parametros: data1 = data no formato dd/mm/aaaa 
--             data2 = data no formato dd/mm/aaaa 
-- Retorna:    nmero de dias entre as duas datas
-- -----------------------------------------------
   function DiasEntre(data1, data2)
      local datadif = date2sec(data2,"%d/%m/%Y") - date2sec(data1,"%d/%m/%Y")
      if ( datadif < 0 ) then
         return ceil(datadif/(24*3600)-1)
      else 
         return floor(datadif/(24*3600)+1)
      end
   end

-- -----------------------------------------------------
-- Parametros: data1 = data no formato [dd/mm/aaaa]
--             data2 = data no formato [dd/mm/aaaa]
-- Retorna:    menor data entre as duas datas passadas
-- -----------------------------------------------------
   function MenorData(data1, data2)
      local data1temp = date2sec(data1,"%d/%m/%Y") 
      local data2temp = date2sec(data2,"%d/%m/%Y")
      if ( data1temp < data2temp ) then
         return data1
      else 
         return data2
      end
   end

-- --------------------------------------------------------------------------------------------
-- As datas de parametro para as funcoes pegaDia, pegaMes e pegaAno sao no formato dd/mm/aaaa
-- Os valores sao retornados no formato de string
-- --------------------------------------------------------------------------------------------
   function pegaDia(data)
      return strsub(gsub(data,' ','0'), 1, 2)
   end
   
   function pegaMes(data)
      return strsub(gsub(data,' ','0'), 4, 5)
   end
   
   function pegaAno(data)
      return strsub(gsub(data,' ','0'), 7, 10)
   end

   function DataAtual()
      local data = date('%c')
      return data
   end


-- --------------------------------------------
-- --------------------------------------------
-- INFORMACOES REFERENTES AO USUARIO CORRENTE
-- --------------------------------------------
-- --------------------------------------------

-- ------------------------------------------
-- Captura o "user name" do usuario corrente
-- ------------------------------------------
   function getuser()
      usuario = getenv('REMOTE_USER')
      if 	not usuario then
        	error('No foi especificado o login do usurio!')
      end
      local ini = strfind (usuario, '\\')
      if ini == nil then
          ini = strfind (usuario, '/')
      end    
      if ini then
      	  usuario = strsub(usuario, ini+1, strlen(usuario))
      end
      return usuario
   end

-- -----------------------------------------------------------------------------------------
-- Essa funo faz uso das funes de Permisso de Acesso, ou seja, utiliza o arquivo 
-- permissao_acesso.lua, mas quando a insero da permissao_acesso.lua  feita 
-- diretamente em GLOBAL.LUA, ocorrem problemas na execuo da funo nas pginas do 
-- diretrio raiz (PEGASO), extenso HTM. Deve ocorrer alguma perda de referncia 
-- quanto aos diretrios. Portanto,  necessrio fazer a incluso do permissao_acesso.lua
-- quando for necessrio utilizar a funo PERTENCE()
-- -----------------------------------------------------------------------------------------
-- assert(dofile('permissao_acesso.lua'))
-- -----------------------------------------------------------------------------------------
   function Pertence()
      flag_org, flag_unid, org_cod = validaAcesso()
      
      if (temConteudo(org_cod) and strfind (','..org_cod..',', ',5,') ) then
         return "SIM"
      elseif (org_cod == nil) then
      	 return "SIM"
      end
      return "NAO"
   end      


-- -----------------------
-- -----------------------
-- TRATAMENTO DE STRINGS
-- -----------------------
-- -----------------------

-- -----------------------------------------------------------------------------------------------------
-- Funo para verificar se o campo for nil, se for, retorna vazio, para evitar erros de insero no BD
-- -----------------------------------------------------------------------------------------------------
   function troca_nil_por_vazio(campo)
      if campo == nil then
         campo = ''
      end          
      return campo
   end

-- --------------------------------------------------------------------------------------                                      
-- Verifica se o conteudo de uma variavel nao eh NULO (nil), BRANCO (' ') nem VAZIO ('')
-- Retorna 1 se a variavel/CGI tem conteudo
-- Retorna NIL caso contrario
-- USO: if temConteudo(conteudo) then
--         ...
--      end	   
-- OU:
--      if not( temConteudo(conteudo) ) then
--         ...
--      end   
-- -------------------------------------------------------------------------------------                                       
   function temConteudo(conteudo) 
      -- '%S' = Tudo exceto caracteres brancos (espacos, tabs e quebras de linha)
      if conteudo and strfind(tostring(conteudo),'%S') and conteudo ~= "NULL" then 
         return 1 
      else   
         return nil
      end   
   end

-- -----------------------------------------------------------
-- Esta funcao restorna a string str 
-- sem os espaos, tabulaes quebra de linha, etc
-- Ex: retira_espacos("   Maria Pereira da Silva   ")
-- retorna "MariaPereiradaSilva"
-- -----------------------------------------------------------
   function retira_espacos( str )
   		return gsub( str, "%s", "") 
   end

-- -----------------------------------------------------------
-- Esta funcao acrescenta valores a uma lista usando um 
-- separador informado pelo usuario, caso o separador nao
-- seja informado a funcao usa ', ' como tal
-- -----------------------------------------------------------
function acrescentaLista(lista, valor, separador) 
   if not( temConteudo(separador) ) then
      separador = ', '
   end	
   if temConteudo(lista) and temConteudo(valor) then
      lista = lista .. separador .. valor
   elseif temConteudo(valor) then
      lista = valor
   end
   return lista
end

-- --------------------------------------------------------------------------------------        
-- FUNO QUE RECEBE UM STRING COM PALAVRAS ENTRE VIRGULA E QUEBRA AS PALAVRAS COLOCANDO
-- EM UMA TABELA.
-- Ex: recebe ->  um,dois,tres
--     retorna -> tab[1] = um
--                tab[2] = dois
--                tab[3] = tres
-- --------------------------------------------------------------------------------------        
   function Separa_Variaveis(variaveis)
      local tabela = {}
      local i = 1
      local flag = 0
      while ((variaveis) and (flag ~= 1)) do
        if strfind(variaveis,',') then
           tabela[i] = strsub(variaveis,0,strfind(variaveis,',')-1)
        else
           tabela[i] = variaveis
        end 
        if strfind(variaveis,',') then
           variaveis = strsub(variaveis,strfind(variaveis,',')+1)
        else
           flag = 1
        end
        i = i + 1
      end
      return tabela
   end   

   function QuebraLinha(texto, quebra) 
      if temConteudo( texto ) then 
         local tamanho=strlen(texto)
         local i=1
         local j=1
    
         while i <= tamanho do   
            local caracter_corr = strsub (texto, i, i)    
            local caracter_pos_corr = strsub (texto, i+1, i+1)
       
            if caracter_corr == "\n" then
               j = 1  -- REINICIA A CONTAGEM, CASO TENHA UM ENTER
               write ('<BR>') 
            end                  
            if j == quebra and caracter_pos_corr ~= "\n" then
           
               if caracter_corr == " " then
                  write (caracter_corr,'<BR>')
               else
                  while i <= tamanho and caracter_corr ~= " " do
                     write (caracter_corr)
                     i = i + 1
                     caracter_corr = strsub (texto,i,i)
                  end
                  if caracter_corr == " " then
                     write ('<BR>')                   
                  end
               end
               j = 1
            else
               write(caracter_corr)
               j = j + 1
            end
            i = i + 1     
         end
      else
         write('')
      end
   end

-- -----------------------------------------------------------------------------------
-- Em alguns campos de descricao o usuario pode querer inserir plics (') em seu texto
-- Isso pode causar problemas com os plics das concatenacoes e writes em LUA. 
-- Essa funcao procurapor plics em uma cadeia de caracteres e os substitui por aspas
-- -----------------------------------------------------------------------------------
   function forca_plic( s )
     local s = gsub( s, "'", '\"')
     return s
   end
   
-- -----------------------------------------------------------------------------------
-- Substitui as aparicoes de virgulas ',' por ponto '.'
-- Eh muito utilizada na insercao de valores numericos na base de dados, jah que no
-- banco os valores numericos soh apresentam o ponto como elemento separador
-- -----------------------------------------------------------------------------------
   function troca_virgula_por_ponto(valor)
      if temConteudo(valor) then
      -- -----------------------------------------
      -- Substitui vrgula ( , ) por ponto ( . )
      -- -----------------------------------------
         valor = gsub( valor, ",", "." )  
      end  
      return valor 
   end

-- ---------
-- ---------
-- E-MAIL
-- ---------
-- ---------
    
-- -----------------------------------------------------------------
-- Funo para envio de E-MAIL  
-- _MAILSERVER_ eh uma constante e estah definida em CONSTANTES.LUA          
-- -----------------------------------------------------------------      
   function envia_mail(para, de, assunto, msg, copia)  
   -- -----------------------------------------------------------------------------------------
   -- Recupera o endereo do servidor de e-mail (_MAILSERVER_) e a extenso de e-mail (_MAIL_)
   -- -----------------------------------------------------------------------------------------   
      assert(dofile( path..'constantes.lua' ))
            
      local ok, errmsg
      if copia and copia ~= '' and copia ~= ' ' then
         ok, errmsg = mail{ mailserver=_MAILSERVER_, to=para, cc=copia, from=de, subject=assunto, message=msg }                  
      else   
         ok, errmsg = mail{ mailserver=_MAILSERVER_, to=para, from=de, subject=assunto, message=msg }                  
      end
 
      return ok,errmsg        
   end   


-- ----------------
-- ----------------
-- BANCO DE DADOS
-- ----------------
-- ----------------

-- --------------------------------------------
-- Entrada : codigo da chave do usuario(log)
-- Saida : codigo da pessoa na tabela pessoal
-- --------------------------------------------
   function retornaPessoaCod( cod )
      local result = {}

      if ( cod ) then
         result = ConsultaBD2( "pessoal", "pes_cod", "pes_chave = '" .. cod .. "'" , nil )
         if ( not result[1][1] ) then
            result[1][1] = ''
         end
         return result[1][1]
      else
         return 'Cdigo de Pessoa Invlido!'
      end
   end

-- -------------------------------------------------------------------------------------- 
-- Atualizacao diaria do campo PROJ_DESEMPENHO na tabela PROJETO dos projetos INICIADOS 
-- -------------------------------------------------------------------------------------- 
   function atualizaProjDesempenhoDia()
      local tab_projs = {}
      local msg, i
      -- Os projetos nao iniciados (em edio) e os cancelados nao sao considerados
      tab_projs = ConsultaBD2( "projeto", "proj_cod", "stat_cod in (2, 4) and last_versao = 's'", nil )      
   
      i = 1
      while tab_projs[i] do
         gravaProjDesempenho(tab_projs[i].proj_cod)   
         i = i + 1
      end
   
      if ( i - 1 ) == 1 then
         msg = (i-1)..' projeto teve o campo PROJ_DESEMPENHO atualizado!'
      elseif ( i - 1 ) > 1 then
         msg = (i-1)..' projetos tiveram o campo PROJ_DESEMPENHO atualizados!'  
      end
      return (msg)   
   end   

-- -------------------------------------------------------------------------------------- 
-- Atualizao do campo PROJ_DESEMPENHO na tabela PROJETO quando so gravadas alteraes
-- nas atividades do PROJETO (datas e percentuais)
-- -------------------------------------------------------------------------------------- 
   function atualizaProjDesempenho(proj_cod)
      assert( dofile( path..'desempenho.lua' ) )
      gravaProjDesempenho(proj_cod)
   end
   

-- --------------------------------------------------------------------------------------
-- FUNCAO AUXILIAR usada por atualizaProjDesempenho() e atualizaProjDesempenhoDia()
-- --------------------------------------------------------------------------------------
-- Atualizacao do campo PROJ_DESEMPENHO na tabela PROJETO quando so gravadas alteraes
-- nas atividades do PROJETO (datas e percentuais)
-- -------------------------------------------------------------------------------------- 
   function gravaProjDesempenho(proj_cod)
      local realizacao = Desempenho(proj_cod)
      local novos_valores = ''
      local desempenho = 0
      if (realizacao.planejado and realizacao.realizado) then
         novos_valores = 'proj_perc_fisico_planejado = '..format("%.2f",realizacao.planejado)..', '..
                         'proj_perc_fisico_realizado = '..format("%.2f",realizacao.realizado)..', '
         local diferenca = realizacao.realizado - realizacao.planejado
         if (diferenca < 0) then         -- abaixo do previsto
            desempenho = 1
         elseif (diferenca == 0) then    -- de acordo com o previsto
            desempenho = 2
         else -- (diferenca > 0) then    -- acima do previsto
            desempenho = 3
         end
      end
      novos_valores = novos_valores..'proj_desempenho = '..desempenho
      AlteraBD('projeto', novos_valores, 'proj_cod = '..proj_cod)
   end


-- -----------------------------------         
-- Atualizao da tabela de FOLLOW_UP
-- -----------------------------------       
   function atualizaFollowUp(proj_cod, verificacao, descricao)           
      local linha, repetido
      local user = getuser()
      local data = date("%Y%m%d")
      
      local fup_campos = "fup_proj_cod, fup_resp, fup_data, fup_verificacao, fup_descr"
      local fup_valores = proj_cod..", '"..user.."', '"..data.."',"..verificacao..", '"..descricao.."'"                 
      
      -- -----------------------------------------------------------------------------------
      -- Verifica se os dados a serem cadastrados em FOLLOW_UP jah constam na base de dados
      -- -----------------------------------------------------------------------------------
      if ESTADO_BD ~= 'aberto' then
         AbreBD()
      end
      local sql = "SELECT fup_cod FROM follow_up WHERE fup_proj_cod = "..proj_cod.." AND fup_resp = '"..user.."' AND "..
                  "fup_data = '"..data.."' AND fup_verificacao = "..verificacao.." AND fup_descr LIKE '%"..descricao.."%'"
      DBExec(sql)     
      repetido = DBRow()                     
     
      -- ---------------------------------------------------------------------------------
      -- Se os dados fornecidos jah constarem na base de dados nao grava novo registro na 
      -- tabela de FOLLOW_UP mas ZERA o contador de follow-up na tabela de PROJETO
      -- ---------------------------------------------------------------------------------
      if not repetido then 	
         EscreveBD("follow_up", fup_campos, fup_valores)
         
         -- -----------------------------------------------------------------------
         -- Consulta a tabela de FOLLOW_UP para saber o codigo do ultimo follow-up 
         -- para inseri-lo na tabela de PROJETO
         -- -----------------------------------------------------------------------
         local ultimo = "SELECT fup_cod FROM follow_up WHERE fup_proj_cod = "..proj_cod.." AND fup_resp = '"..user.."' "..
                        "AND fup_data = '"..data.."' AND fup_verificacao = "..verificacao.." AND "..
                        "fup_descr LIKE '%"..descricao.."%' "
         DBExec(ultimo)
         linha = DBRow()
         
         -- -----------------------------------------------------------------------------------
         -- Quando uma atualizacao na tabela de FOLLOW_UP eh efetuada, o contador de follow-up
         -- na tabela de PROJETO deve ser zerado e deve ser inserido o codigo do ultimo 
         -- follow-up na tabela de PROJETO
         -- -----------------------------------------------------------------------------------
         AlteraBD('projeto', 'proj_cont_fup = 0, last_fup_cod = '..linha.fup_cod..' ', 'proj_cod = '..proj_cod)
      
      end                                                                                                             
   end

-- ----------------------------------------------------------------------------------                                      
-- Verifica se uma dada entrada estah armazenada no BD
-- Recebe a TABELA a ser consultada, os CAMPOS a serem retornados e a CONDICAO a ser 
-- avaliada (usa a funo ConsultaBD2() da acessabd.lua)
-- Retorna 1 se jah existe uma entrada na base de dados
-- Retorna NIL caso contrario
-- ----------------------------------------------------------------------------------                                      
   function temRegistroBD(tabela, campos, condicao) 
      local repetido
      -- ---------------------------------------------
      -- ConsultaBD2(tabelas,colunas,condicoes,ordem)  
      -- ---------------------------------------------
      repetido = ConsultaBD2(tabela, campos, condicao, nil)

      if repetido[1] then           
         return 1 
      else    
         return nil
      end   
   end

   function DadosProjeto(codigo)
      assert(dofile(path.."acessabd.lua"))
      local dados = {}
      dados = ConsultaBD("projeto a, orgao b, proj_ot c","*","a.proj_cod = "..codigo.." and a.proj_cod = c.proj_cod and a.org_cod = b.org_cod",nil)
      return dados
   end

-- -----------------------------------------------------------------------------------
-- retorna o nome da unidade dado o seu codigo
-- -----------------------------------------------------------------------------------
   function pegaNomeUnidade(codigo)
      local plinha
      DBExec('select unid_nome from unidade where unid_cod = '..codigo)   
      plinha = DBRow()
      return(plinha.unid_nome)  
   end

-- -----------------------------------------------------------------------------------
-- retorna o nome da area de negocio dado o seu codigo
-- -----------------------------------------------------------------------------------
   function pegaNomeArea(codigo)
      local plinha
      DBExec('select org_nome from orgao where org_cod = '..codigo)  
      plinha = DBRow()
      return(plinha.org_nome)  
   end

-- -----------------------------------------------------------------------------------
-- retorna o email dado a chave
-- -----------------------------------------------------------------------------------
   function pegaEmail(pes_chave)
      local plinha
      if (not pes_chave) then
          return
      end

      DBExec("select pes_email from pessoal where pes_chave = '"..pes_chave.."'")   
      plinha = DBRow()

      if plinha then
         return plinha.pes_email
      else
         return nil
      end
   end


-- -----------------------
-- -----------------------
-- OPERACOES EM TABELAS
-- -----------------------
-- -----------------------

   function BuscaTab2(tabela, ano, campo)
      local i, j, tabela1
      i = 1
      j = 1
      tabela1 = {}
      while tabela[i] do
         if ano == tonumber(strsub(formataData(tabela[i][campo]),7,10)) then
            tabela1[j] = tabela[i]
            j = j + 1
         end
         i = i + 1
      end
      return tabela1 , j - 1
   end

-- ---------------------------------------------
-- ---------------------------------------------
-- FORMATACAO (EXIBICAO DA INF PARA O USUARIO
-- ---------------------------------------------
-- ---------------------------------------------

-- ----------------------
-- Alterna cor de linha
-- ----------------------
   function AlternaCorLinha(mcor)
      local cor
      if mcor == "#CCCCCC" then
         mcor="#ECECEC"
      else
         mcor="#CCCCCC"
      end
      cor = mcor
      return cor
   end

  ------------------------------------------------------
  --Verifica se uma variavel e igual a string "NULL". --
  --Se igual a "NULL" retorna um espaco em branco, se --
  --diferente de "NULL" retorna a variavel.           --
  ------------------------------------------------------
  function temNull(str)
     if str == "NULL" then
        return ""
     else
        return str
     end
  end