dofile( "lua/acessabd.lua" )

-- Constantes Globais
ctTransTipo={}
   ctTransTipo.Operacao = 1
   ctTransTipo[ctTransTipo.Operacao] = 'O'
   ctTransTipo.Investimento = 2
   ctTransTipo[ctTransTipo.Investimento] = 'I'
   ctTransTipo.Ambos = 3
   ctTransTipo[ctTransTipo.Ambos] = nil 

ctMes={  "Janeiro", 
         "Fevereiro", 
         "Maro", 
         "Abril", 
         "Maio", 
         "Junho", 
         "Julho", 
         "Agosto", 
         "Setembro", 
         "Outubro", 
         "Novembro",
         "Dezembro" }
          
-- -------------------------------------------------------------------------------------
-- Function : val
-- Description : converte em numero, retornando 0 se for Nil
-- Parameters : valor
-- Return : o numero
-- -------------------------------------------------------------------------------------
function val(valor)
   local nVal = 0
   
   nVal = tonumber(valor)
   
   if nVal then
      return nVal
   end 
      
   return 0
end 

-- -------------------------------------------------------------------------------------
-- Function : str
-- Description : converte em string, retornando '' se for Nil
-- Parameters : valor
-- Return : a string
-- -------------------------------------------------------------------------------------
function str(valor)
   
   valor = tostring(valor)
   
   if (valor=='nil') then
      return '0'
   end 
      
   return valor
end 


-- -------------------------------------------------------------------------------------
-- Function : sqlData
-- Description : converte 3 inteiros em uma data no formato sql
-- Parameters : dia 
--              mes
--              ano
-- Return : a data no formato sql: Cast('dia/mes/ano' as datatime)
-- -------------------------------------------------------------------------------------
function sqlData(dia, mes, ano)
   local sData = ''
   
   while ( mes  > 12 ) do
      mes = mes - 12
      ano = ano + 1
   end
   
   sData = "cast("..dia.."/"..mes.."/"..ano.." as datetime)" -- data no formao especificado
   
   return sData
   
end 

-- -------------------------------------------------------------------------------------
-- Function : getUnidades()
-- Description : le as unidades da transpetro
-- Parameters : 
-- Return : retorna uma tabela com o nome das unidades e o seu codigo
-- -------------------------------------------------------------------------------------
function getUnidades()
   -- tabela auxiliar
   local tbTemp = {}
   -- contador
   local i = 0
   
   -- obtem as unidades da transpetro
   tbTemp = ConsultaBD2("unidade u, orgao o", "u.unid_sigla, u.unid_cod", "u.org_cod = o.org_cod AND o.org_sigla = 'TRANSPETRO'", nil)
   
   
   if tbTemp[1] then -- se houverem unidades
      return tbTemp
   end 
   
   return nil   
end 

-- -------------------------------------------------------------------------------------
-- Function : getDadosAnoProj
-- Description : le os dados dos projetos daquela unidade, por ano
-- Parameters : cod_unid - codigo da unidade
--              trans_tipo - tipo de custo, pode ser Operacoes(1), Investimentos(2) ou Ambos(3)
--              ano_ref - ano de referencia da busca
-- Return : tabela com os dados de custos acumulados do projeto
--          previsto - os custos previstos para o ano
--          realizado - realizado no ano
-- -------------------------------------------------------------------------------------
function getDadosAnoProj( cod_unid, trans_tipo, ano_ref)
   -- Tabelas
   local tbTemp = {}
   local tbRet = {}
   -- dados para a query
   local tabelas = ''
   local tabelasDef =" ,transpetro t, projeto p, orgao o"
   local campos = ''
   local criterio = ''
   local criterioTrans = ''
   local criterioProj = " AND p.org_cod = o.org_cod AND o.org_sigla = 'TRANSPETRO' AND p.unid_cod = " .. cod_unid .. " AND p.proj_cod = t.proj_cod"
   
   -- verifico qual o tipo de transacao que sera verificada
   if ( ctTransTipo[trans_tipo] ) then
      criterioTrans = " AND t.trans_tipo = '" .. ctTransTipo[trans_tipo] .. "'"
   else
      criterioTrans = ' '
   end 

   -- seta dados para a query 
   -- Custo previsto no ano
   tabelas = "custo c"
   campos = "SUM(c.custo_prev_ano) as c_prev_ano"
   criterio = "c.proj_cod = p.proj_cod AND c.custo_ano = " .. ano_ref 
   
   tbTemp = ConsultaBD2(tabelas .. tabelasDef, campos, criterio .. criterioTrans .. criterioProj, nil)
   
   if ( tbTemp[1] ) then -- se existirem os custos anuais para o ano especificado
      tbRet.previsto = tbTemp[1].c_prev_ano
   end 
   
   -- seta dados para a query 
   -- Custo realizado no ano
   tabelas = "custo_realizado c"
   campos = "SUM(c.valor_real) as real_ano"
   criterio = "c.proj_cod = p.proj_cod AND c.dt_realizacao like '%" .. ano_ref .. "%' "
   
   tbTemp = ConsultaBD2(tabelas .. tabelasDef, campos, criterio .. criterioTrans .. criterioProj, nil)
   
   if ( tbTemp[1] ) then -- se existirem os custos anuais para o ano especificado
      tbRet.realizado = tbTemp[1].real_ano
   end
   
   return tbRet
   
end -- function

-- -------------------------------------------------------------------------------------
-- Function : getDadosMesProj
-- Description : le os dados dos projetos daquela unidade, por mes
-- Parameters : cod_unid - codigo da unidade
--              trans_tipo - tipo de custo, pode ser Operacoes(1), Investimentos(2) ou Ambos(3)
--              ano_ref - ano de referencia da busca (inteiro)
--              mes_ref - mes de referencia da busca (inteiro)
-- Return : tabela com os dados de custos acumulados do projeto
--          [ANO][MES].realizado - os custos realizados no mes
--          [ANO][MES].acumulado - custos acumulados ate o mes
-- -------------------------------------------------------------------------------------
function getDadosMesProj( cod_unid, trans_tipo, mes_ref, ano_ref)
   local tbTemp = {}
   local tbRet ={}
   -- dados para a query
   local tabelas = ''
   local tabelasDef =" ,transpetro t, projeto p, orgao o"
   local campos = ''
   local criterio = ''
   local criterioTrans = ''
   local criterioProj = " AND p.org_cod = o.org_cod AND o.org_sigla = 'TRANSPETRO' AND p.unid_cod = " .. cod_unid .. " AND p.proj_cod = t.proj_cod"
   -- contadores
   local i = 1
   local j 
   
   -- Inicializacao da tabela
   ano_ref = val(ano_ref)
   tbRet[ano_ref]={}
   
   -- verifico qual o tipo de transacao que sera verificada
   if ( ctTransTipo[trans_tipo] ) then
      criterioTrans = " AND t.trans_tipo = '" .. ctTransTipo[trans_tipo] .. "'"
   else
      criterioTrans = ""
   end 
  
   -- seta dados para a query 
   -- Custo realizado no mes   
   tabelas = "custo_realizado c"
   campos = "SUM(c.valor_real) as c_acum_mes"

   i=1
   while (i <= val(mes_ref) ) do
      criterio = "c.proj_cod = p.proj_cod AND DATEPART(year,c.dt_realizacao) = " .. ano_ref .. 
                                        " AND DATEPART(month,c.dt_realizacao) = " .. i      
      tbTemp = ConsultaBD2(tabelas .. tabelasDef, campos, criterio .. criterioTrans .. criterioProj, nil)
      tbRet[ano_ref][i] = {}
      tbRet[ano_ref][i].realizado = tbTemp[1].c_acum_mes
      j = 2
      while tbTemp[j] do -- armazena os custos mensais
         tbRet[ano_ref][i].realizado = tbRet[ano_ref][i].realizado + tbTemp[j].c_acum_mes
         j = j+1
      end
      
      i = i+1 -- proximo mes
   end
   
   i = 1
   while (i <= val(mes_ref) ) do
      
      tbRet[ano_ref][i].acumulado = val(tbRet[ano_ref][1].realizado)
      j = 2
      while j <= i do -- armazena os custos mensais
         tbRet[ano_ref][i].acumulado = val(tbRet[ano_ref][i].acumulado) + val(tbRet[ano_ref][j].realizado)
         j = j+1        
      end 
     
      i = i+1 -- proximo mes
   end 
   
   return tbRet
   
end


-- -------------------------------------------------------------------------------------
-- Function : CalculaPerc
-- Description : Calcula a percentagem de parte sobre o total
-- Parameters : parte - parte a qual se deseja saber a porcentagem
--              total - total sobre qual deseja-se calcular a porcentagem
-- Return : inteiro contendo o valor da porcentagem
-- -------------------------------------------------------------------------------------
function CalculaPerc (parte, total)
   total = val(total)
   if (total == 0) then
   	return nil
   end -- if
   return (val(format("%8.2f", ((val(parte)/total)*100) ) ) )

end 


-- -------------  FUNCOES QUE RETORNAM HTML --------------------------------------------
-- -------------------------------------------------------------------------------------
-- Function : htmHeaderAnual
-- Description : retorna o header da tabela de relatorio anual
-- Parameters : ano - ano do relatorio
--              mes - mes mes limite do relatorio
--              trans - tipo de transacao
-- Return : uma string contendo o HTML do header
-- -------------------------------------------------------------------------------------
function htmHeaderAnual( mes, ano, trans )
   -- htm
   local htmHeader = ''
   -- string
   local sTitle = '' 
   -- contadores
   local i = 0
   
   if (trans == ctTransTipo.Operacao ) then
      sTitle = "Opera&ccedil;oes"
   elseif (trans == ctTransTipo.Investimento ) then
      sTitle = "Investimentos"
   else
      sTitle = "Opera&ccedil;oes<br>Investimentos"
   end 
   
   htmHeader = ""..
   [[<tr>
      <td>]] .. sTitle .. [[ </td>
      <td> Previsao p/ ano ]] .. ano .. [[ </td>
   ]]   
   i = 1
   while (i <= val(mes)) do
      
      htmHeader = htmHeader .. [[<!-- repete para cada mes -->
      <td><table border = 0 width = 219>
            <tr width = 219>]].. ctMes[i] .." de " .. ano .. [[ </tr>
            <tr width = 219>
               <td width = 75>Realiz. no Mes</td>
               <td width = 75>Realiz. ate o mes </td>
               <td width = 75>(%) ate o mes </td>
            </tr>
         </table>
      </td>
      <!-- FIM  repete para cada mes --> ]]
      i = i+1
   end -- while
   
   htmHeader = htmHeader .. "\n</tr>"
   
   return htmHeader

end -- function


-- -------------------------------------------------------------------------------------
-- Function : htmDataAnual
-- Description :monta uma linha da tabela anual
-- Parameters : unid_sigla: sigla da unidade
--              mes: mes de referencia
--              ano: ano de referencia
--              tbMes: tabela de custos mensais
--              tbAno: tabela de custos anuais
--              tbTotal: tabela de totais *
-- Return : html com uma linha de dados da tabela
--
-- Obs: Sintaxe da tabela de totais: - tbTotal.ano = total acum previsto para o ano
--                                     tbTotal[mes].realizado = total realizado
--                                     tbTotal[mes].acumulado = total acumulado
-- -------------------------------------------------------------------------------------
function htmDataAnual( unid_sigla, mes, ano, tbMes, tbAno, tbTotal )
   -- htm
   local htmData
   -- contadores
   local i =0
   
   if not tbTotal then
    	tbTotal = {}
   	  tbTotal.ano = 0
   end 
   
   tbTotal.ano = val(tbTotal.ano) + val(tbAno.previsto) -- acumula os totais anuais
   
   htmData = ""..
   [[<tr>
      <td>]] .. unid_sigla .. [[ </td>
      <td>]] .. format("%8.2f",val(tbAno.previsto)) .. [[ </td>
   ]]   
   i = 1
   while (i<=mes) do
      -- acumula os totais
      if not tbTotal[i] then
      	tbTotal[i]={}
      	tbTotal[i].realizado = 0
      	tbTotal[i].acumulado = 0
     	end 
      tbTotal[i].realizado = val(tbTotal[i].realizado) + val(tbMes[ano][i].realizado)
      tbTotal[i].acumulado = val(tbTotal[i].acumulado) + val(tbMes[ano][i].acumulado)
      
      -- monta html
      htmData = htmData .. [[<!-- repete para cada mes -->
      <td><table border = 0 width = 219>
            <tr width = 219>
               <td width = 75>]] .. format("%8.2f",val(tbMes[ano][i].realizado)) .. [[</td>
               <td width = 75>]] .. format("%8.2f",val(tbMes[ano][i].acumulado)) .. [[</td>
               <td width = 75>]] .. str(CalculaPerc(tbMes[ano][i].acumulado, tbAno.previsto)) .. [[</td>
            </tr>
         </table>
      </td>
      <!-- FIM  repete para cada mes --> ]]
      i = i+1
   end -- while
   
   htmData = htmData .. "\n</tr>"   
   
   return htmData, tbTotal

end -- function


-- -------------------------------------------------------------------------------------
-- Function : htmTotalAnual
-- Description : Monta a linha de totais da tabela
-- Parameters : mes : mes de referecia
--              tbTotal : utaberla com os totais *
-- Return : html com a linha de totais
-- -------------------------------------------------------------------------------------
function htmTotalAnual( mes, tbTotal )
   -- htm
   local htmData
   -- contadores
   local i =0
   
   htmData = ""..
   [[<tr>
      <td>Total </td>
      <td>]] .. tbTotal.ano .. [[ </td>
   ]]   
   i = 1
   while (i<=mes) do
      -- monta html
      htmData = htmData .. [[
      <td><table border = 0 width = 219>
            <tr width = 219>
               <td width = 75>]] .. format("%8.2f",tbTotal[i].realizado) .. [[</td>
               <td width = 75>]] .. format("%8.2f",tbTotal[i].acumulado) .. [[</td>
               <td width = 75>]] .. str(CalculaPerc(tbTotal[i].acumulado, tbTotal.ano)) .. [[</td>
            </tr>
         </table>
      </td>
       ]]
      i = i+1
   end -- while
   
   htmData = htmData .. "\n</tr>"
   
   return htmData

end -- function

-- -------------------------------------------------------------------------------------
-- Function : htmRenderAnual
-- Description : renderiza uma tabela contendo o formulario anual
-- Parameters : mes : mes de referecia
--              ano : ano de referencia
--
-- Return : a tabela com o relatorio
-- -------------------------------------------------------------------------------------
function htmRenderAnual ( mes, ano)
   -- tabelas
   local tbAno = {}
   local tbMes = {}
   local tbUnid = {}
   local tbTotal = {} -- guarda os dados totais
   -- contadores
   local i = 0
   --html
   local htmTemp = ""
   local htmData = ""
   local htmHeader = ""
   local htmTotal = ""
   local htmRender = ""
   
   -- inicializa mes e ano
   mes = val (mes)
   ano = val (ano)
   
   tbUnid = getUnidades()
   --Transacao INVESTIMENTO
   tbTotal = {}
   htmHeader = htmHeaderAnual(mes, ano, ctTransTipo.Investimento)
   i = 1
   while tbUnid[i] do
      tbMes = getDadosMesProj(tbUnid[i].unid_cod, ctTransTipo.Investimento, mes, ano )
      tbAno = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Investimento, ano)
      htmTemp, tbTotal = htmDataAnual(tbUnid[i].unid_sigla,mes, ano, tbMes, tbAno, tbTotal)
      htmData = htmData .. htmTemp
      i = i+1
   end -- while
   htmTotal = htmTotalAnual(mes, tbTotal)
   htmRender = [[ <table border = 1>
                  ]] .. htmHeader .. htmData .. htmTotal .. [[
                 </table>]]
   
   htmRender = htmRender .. "<P>"
   
   --Transacao OPERACAO
   htmData = ""
   htmTemp = ""
   tbTotal = {}
   htmHeader = htmHeaderAnual(mes, ano, ctTransTipo.Operacao)
   i = 1
   while tbUnid[i] do
      tbMes = getDadosMesProj(tbUnid[i].unid_cod, ctTransTipo.Operacao, mes, ano )
      tbAno = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Operacao, ano)
      htmTemp, tbTotal = htmDataAnual(tbUnid[i].unid_sigla, mes, ano, tbMes, tbAno, tbTotal)
      htmData = htmData .. htmTemp
      i = i+1
   end -- while
   
   htmTotal = htmTotalAnual(mes, tbTotal)
   htmRender = htmRender .. [[ <table border= 1>
                  ]] .. htmHeader .. htmData .. htmTotal .. [[
                 </table>]]
                 
   htmRender = htmRender .. "<P>"   
   
   --Transacao OPERACAO
   tbTotal = {}
   htmTemp = ""
   htmData = ""
   htmHeader = htmHeaderAnual(mes, ano, ctTransTipo.Ambos)
   i = 1
   while tbUnid[i] do
      tbMes = getDadosMesProj(tbUnid[i].unid_cod, ctTransTipo.Ambos, mes, ano )
      tbAno = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Ambos, ano)
      htmTemp, tbTotal = htmDataAnual(tbUnid[i].unid_sigla,mes, ano, tbMes, tbAno, tbTotal)
      htmData = htmData .. htmTemp
      i = i+1
   end -- while
   htmTotal = htmTotalAnual(mes, tbTotal)
   htmRender = htmRender .. [[ <table border = 1>
                  ]] .. htmHeader .. htmData .. htmTotal .. [[
                 </table>]]
   
   htmRender = htmRender .. "<P>"

   return htmRender

end -- function

-- -------------------------------------------------------------------------------------
-- Function : htmHeaderPlurianual
-- Description : gera o header do relatorio plurianual
-- Parameters : NULL
-- Return : retorna uma string html com o header da tabela de relatorio plurianual
-- -------------------------------------------------------------------------------------
function htmHeaderPlurianual()
   -- htm
   local htmHeader = ""
   -- contadores
   local i = 2000
   -- ano corrente
   local ano = val(date("%Y"))
   -- Geral
   local sRealizado = ""

   
   htmHeader = [[
   <tr>
      <td></td>
   ]] 
   -- gera os cabecalhos dos anos
   while (i <= 2003 ) do
      if (i < ano) then
         sRealizado = "(Realizado)"
      else
         sRealizado = "(Previsto)"
      end -- if
      -- monta o cabecalho
      htmHeader = htmHeader .. [[<td>
         <table border = 0 width = 219>
            <tr align = center valign = center width = 219>]] .. i .. sRealizado .. [[ -  Mil R$</tr>
            <tr align = center valign = center width = 219>
               <td align = center valign = center width = 75>Invest.</td>
               <td align = center valign = center width = 75>Oper.</td>
               <td align = center valign = center width = 75>Total</td>
            <tr>
         </table>
      </td>
      ]]
      i = i+1
   end -- while
   htmHeader = htmHeader .. [[<td>
    <table border = 0 width = 219>
       <tr align = center valign = center width = 219><center>Total -  Mil R$</center> </tr>
       <tr align = center valign = center width = 219>
          <td align = center valign = center width = 75>Invest.</td>
          <td align = center valign = center width = 75>Oper.</td>
          <td align = center valign = center width = 75>Total</td>
       <tr>
    </table>
   </td>
   ]]
   
   htmHeader = htmHeader .. "</tr>"
   
   return htmHeader
   
end -- function

-- -------------------------------------------------------------------------------------
-- Function : htmDataPlurianual
-- Description : posta  os dados no relatorio plurianual
-- Parameters : unid_sigla - sigla da unidade
--              tbAnoInvest - tabela que contem os custos anuais de investimento *
--              tbAnoOper - tabela que contem os dados anuais de operacao *
--              tbTotal - tabela que contem os custos totais acumulados das unidades**
-- Return : string html contendo o html com os dados do formulario plurianual, tabela de totais
--
-- Obs.: * Essas tabelas tem o formato tab[ano].realizado e tab[ano].previsto
--       ** possui o formato tab[ano].invest, tab[ano].oper, tab[ano].total, o ano do total = 1
-- -------------------------------------------------------------------------------------
function htmDataPlurianual(unid_sigla, tbAnoInvest, tbAnoOper, tbTotal)
   -- htm
   local htmData = ""
   -- contadores
   local i = 2000
   -- ano corrente
   local ano = val(date("%Y"))
   -- Auxiliares
   local fInvest = 0 -- investimentos, operacao e  total por ano
   local fOper = 0
   local fTotal = 0
   
   local fAcInvest = 0 -- investimentos, operacao e  total, acumulados por ano
   local fAcOper = 0
   local fAcTotal = 0   
   
   htmData = [[
   <tr>
      <td>]] .. unid_sigla .. [[</td>
   ]] 
   
   -- gera os dados dos anos (2000 - 2003)
   i = 2000
   while (i <= 2003 ) do
      
      -- atribui os custos anuais
      if (i < ano) then 
         fInvest = val(tbAnoInvest[i].realizado)
         fOper = val(tbAnoOper[i].realizado)
      else
         fInvest = val(tbAnoInvest[i].previsto)
         fOper = val(tbAnoOper[i].previsto)
      end -- if
      
      fTotal = fInvest + fOper
      
      -- acumula os totais do orgao
      if not tbTotal[i] then
         tbTotal[i] = {}
         tbTotal[i].oper = 0
         tbTotal[i].invest = 0
         tbTotal[i].total = 0
      end -- if
      tbTotal[i].oper = tbTotal[i].oper + fOper
      tbTotal[i].invest = tbTotal[i].invest + fInvest
      tbTotal[i].total = tbTotal[i].total + fTotal
      
      -- acumula os totais gastos  
      fAcInvest = fAcInvest + fInvest
      fAcOper = fAcOper + fOper
      fAcTotal = fAcTotal + fTotal
            
      -- monta o cabecalho
      htmData = htmData .. [[<td>
         <table>
            <tr align = center valign = center width = 219>
               <td align = left valign = center width = 75>]] .. format("%8.2f",val(fInvest)) .. [[</td>
               <td align = left valign = center width = 75>]] .. format("%8.2f",val(fOper)) .. [[</td>
               <td align = left valign = center width = 75>]] .. format("%8.2f",val(fTotal)) .. [[</td>
            <tr>
         </table>
      </td>
      ]]
      i = i+1
   end -- while
   htmData = htmData .. [[<td>
         <table>
            <tr align = center valign = center width = 219>
               <td align = left valign = center width = 75>]] .. format("%8.2f",val(fAcInvest)) .. [[</td>
               <td align = left valign = center width = 75>]] .. format("%8.2f",val(fAcOper)) .. [[</td>
               <td align = left valign = center width = 75>]] .. format("%8.2f",val(fAcTotal)) .. [[</td>
            <tr>
         </table>
      </td>
      ]]
   -- acumula os totais do orgao
   if not tbTotal[1] then
         tbTotal[1] = {}
         tbTotal[1].oper = 0
         tbTotal[1].invest = 0
         tbTotal[1].total = 0
      end -- if
      tbTotal[1].oper = tbTotal[1].oper + fAcOper
      tbTotal[1].invest = tbTotal[1].invest + fAcInvest
      tbTotal[1].total = tbTotal[1].total + fAcTotal
   
   htmData = htmData .. "</tr>"
   
   return htmData, tbTotal

end -- function

-- -------------------------------------------------------------------------------------
-- Function : htmTotalPlurianual
-- Description : posta os totais do no relatorio plurianual
-- Parameters : tbTotal - tabela que contem os custos totais acumulados das unidades*
-- Return : string html contendo o html com os totais do relatorio plurianual
--
-- Obs.: * possui o formato tab[ano].invest, tab[ano].oper, tab[ano].total, o ano do total = 1
-- -------------------------------------------------------------------------------------
function htmTotalPlurianual(tbTotal)
   -- htm
   local htmData = ""
   -- contadores
   local i = 2000
   -- ano corrente
   local ano = date("%Y")
      
   
   htmData = [[
   <tr>
      <td> TOTAL </td>
   ]] 
      
   -- gera os dados totais dos anos (2000 - 2003)
   while (i <= 2003 ) do
      
           
      -- monta o cabecalho
      htmData = htmData .. [[<td>
         <table>
            <tr align = center valign = center width = 219>
               <td align = left valign = center width = 75>]] .. format("%8.2f",tbTotal[i].invest) .. [[</td>
               <td align = left valign = center width = 75>]] .. format("%8.2f",tbTotal[i].oper) .. [[</td>
               <td align = left valign = center width = 75>]] .. format("%8.2f",tbTotal[i].total) .. [[</td>
            <tr>
         </table>
      </td>
      ]]
      i = i+1
   end -- while
   htmData = htmData .. [[<td>
         <table>
            <tr align = center valign = center width = 219>
               <td align = left valign = center width = 75>]] .. format("%8.2f",tbTotal[1].invest) .. [[</td>
               <td align = left valign = center width = 75>]] .. format("%8.2f",tbTotal[1].oper) .. [[</td>
               <td align = left valign = center width = 75>]] .. format("%8.2f",tbTotal[1].total) .. [[</td>
            <tr>
         </table>
      </td>
   ]]
   
   htmData = htmData .. "</tr>"
   
   return htmData

end -- function



-- -------------------------------------------------------------------------------------
-- Function : htmRenderPlurianual
-- Description : Retorna um html com o relatorio plurianual
-- Parameters : null
-- Return : html com o relatorio plurianual em forma de string
-- -------------------------------------------------------------------------------------
function htmRenderPlurianual()
   -- html
   local htmRender = ""
   local htmData = ''
   -- contadores 
   local i = 0
   local ano = 2000
   --tabelas
   local tbAnoInvest = {}
   local tbAnoOper = {}
   local tbUnid = {}
   local tbTotal = {}
   local tbAux = {}
      
   htmRender = "<table border = 1>\n" .. htmHeaderPlurianual()
   
   tbUnid = getUnidades()
   
   i = 1
   while (tbUnid[i]) do
      ano = 2000
      while (ano<=2003) do
         tbAux = {}
         tbAux = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Investimento, ano)
         tbAnoInvest[ano]={}
         tbAnoInvest[ano]=tbAux
         
         tbAux = {}
         tbAux = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Operacao,ano)
         tbAnoOper[ano]={}
         tbAnoOper[ano]= tbAux
         ano = ano + 1
      end -- while
      htmData, tbTotal = htmDataPlurianual(tbUnid[i].unid_sigla, tbAnoInvest, tbAnoOper, tbTotal)
      htmRender = htmRender .. htmData
      i = i+1
   end -- while
   htmRender = htmRender .. htmTotalPlurianual(tbTotal) .. "\n</table>"
   
   return htmRender
   

end -- function

-- -------------------------------------------------------------------------------------
-- Function : htmHeaderDotOrcPlurianual
-- Description : monta o header do relatorio de acompanhamento do pegaso plurianual com
--               dotacao orcamentaria
-- Parameters : mes - mes de referencia
--              ano - ano de referencia  
--              trans - tipo de transacao
-- Return : string html contendo o header da tabela
-- -------------------------------------------------------------------------------------
function htmHeaderDotOrcPlurianual(trans, mes, ano )
   --contadores
   local i = 0
   --hmtl
   local htmHeader = ""
   -- auxilliar
   local sTitle
            
   if (trans == ctTransTipo.Operacao ) then
      sTitle = "Opera&ccedil;oes"
   elseif (trans == ctTransTipo.Investimento ) then
      sTitle = "Investimentos"
   else
      sTitle = "Opera&ccedil;oes<br>Investimentos"
   end -- if            
            
   htmHeader = "<tr>\n   <td align = 'center' valign = 'center'> "..sTitle.." </td>\n"
   
   --dotacao orcamentaria
   ano = val(ano)
   mes = val(mes)
   htmHeader = htmHeader .. "   <td><table width = 400 border = 1>\n      <tr align = center valign = center>DOTACAO ORCAMENARIA</tr>\n      <tr>\n"
   i=2000
   while (i <= 2003) do
      if (i < ano) then
         htmHeader = htmHeader .. "         <td align = center valign = center width = 79>Realiz. " .. i .." </td>\n"
      else
         htmHeader = htmHeader .. "         <td align = center valign = center width = 79>Prev. " .. i .." </td>\n"
      end --if
      i = i + 1
   end -- while                                                                                            
   htmHeader = htmHeader .. "         <td align = center valign = center width = 79>Total</td>\n"
   htmHeader = htmHeader .. "      </tr>\n   </table></td>\n"
   
   -- realizado ate o ano
   if (ano>2000)then
      htmHeader = htmHeader .. "   <td align = center valign = center width = 80 border = 1 >Realiz. at Dez. de " .. (ano - 1) .. "</td>" 
   end -- if
   
   -- meses                                                                                                    
   i=1
   while (i <= mes) do
      htmHeader = htmHeader .. "   <td><table width = 210 border = 1><tr align = center valign = center>" .. ctMes[i] .. " de " .. ano .. "<tr>\n"
      htmHeader = htmHeader .. "      <tr>\n"
      htmHeader = htmHeader .. "         <td align = center valign = center width = 75>Realiz. no Mes</td>\n"
      htmHeader = htmHeader .. "         <td align = center valign = center width = 75>Realiz. at o Mes</td>\n"
      htmHeader = htmHeader .. "         <td align = center valign = center width = 75>(%) ate o Mes</td>\n"
      htmHeader = htmHeader .. "      </tr>\n   </table></td>\n"
      i = i+1
   end --while
   htmHeader = htmHeader .. " </tr>"                                     
      
   return htmHeader   
end -- function

-- -------------------------------------------------------------------------------------
-- Function : htmDadosDotOrcPlurianual
-- Description : escreve os dados do relatorio de acompanhamento do pegaso com dotacao 
--               orcamentaria
-- Parameters : unid_sigla - sigla da unidade
--              ano - ano de referencia
--              mes - mes de referencia      
--              tbAno - tabela que contm os dados anuais
--                    [ano].realizado - realizado no ano
--                    [ano].previsto  - previsto no ano
--              tbMes - tabela que contem os dados mensais
--                    [ano][mes].realizado - realizado no mes
--                    [ano][mes].acumulado - acumulado ate o mes
--              tbTotal - tabela que acumula os totais da ultima linha
--                    [ano] - acumula os totais para o ano
--                    ['total'] - acumula o total dos anos
--                    ['realizado'] - acumula os totais realizados
--                    [mes].relizado - acumula os totais realizados no mes
--                    [mes].acumulado - acumula os totais acumulados ate o mes
--              
-- Return : html contendo a nova linha da tabela, tbTotal
-- -------------------------------------------------------------------------------------
function htmDadosDotOrcPlurianual( unid_sigla, ano, mes, tbAno, tbMes, tbTotal)
   -- html
   local htmData = ''
   --contadores
   local i = 0
   --acumuladores
   local totAcum = 0
   local totReal = 0
      
   htmData = [[
      <tr>
            <td>]] .. str(unid_sigla) .. [[</td>
            ]]
              
   htmData = htmData .. [[ <td><table width = 400 border = 0><tr> ]]
   ano = val(ano)
   mes = val(mes)
   i=2000
   while (i<=2003) do
      if (i<ano) then
         htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",val(tbAno[i].realizado)) .. [[</td>
         ]]
         if not tbTotal[i] then
         	tbTotal[i] = 0
         end -- if
         tbTotal[i] = val(tbTotal[i]) + val(tbAno[i].realizado)
         totAcum = totAcum + val(tbAno[i].realizado)
         totReal = totReal + val(tbAno[i].realizado)
      else
         htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",val(tbAno[i].previsto)) .. [[</td>
         ]]
         if not tbTotal[i] then
         	tbTotal[i] = 0
         end --if                                  
         tbTotal[i] = val(tbTotal[i]) + val(tbAno[i].previsto)
         totAcum = totAcum + val(tbAno[i].previsto)
      end --if
      i = i+1
   end -- while
   
   htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",val(totAcum)) .. [[</td>
         </tr></table></td>
   ]]
   
   
   if not tbTotal['total'] then
   	tbTotal['total'] = 0
   end --if
   tbTotal['total'] = val(tbTotal['total']) + totAcum
   if (ano>2000) then
	   htmData = htmData .. [[<td align = right width = 79>]] .. format("%8.2f",val(totReal)) ..[[</td>
	   ]]                                        
   
	   if not tbTotal['realizado'] then
	   	tbTotal['realizado'] = 0
	   end --if
	   tbTotal['realizado'] = val(tbTotal['realizado']) + totReal                                                
   end --if                                             
   i=1
   while ( i <= mes ) do
      if not tbTotal[i] then
         tbTotal[i] = {}
         tbTotal[i].realizado = 0	
         tbTotal[i].acumulado = 0
      end -- if
      
      tbTotal[i].realizado = val(tbTotal[i].realizado) + val(tbMes[ano][i].realizado)
      tbTotal[i].acumulado = val(tbTotal[i].acumulado) + val(tbMes[ano][i].acumulado)
      
      htmData = htmData .. [[ <td><table width = 210 border = 0><tr>
         <td align = center width = 75>]] .. format("%8.2f",val(tbMes[ano][i].realizado)) .. [[</td>
         <td align = center width = 75>]] .. format("%8.2f",val(tbMes[ano][i].acumulado) + val(totReal)) .. [[</td>
         <td align = center width = 75>]] .. format("%8.2f",val(CalculaPerc(tbMes[ano][i].acumulado, totAcum)))  .. [[</td>
       </tr></table></td>
      ]]

      i = i+1                     
      
   end -- while                   
   
   htmData = htmData .. "</tr>"   
   
   return htmData, tbTotal
end -- function

-- -------------------------------------------------------------------------------------
-- Function : htmTotalDotOrcPlurianual
-- Description : gera html da linha de totais da tabela de dotacao orcamentaria plurianual
-- Parameters : mes - mes de referencia
--              tbTotal - tabela com os dados de totais
-- Return : string html contendo a linha com os totais
-- ------------------------------------------------------------------------------------
function htmTotalDotOrcPlurianual (mes, tbTotal)
   -- html
   local htmData = ''
   --contadores
   local i = 0
   --acumuladores
   local totAcum = 0
   local totReal = 0
      
   -- inicializacao
   mes = val(mes)
   ano = val(ano)
   
   htmData = [[
      <tr>
            <td> Total </td>
            ]]
              
   htmData = htmData .. [[ <td><table border = 0 width = 400><tr> ]]
   i=2000
   while (i<=2003) do
      htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",tbTotal[i]) .. [[</td>
         ]]
      i = i+1
   end -- while
   htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",tbTotal['total']) .. [[</td>
         </tr></table></td>
   ]]
   
   if tbTotal['realizado'] then
   	htmData = htmData .. [[<td align = right width = 79>]] .. format("%8.2f",tbTotal['realizado']) ..[[</td>
   	]]
   end                                      
   
   i=1
   while ( i <= mes ) do
      htmData = htmData .. [[ <td><table widh = 210 border = 0><tr>
         <td align = center width = 75>]] .. format("%8.2f",tbTotal[i].realizado) .. [[</td>
         <td align = center width = 75>]] .. format("%8.2f",tbTotal[i].acumulado + tbTotal['realizado']) .. [[</td>
         <td align = center width = 75>]] .. format("%8.2f",val(CalculaPerc( tbTotal[i].acumulado, tbTotal['total'] ) )) .. [[</td>
      </tr></table></td>
      ]]
      i = i+1
   end -- while
   htmData = htmData .. "</tr>"   
   return htmData

end --function

-- -------------------------------------------------------------------------------------
-- Function :  htmRenderDotOrcPlurianual
-- Description : renderia a tabela do relatorio plurianual com dotacao orcamentaria
-- Parameters : mes - mes de referencia
--              ano - ano de referencia
-- Return : string html contendo o relatorio
-- -------------------------------------------------------------------------------------
function  htmRenderDotOrcPlurianual ( mes, ano )
   --html
   local htmRender = ''
   local htmAux = ''
   --contadores
   local i=0
   local j=0
   --Tabelas
   local tbAno = {}
   local tbMes = {}
   local tbTotal = {}
   local tbUnid = {}
   
   
   tbUnid = getUnidades()
   
   -- Investimentos
   htmRender = "<table border = 1>\n"                    
   htmRender = htmRender .. htmHeaderDotOrcPlurianual( ctTransTipo.Investimento,mes, ano )
     
   i=1
   while (tbUnid[i]) do
      j=2000        
      while ( j<=2003 ) do
         tbAno[j] = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Investimento, j)
         j = j+1
      end -- while
      tbMes = getDadosMesProj(tbUnid[i].unid_cod, ctTransTipo.Investimento, mes, ano )
      htmAux, tbTotal = htmDadosDotOrcPlurianual( tbUnid[i].unid_sigla, ano, mes, tbAno, tbMes, tbTotal)
      htmRender = htmRender .. htmAux   
   
      i = i + 1   
   end --while
   
   htmRender = htmRender .. htmTotalDotOrcPlurianual(mes, tbTotal) .. "\n</table>"
   htmRender = htmRender .. "<P>"                                                 
 
   tbTotal = {}
   tbAno = {}
   tbMes = {}
     
   -- Operacao
   htmRender = htmRender .. "<table border = 1>\n"                    
   htmRender = htmRender .. htmHeaderDotOrcPlurianual( ctTransTipo.Operacao,mes, ano )
     
   i=1
   while (tbUnid[i]) do
      j=2000        
      while ( j<=2003 ) do
         tbAno[j] = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Operacao, j)
         j = j+1
      end -- while
      tbMes = getDadosMesProj(tbUnid[i].unid_cod, ctTransTipo.Operacao, mes, ano )
      htmAux, tbTotal = htmDadosDotOrcPlurianual( tbUnid[i].unid_sigla, ano, mes, tbAno, tbMes, tbTotal)
      htmRender = htmRender .. htmAux   
   	
      i = i + 1   
      
   end --while
   
   htmRender = htmRender .. htmTotalDotOrcPlurianual(mes, tbTotal) .. "\n</table>"
   htmRender = htmRender .. "<P>"
   
   tbTotal = {}
   tbAno = {}
   tbMes = {}
   
   -- Operacao
   htmRender = htmRender .. "<table border = 1>\n"                    
   htmRender = htmRender .. htmHeaderDotOrcPlurianual(ctTransTipo.Ambos, mes, ano )
     
   i=1
   while (tbUnid[i]) do
      j=2000        
      while ( j<=2003 ) do
         tbAno[j] = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Ambos, j)
         j = j+1
      end -- while
      tbMes = getDadosMesProj(tbUnid[i].unid_cod, ctTransTipo.Ambos, mes, ano )
      htmAux, tbTotal = htmDadosDotOrcPlurianual( tbUnid[i].unid_sigla, ano, mes, tbAno, tbMes, tbTotal)
      htmRender = htmRender .. htmAux   
   
      i = i + 1   
   end --while
   
   htmRender = htmRender .. htmTotalDotOrcPlurianual(mes, tbTotal) .. "\n</table>"
   htmRender = htmRender .. "<P>"
   
   return htmRender
   
end -- function

-- -------------------------------------------------------------------------------------
-- Function : htmHeaderDotOrcAnual
-- Description : gera o header da tabela de relatorio do pegaso com dotacao orcamentaria
--               anual
-- Parameters : ano - ano de referencia
--              mes - mes de referencia
--              trans - tipo de transacao
-- Return : html string que contem o header do formulario descrito acima
-- -------------------------------------------------------------------------------------
function htmHeaderDotOrcAnual( trans, mes, ano )
   --contadores
   local i = 0
   --hmtl
   local htmHeader = ""  
   -- auxiliares  
   local sTitle     
   
   if (trans == ctTransTipo.Operacao ) then
      sTitle = "Opera&ccedil;oes"
   elseif (trans == ctTransTipo.Investimento ) then
      sTitle = "Investimentos"
   else
      sTitle = "Opera&ccedil;oes<br>Investimentos"
   end -- if
   
   htmHeader = "<tr>\n   <td align = 'center' valign = 'center'> "..sTitle.." </td>\n"
   
   --dotacao orcamentaria
   i=2000
   htmHeader = htmHeader .. "   <td><table width = " .. str(80*(val(ano)-i+2) + 1) .. " >\n      <tr align = center valign = center>DOTACAO ORCAMENARIA</tr>\n      <tr>\n"
   ano = val(ano)
   mes = val(mes)
   while (i < ano) do
      htmHeader = htmHeader .. "         <td align = center valign = center width = 79>Realizado " .. str(i) .." </td>\n"
      i = i + 1
   end -- while
   htmHeader = htmHeader .. "         <td align = center valign = center width = 79>Previsto " .. str(ano) .." </td>\n"   
                                                                                               
   htmHeader = htmHeader .. "         <td align = center valign = center width = 79>Total</td>\n"
   htmHeader = htmHeader .. "      </tr>\n   </table></td>\n"
   
   -- realizado ate o ano
   if (ano>2000) then
      htmHeader = htmHeader .. "   <td align = center valign = center width = 80>Realizado at Dez. de " .. str(ano-1) .. "</td>" 
   end -- if
   
   -- meses                                                                                                    
   i=1
   while (i <= mes) do
      htmHeader = htmHeader .. "   <td><table width = 210><tr align = center valign = center width = 210>" .. str(ctMes[mes]) .. " de " .. str(ano).. "<tr>\n"
      htmHeader = htmHeader .. "      <tr>\n"
      htmHeader = htmHeader .. "         <td align = center valign = center width = 75>Realizado no Mes</td>\n"
      htmHeader = htmHeader .. "         <td align = center valign = center width = 75>Realizado at o Mes</td>\n"
      htmHeader = htmHeader .. "         <td align = center valign = center width = 75>(%) ate o Mes</td>\n"
      htmHeader = htmHeader .. "      </tr>\n   </table></td>\n"
      i = i+1
   end --while
   htmHeader = htmHeader .. " </tr>"                                     
      
   return htmHeader   
end -- function


-- -------------------------------------------------------------------------------------
-- Function : htmDadosDotOrcAnual
-- Description : escreve os dados do relatorio de acompanhamento do pegaso com dotacao 
--               orcamentaria
-- Parameters : unid_sigla - sigla da unidade
--              ano - ano de referencia
--              mes - mes de referencia      
--              tbAno - tabela que contm os dados anuais
--                    [ano].realizado - realizado no ano
--                    [ano].previsto  - previsto no ano
--              tbMes - tabela que contem os dados mensais
--                    [ano][mes].realizado - realizado no mes
--                    [ano][mes].acumulado - acumulado ate o mes
--              tbTotal - tabela que acumula os totais da ultima linha
--                    [ano] - acumula os totais para o ano
--                    ['total'] - acumula o total dos anos
--                    ['realizado'] - acumula os totais realizados
--                    [mes].relizado - acumula os totais realizados no mes
--                    [mes].acumulado - acumula os totais acumulados ate o mes
--              
-- Return : html contendo a nova linha da tabela, tbTotal
-- -------------------------------------------------------------------------------------
function htmDadosDotOrcAnual( unid_sigla, ano, mes, tbAno, tbMes, tbTotal)
   -- html
   local htmData = ''
   --contadores
   local i = 0
   --acumuladores
   local totAcum = 0
   local totReal = 0
      
   htmData = [[
      <tr>
            <td>]] .. unid_sigla .. [[</td>
            ]]
              
   i=2000
   htmData = htmData .. [[ <td><table border = 1 width = ]] .. str(80*(ano-i+2)+1) .. [[><tr> ]]
   while (i<ano) do
      htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",val(tbAno[i].realizado)) .. [[</td>
      ]]
      if not tbTotal[i] then
         tbTotal[i] = 0
      end -- if
      tbTotal[i] = tbTotal[i] + val(tbAno[i].realizado)
      totAcum = totAcum + val(tbAno[i].realizado)
      totReal = totReal + val(tbAno[i].realizado)
      i = i+1
   end -- while
   htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",val(tbAno[ano].previsto)) .. [[</td>
      ]]
   htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",val(totAcum)) .. [[</td>
         </tr></table></td>
   ]]
   if not tbTotal['total'] then
      tbTotal['total'] = 0
   end -- if
   tbTotal['total'] = tbTotal['total'] + totAcum
   
   htmData = htmData .. [[<td align = right width = 79>]] .. format("%8.2f",val(totReal)) ..[[</td>
   ]]       
   if not tbTotal['realizado'] then
      tbTotal['realizado'] = 0
   end -- if                                 
   tbTotal['realizado'] = tbTotal['realizado'] + totReal                                                
                                                
   
   i=1
   while ( i <= mes ) do
      if not tbTotal[i] then
         tbTotal[i] = {}
         tbTotal[i].realizado = 0
         tbTotal[i].acumulado = 0
      end -- if
      tbTotal[i].realizado = tbTotal[i].realizado + val(tbMes[ano][i].realizado)
      tbTotal[i].acumulado = tbTotal[i].acumulado + val(tbMes[ano][i].acumulado)
      htmData = htmData .. [[ <td><table width = 210 border = 1><tr>
         <td align = center width = 75>]] .. format("%8.2f",val(tbMes[ano][i].realizado)) .. [[</td>
         <td align = center width = 75>]] .. format("%8.2f",val(tbMes[ano][i].acumulado) + val(totReal)) .. [[</td>
         <td align = center width = 75>]] .. format("%8.2f",val(CalculaPerc(tbMes[ano][i].acumulado, totAcum)))  .. [[</td>
      </tr></table></td>
      ]]
      i = i+1                     
   end -- while                   
   
   htmData = htmData .. "</tr>"   
   
   return htmData, tbTotal
end -- function

-- -------------------------------------------------------------------------------------
-- Function : htmTotalDotOrcAnual
-- Description : gera html da linha de totais da tabela de dotacao orcamentaria anual
-- Parameters : mes - mes de referencia
--              ano - ano de referencia
--              tbTotal - tabela com os dados de totais
-- Return : string html contendo a linha com os totais
-- ------------------------------------------------------------------------------------
function htmTotalDotOrcAnual (mes, ano, tbTotal)
   -- html
   local htmData = ''
   --contadores
   local i = 0
   --acumuladores
   local totAcum = 0
   local totReal = 0
      
   htmData = [[
      <tr>
            <td> Total </td>
            ]]
   
   i=2000
   htmData = htmData .. [[ <td><table width = ]] .. str(80*(ano-i+2)+1) .. [[><tr> ]]
   while (i<=ano) do
      htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",val(tbTotal[i])) .. [[</td>
         ]]
      i = i+1
   end -- while
   htmData = htmData .. [[<td align = right width = 79> ]] .. format("%8.2f",val(tbTotal['total'])) .. [[</td>
         </tr></table></td>
   ]]
   
   
   htmData = htmData .. [[<td align = right width = 79>]] .. format("%8.2f",val(tbTotal['realizado'])) ..[[</td>
   ]]                                        
   
                                                
   
   i=1
   while ( i <= mes ) do
      htmData = htmData .. [[ <td><table width = 210><tr> 
         <td align = center width = 75>]] .. format("%8.2f",val(tbTotal[i].realizado)) .. [[</td>
         <td align = center width = 75>]] .. format("%8.2f",val(tbTotal[i].acumulado) + val(tbTotal['realizado'])) .. [[</td>
         <td align = center width = 75>]] .. format("%8.2f",val(CalculaPerc(tbTotal[i].acumulado, tbTotal['total'])))  .. [[</td>
      </tr></table></td>
      ]]
      i = i+1
   end -- while
   htmData = htmData .. "</tr>"   
   return htmData

end --function


-- -------------------------------------------------------------------------------------
-- Function :  htmRenderDotOrcAnual
-- Description : renderia a tabela do relatorio anual com dotacao orcamentaria
-- Parameters : mes - mes de referencia
--              ano - ano de referencia
-- Return : string html contendo o relatorio
-- -------------------------------------------------------------------------------------
function  htmRenderDotOrcAnual ( mes, ano )
   --html
   local htmRender = ''
   local htmAux = ''
   --contadores
   local i=0
   local j=0
   --Tabelas
   local tbAno = {}
   local tbMes = {}
   local tbTotal = {}
   local tbUnid = {}
   
   
   tbUnid = getUnidades()
   
   -- Investimentos
   htmRender = "<table border = 1>\n"                    
   htmRender = htmRender .. htmHeaderDotOrcAnual(ctTransTipo.Investimento, mes, ano )
   
   ano = val(ano)
   mes = val(mes)  
   i=1
   while (tbUnid[i]) do
      j=2000        
      while ( j<=ano ) do
         tbAno[j] = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Investimento, j)
         j = j+1
      end -- while
      tbMes = getDadosMesProj(tbUnid[i].unid_cod, ctTransTipo.Investimento, mes, ano )
      htmAux, tbTotal = htmDadosDotOrcAnual( tbUnid[i].unid_sigla, ano, mes, tbAno, tbMes, tbTotal)
      htmRender = htmRender .. htmAux   
   
      i = i + 1   
   end --while
   
   htmRender = htmRender .. htmTotalDotOrcAnual(mes, ano, tbTotal) .. "\n</table>"
   htmRender = htmRender .. "<P>"                                                  
   
   tbTotal = {}
   tbAno = {}
   tbMes = {}
     
   -- Operacao
   htmRender = htmRender .. "<table border = 1>\n"                    
   htmRender = htmRender .. htmHeaderDotOrcAnual( ctTransTipo.Operacao,mes, ano )
     
   i=1
   while (tbUnid[i]) do
      j=2000        
      while ( j<=ano ) do
         tbAno[j] = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Operacao, j)
         j = j+1
      end -- while
      tbMes = getDadosMesProj(tbUnid[i].unid_cod, ctTransTipo.Operacao, mes, ano )
      htmAux, tbTotal = htmDadosDotOrcAnual( tbUnid[i].unid_sigla, ano, mes, tbAno, tbMes, tbTotal)
      htmRender = htmRender .. htmAux   
   
      i = i + 1   
   end --while
   
   htmRender = htmRender .. htmTotalDotOrcAnual(mes, ano, tbTotal) .. "\n</table>"
   htmRender = htmRender .. "<P>"
   
   tbTotal = {}
   tbAno = {}
   tbMes = {}
   
   -- Operacao
   htmRender = htmRender .. "<table border = 1>\n"                    
   htmRender = htmRender .. htmHeaderDotOrcAnual(ctTransTipo.Ambos, mes, ano )
     
   i=1
   while (tbUnid[i]) do
      j=2000        
      while ( j<=ano ) do
         tbAno[j] = getDadosAnoProj(tbUnid[i].unid_cod, ctTransTipo.Ambos, j)
         j = j+1
      end -- while
      tbMes = getDadosMesProj(tbUnid[i].unid_cod, ctTransTipo.Ambos, mes, ano )
      htmAux, tbTotal = htmDadosDotOrcAnual( tbUnid[i].unid_sigla, ano, mes, tbAno, tbMes, tbTotal)
      htmRender = htmRender .. htmAux   
   
      i = i + 1   
   end --while
   
   htmRender = htmRender .. htmTotalDotOrcAnual(mes, ano, tbTotal) .. "\n</table>"
   htmRender = htmRender .. "<P>"
   
   return htmRender
   
end -- function