-- ----------------------------------------------------------------------------------------------------------------
-- REVISAO.LUA
-- ----------------------------------------------------------------------------------------------------------------
-- Funcao de Revisao do Projeto
--
-- Para evitar que um projeto tenha a revisao interrompida por um BACK ou STOP no navegador, ou mesmo um duplo
-- click no botao REVISAR, foi utilizada uma tabela adicional (REVISAO - campo unico e chave primaria proj_cod) 
-- onde serah gravado o codigo do projeto que estah sendo revisado, utilizando a funcao Lua CALL, soh permitindo 
-- que uma unica funcao de revisao crie uma versao do projeto por vez, ou seja, aquela que gravou primeiro o 
-- codigo do projeto na tabela REVISAO, o que garante a atomicidade da operacao
-- ----------------------------------------------------------------------------------------------------------------

-- ------------------------------------------------------------------------------
-- As operacoes de Cadastramento e Revisao encontram-se indisponiveis
-- ------------------------------------------------------------------------------
-- cgilua.redirect('../operacao_indisponivel.html?tipo_servico=revisa')  
-- exit()
-- ------------------------------------------------------------------------------ 

function RevisaProj(codigo_projeto, versao)
   -- ---------------------------------------------------------------------------
   -- Verifica se o projeto que estah sendo revisado corresponde a ULTIMA versao
   -- caso nao seja, retorna o valor NIL que eh tratado no retorno da funcao
   -- ---------------------------------------------------------------------------
   DBExec(" SELECT last_versao FROM projeto WHERE proj_cod = "..codigo_projeto)
   temp = DBRow()   
--   if temp and temp.last_versao == 'n' then  
--       return nil       
--   end 

   local i
   data_hoje = date("%Y%m%d")
   gdia      = strsub(data_hoje,7,8)
   gmes      = strsub(data_hoje,5,6)
   gano      = strsub(data_hoje,1,4)
   
   datacad = DataAtual()  
   -- ------------------------------------------------------------------------------------------------------- 
   -- Altera o valor do campo LAST_VERSAO do projeto que deu origem ao projeto revisado para 'n'
   -- e zera o contador de Follow-Up
   -- -------------------------------------------------------------------------------------------------------   
   AlteraBD("projeto", "last_versao = 'n', proj_cont_fup = 0" ," proj_cod = "..codigo_projeto.." ")

   --------------------------------
   -- Consulta a tabela de pessoal
   --------------------------------
   cresp = ConsultaBD2("pessoal","*","pes_chave = '"..getuser().."'")

   --------------------------------
   -- Consulta a tabela de projeto
   --------------------------------
   cproj = ConsultaBD2("projeto","*","proj_cod = "..codigo_projeto.."")

   ini_prev = inclui_data(cproj[1].proj_dt_ini_prev)
   fim_prev = inclui_data(cproj[1].proj_dt_fim_prev)

   campos = ""
   valores = ""

   campos = campos.."org_cod"
   valores = valores..cproj[1].org_cod
   
   campos = campos..",proj_nome"
   valores = valores..",'"..cproj[1].proj_nome.."'"

   campos = campos..",proj_versao"
   valores = valores..","..versao
   
   campos = campos..",stat_cod"
   valores = valores..","..1

   campos = campos..",proj_dt_incl"
   valores = valores..",'"..datacad.."'"

   campos = campos..",last_versao"
   valores = valores..",'s'"

   campos = campos..",proj_dt_ini_prev"
   valores = valores..",'"..ini_prev.."'"
 
   campos = campos..",proj_dt_fim_prev"
   valores = valores..",'"..fim_prev.."'"
   
   campos = campos..",proj_last_cod"
   valores = valores..","..codigo_projeto

   if cproj[1].proj_sigla then
      campos = campos..",proj_sigla"
      valores = valores..",'"..cproj[1].proj_sigla.."'"
   end
   
   if cproj[1].proj_prior then
      campos = campos..",proj_prior"
      valores = valores..","..cproj[1].proj_prior
   end   

   if cproj[1].proj_descr then
      campos = campos..",proj_descr"
      valores = valores..",'"..cproj[1].proj_descr.."'"
   end
   
   if cproj[1].proj_pae then
      campos = campos..",proj_pae"
      valores = valores..","..cproj[1].proj_pae
   end
   
   if cproj[1].unid_cod then
      campos = campos..",unid_cod"
      valores = valores..","..cproj[1].unid_cod
   end
     
   if cproj[1].proj_dt_ini_real and cproj[1].proj_dt_ini_real ~= '' then
      campos = campos..",proj_dt_ini_real"
      valores = valores..",'"..cproj[1].proj_dt_ini_real.."'"
   end
   
   if cproj[1].proj_dt_fim_real and cproj[1].proj_dt_fim_real ~= '' then
      campos = campos..",proj_dt_fim_real"
      valores = valores..",'"..cproj[1].proj_dt_fim_real.."'"
   end
   
   if cproj[1].proj_num_ote then
      campos = campos..",proj_num_ote"
      valores = valores..",'"..cproj[1].proj_num_ote.."'"
   end   
   
   if cproj[1].ger_cod then
      campos = campos..",ger_cod"
      valores = valores..","..cproj[1].ger_cod
   end
   
   if cproj[1].set_cod then
      campos = campos..",set_cod"
      valores = valores..","..cproj[1].set_cod
   end
   
   if cproj[1].proj_paeid then
      campos = campos..",proj_paeid"
      valores = valores..","..cproj[1].proj_paeid
   end
   
   if cproj[1].pes_cod then
      campos = campos..",pes_cod"
      valores = valores..","..cproj[1].pes_cod      
   end   
   
   -- ------------------------------------------------------------------------------------
   -- PROJ_RESP_CAD: o usuario responsavel pelo cadastramento do projeto ou pela revisao
   --                associa ao codigo do getuser
   -- ------------------------------------------------------------------------------------
   campos = campos..",proj_resp_cad"
   valores = valores..","..retornaPessoaCod(getuser())        
   
   if cproj[1].proj_coord then
      campos = campos..",proj_coord"
      valores = valores..",'"..cproj[1].proj_coord.."'"
   end
   
   -- -----------------------------------------------------------------------------------
   -- O colaborador deve ser alterado na revisao para o usuario responsavel pela revisao
   -- -----------------------------------------------------------------------------------
   -- PROJ_COLAB: o usuario que estah atualmente editando o projeto
   -- -----------------------------------------------------------------------------------
   if cproj[1].proj_colab then
      campos = campos..",proj_colab"
      valores = valores..",'"..getuser().."'"
   end    
       
   if cproj[1].proj_freq then
      campos = campos..",proj_freq"
      valores = valores..","..cproj[1].proj_freq
   end
   
   if cproj[1].proj_plano_rec then
      campos = campos..",proj_plano_rec"
      valores = valores..","..cproj[1].proj_plano_rec
   end
   
   if cproj[1].proj_class then
      campos = campos..",proj_class"
      valores = valores..","..cproj[1].proj_class
   end
   
   if cproj[1].proj_cod_usuario then
      campos = campos..",proj_cod_usuario"
      valores = valores..",'"..cproj[1].proj_cod_usuario.."'"
   end

   if cproj[1].proj_meta then
      campos = campos..",proj_meta"
      valores = valores..","..cproj[1].proj_meta
   end
   
   if cproj[1].proj_soma_ativ then
      campos = campos..",proj_soma_ativ"
      valores = valores..","..cproj[1].proj_soma_ativ
   end
   
   if cproj[1].proj_dt_prev_ativ then
      campos = campos..",proj_dt_prev_ativ"
      valores = valores..","..cproj[1].proj_dt_prev_ativ
   end
   
   if cproj[1].proj_desempenho then
      campos = campos..",proj_desempenho"
      valores = valores..","..cproj[1].proj_desempenho
   end

   if cproj[1].proj_perc_fisico_planejado then
      campos = campos..",proj_perc_fisico_planejado"
      valores = valores..","..cproj[1].proj_perc_fisico_planejado
   end

   if cproj[1].proj_perc_fisico_realizado then
      campos = campos..",proj_perc_fisico_realizado"
      valores = valores..","..cproj[1].proj_perc_fisico_realizado
   end
   
   if cproj[1].proj_perc_financeiro_esperado then
      campos = campos..",proj_perc_financeiro_esperado"
      valores = valores..","..cproj[1].proj_perc_financeiro_esperado
   end

   if cproj[1].proj_perc_financeiro_realizado then
      campos = campos..",proj_perc_financeiro_realizado"
      valores = valores..","..cproj[1].proj_perc_financeiro_realizado
   end


   if cproj[1].proj_est_un then
      campos = campos..",proj_est_un"
      valores = valores..",'"..cproj[1].proj_est_un.."'"
   end

   if cproj[1].proj_funcao_un then
      campos = campos..",proj_funcao_un"
      valores = valores..",'"..cproj[1].proj_funcao_un.."'"
   end

   if cproj[1].proj_tipo_un then
      campos = campos..",proj_tipo_un"
      valores = valores..",'"..cproj[1].proj_tipo_un.."'"
   end

   -- -----------------------------------------
   -- Cadastra o projeto com a nova versao 
   -- -----------------------------------------
   EscreveBD("projeto", campos, valores)  
   
   -- ---------------------------------------------------------
   -- Consulta o codigo do projeto incluido com nova versao 
   -- ---------------------------------------------------------
   dproj = ConsultaBD2("projeto","proj_cod","proj_sigla = '"..cproj[1].proj_sigla.."' and proj_versao = "..versao .."")

   -- --------------------------------------------------------------------------
   -- Cadastra os dados na tabela de "colaboradores" para o projeto revisado 
   -- O responsavel pelo cadastro antes da revisao (que eh tb o proj_colab)  
   --  inserido na tabela de colaboradores quando da revisao                
   -- --------------------------------------------------------------------------
   cod_proj = ConsultaBD2("projeto","proj_cod, proj_resp_cad","last_versao = 's' AND stat_cod = 1 AND proj_sigla = '"..cproj[1].proj_sigla.."'", nil)
   if cproj[1].proj_resp_cad ~= cod_proj[1].proj_resp_cad then    
      EscreveBD("colaboradores", "proj_cod, colaborador", cod_proj[1].proj_cod..", "..cproj[1].proj_resp_cad.."") 
   end
  
   ccolaboradores = ConsultaBD2("colaboradores","*","proj_cod = "..codigo_projeto, nil)
   local z = 1
   while ccolaboradores[z] and ccolaboradores[z].colaborador do
      -- --------------------------------------------------------------------------------------------------
      -- Se for coordenador, colaborador atual ou o colaborador antigo no insere na tabela COLABORADORES.
      -- O colaborador antigo j foi inserido na tabela COLABORADORES acima.
      -- --------------------------------------------------------------------------------------------------
      if ((ccolaboradores[z].colaborador ~= cproj[1].proj_resp_cad) and (ccolaboradores[z].colaborador ~= cresp[1].pes_cod) and (ccolaboradores[z].colaborador ~= retornaPessoaCod(cproj[1].proj_coord)) ) then
         EscreveBD("colaboradores","proj_cod, colaborador"," "..dproj[1].proj_cod..", "..ccolaboradores[z].colaborador.." ")
      end   
      z = z + 1
   end
   
   -- -----------------------------------------------------------------
   -- Consulta os dados da tabela "proj_ot" para o projeto revisado 
   -- -----------------------------------------------------------------
   cprojot = ConsultaBD2("proj_ot", "clas_cod, proj_ot_subcl_cod, ot_cod,proj_ot_ot_subclas, ot_oper_cod", "proj_cod = "..codigo_projeto.."")

   if temConteudo( cprojot[1].ot_cod ) then
      -- --------------------------------------------------------------------
      -- Cadastra os dados na tabela de "proj_ot" para o projeto revisado 
      -- para OT ESPECIFICA, OT OPERACIONAL e projetos sem OT
      -- --------------------------------------------------------------------
      coluna = ""
      valor  = ""      
       
      if temConteudo( cprojot[1].clas_cod ) then   
         coluna = coluna..",clas_cod"
         valor  = valor..","..cprojot[1].clas_cod..""  
      end
      
      if temConteudo( cprojot[1].proj_ot_subcl_cod ) then   
         coluna = coluna..",proj_ot_subcl_cod"
         valor  = valor..","..cprojot[1].proj_ot_subcl_cod..""  
      end
   
      if temConteudo( cprojot[1].proj_ot_ot_subclas ) then  
         coluna = coluna..",proj_ot_ot_subclas"
         valor  = valor..","..cprojot[1].proj_ot_ot_subclas..""
      end
   
      if temConteudo( cprojot[1].ot_oper_cod ) then   
         coluna = coluna..",ot_oper_cod"
         valor  = valor..","..cprojot[1].ot_oper_cod..""
      end
   
      EscreveBD("proj_ot","proj_cod, ot_cod "..coluna.." ", " "..dproj[1].proj_cod..", "..cprojot[1].ot_cod..
                " "..valor.." ")
   end
   -- --------------------------------------------------------------------
   -- Consulta os dados da tabela "transpetro" para o projeto revisado 
   -- --------------------------------------------------------------------
   ctransp = ConsultaBD2("transpetro","*", "proj_cod = "..codigo_projeto.."")

   -- -----------------------------------------------------------------------
   -- Cadastra os dados na tabela de "transpetro" para o projeto revisado 
   -- -----------------------------------------------------------------------
   if ctransp[1].proj_cod then
      if not ctransp[1].trans_eng then
         coluna1 = ""
         valor1 = ""
      else
         coluna1 = ",trans_eng"
         valor1 = ",'"..ctransp[1].trans_eng.."'"
      end

      if not ctransp[1].trans_cbi then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_cbi"
         valor1 = valor1..",'"..ctransp[1].trans_cbi.."'"
      end

      if not ctransp[1].trans_cac then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_cac"
         valor1 = valor1..",'"..ctransp[1].trans_cac.."'"
      end

      if not ctransp[1].trans_cad then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_cad"
         valor1 = valor1..",'"..ctransp[1].trans_cad.."'"
      end

      if not ctransp[1].trans_ui then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_ui"
         valor1 = valor1..",'"..ctransp[1].trans_ui.."'"
      end

      if not ctransp[1].trans_prog then
         coluna1 = coluna1..""
         valor1 = valor1""
      else
         coluna1 = coluna1..",trans_prog"
         valor1 = valor1..",'"..ctransp[1].trans_prog.."'"
      end

      if not ctransp[1].trans_proj then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_proj"
         valor1 = valor1..",'"..ctransp[1].trans_proj.."'"
      end

      if not ctransp[1].trans_tipo then
         coluna1 = coluna1..""
         valor1 = valor1..""
      else
         coluna1 = coluna1..",trans_tipo"
         valor1 = valor1..",'"..ctransp[1].trans_tipo.."'"
      end
      EscreveBD("transpetro","proj_cod "..coluna1.." "," "..dproj[1].proj_cod.." "..valor1.." ")
   end   
  
   -- -------------------------------------------------------------
   -- Consulta os dados da tabela "log" para o projeto revisado 
   -- -------------------------------------------------------------
   clog = ConsultaBD2("log","pes_cod,log_justifica", "proj_cod = "..codigo_projeto.."")
  
   -- ----------------------------------------------------------------
   -- Cadastra os dados na tabela de "log" para o projeto revisado 
   -- ----------------------------------------------------------------
   if clog[1] and clog[1].pes_cod then
      EscreveBD("log","proj_cod,pes_cod,log_data,log_justifica", " "..dproj[1].proj_cod..","..
                 clog[1].pes_cod..", '"..datacad.."','"..clog[1].log_justifica.."' ")
   end
  
   -- ---------------------------------------------------------------
   -- Consulta os dados da tabela "dutos" para o projeto revisado 
   -- Coloca o resultado na TABELA cdutos                         
   -- ---------------------------------------------------------------
   cdutos = ConsultaBD2("dutos","duto_nome,duto_diametro,duto_espessura,duto_extensao,duto_materiais,duto_origem,duto_destino,duto_produto,duto_outras","proj_cod = "..codigo_projeto.." ")

   -- ------------------------------------------------------------------
   -- Cadastra os dados na tabela de "dutos" para o projeto revisado 
   -- ------------------------------------------------------------------   
   coluna2 = ""
   valor2 = ""
  
   if cdutos[1].duto_nome then
      coluna2 = coluna2..",duto_nome"
      valor2 = valor2..",'"..cdutos[1].duto_nome.."'"
   end
   
   if cdutos[1].duto_diametro then
      coluna2 = coluna2..",duto_diametro"
      valor2 = valor2..","..cdutos[1].duto_diametro..""
   end
   
   if cdutos[1].duto_espessura then
      coluna2 = coluna2..",duto_espessura"
      valor2 = valor2..","..cdutos[1].duto_espessura..""
   end
 
   if cdutos[1].duto_extensao then
      coluna2 = coluna2..",duto_extensao"
      valor2 = valor2..","..cdutos[1].duto_extensao..""
   end
 
   if cdutos[1].duto_materiais then
      coluna2 = coluna2..",duto_materiais"
      valor2 = valor2..",'"..cdutos[1].duto_materiais.."'"
   end
   
   if cdutos[1].duto_origem then
      coluna2 = coluna2..",duto_origem"
      valor2 = valor2..",'"..cdutos[1].duto_origem.."'"
   end
 
   if cdutos[1].duto_destino then
      coluna2 = coluna2..",duto_destino"
      valor2 = valor2..",'"..cdutos[1].duto_destino.."'"
   end
   
   if cdutos[1].duto_produto then
      coluna2 = coluna2..",duto_produto"
      valor2 = valor2..",'"..cdutos[1].duto_produto.."'"
   end
   
   if cdutos[1].duto_outras then
      coluna2 = coluna2..",duto_outras"
      valor2 = valor2..",'"..cdutos[1].duto_outras.."'"
   end
 
   if cdutos[1] and cdutos[1] ~= '' then
      EscreveBD("dutos","proj_cod "..coluna2.." ", " "..dproj[1].proj_cod.." "..valor2.." ")
   end
    
   -- ---------------------------------------------------------------------------------
   -- Consulta os dados da tabela de custo referente ao projeto passado como parametro              
   ------------------------------------------------------------------------------------
   ccusto = ConsultaBD2("custo","*","proj_cod = "..codigo_projeto.."","custo_ano")
    
   -- -----------------------------------------------------------------
   -- Cadastra os dados de custo anual previsto do projeto revisado 
   -- -----------------------------------------------------------------
   i = 1
   while ccusto[i] do
      EscreveBD("custo", "proj_cod, custo_ano, custo_prev_ano", ""..dproj[1].proj_cod..", "..tonumber(ccusto[i].custo_ano)..", "..tonumber(ccusto[i].custo_prev_ano).."")
      i = i + 1
   end
   
   -- -------------------------------------------------------------------------------------------
   -- Consulta os dados da tabela de custo_previsto referente ao projeto passado como parametro                      
   -- -------------------------------------------------------------------------------------------
   cmcusto = ConsultaBD2("custo_previsto","dt_prevista,valor_real","proj_cod = "..codigo_projeto.."","dt_prevista")  
  
   -- ------------------------------------------------------------------
   -- Cadastra os dados de custo mensal previsto do projeto revisado 
   -- ------------------------------------------------------------------
   i = 1
   while cmcusto[i] do
      if cmcusto[i].dt_prevista then 	
         -- -------------------------------------------------------------------------------------------------------           	
         -- Remove as barras colocando no formato AAAAMMDD (inclui_data) e substitui brancos por "0" (formataData)
         -- -------------------------------------------------------------------------------------------------------           	
         cmdata = inclui_data(cmcusto[i].dt_prevista)
         EscreveBD("custo_previsto","proj_cod,dt_prevista,valor_real",""..dproj[1].proj_cod..",'"..cmdata.."',"..cmcusto[i].valor_real.."")           
      end   
      i = i + 1
   end

   -- -------------------------------------------------------------------------------------------
   -- Consulta os dados da tabela de custo_realizado referente ao projeto passado como parametro       
   -- -------------------------------------------------------------------------------------------
   crcusto = ConsultaBD2("custo_realizado", "*", "proj_cod = "..codigo_projeto.."", "dt_realizacao")
  
   -- ------------------------------------------------------------
   -- Cadastra os dados de custo realizado do projeto revisado 
   -- ------------------------------------------------------------
   i = 1
   while crcusto[i] do
      if crcusto[i].dt_realizacao then
         -- -------------------------------------------------------------------------------------------------------
         -- Remove as barras colocando no formato AAAAMMDD (inclui_data) e substitui brancos por "0" (formataData)                       
         -- -------------------------------------------------------------------------------------------------------
         crdata1 = inclui_data(crcusto[i].dt_realizacao)
         crdata2 = gano..gmes..gdia
         EscreveBD ("custo_realizado", "proj_cod, dt_realizacao, dt_inclusao, valor_real, seq_real", " "..dproj[1].proj_cod..", '"..crdata1.."', '"..crdata2.."', "..crcusto[i].valor_real..", "..crcusto[i].seq_real.." ")
      end
      i = i + 1
   end
   
   -- --------------------------------------------------
   -- Consulta as tabelas de atividade e dependencia 
   -- --------------------------------------------------
   cativ = ConsultaBD2("atividade a, dependencia b, perc_realizado c","a.ativ_cod, c.dt_atual, a.ativ_nome, "..
                       "a.ativ_dt_ini_prev, a.ativ_dt_fim_prev, a.ativ_dt_ini_real, a.ativ_dt_fim_real, "..
                       "a.ativ_peso_rel_proj,a.ativ_perc_real, a.ativ_alerta, b.dep_area_cod, b.dep_descr, "..
                       "c.perc_real"," "..
                       "a.proj_cod = "..codigo_projeto.." and a.ativ_cod *= b.dep_ativ_cod and "..
                       "a.ativ_cod *= c.ativ_cod and "..
                       "c.dt_atual = (SELECT MAX(dt_atual) FROM perc_realizado WHERE perc_realizado.ativ_cod=a.ativ_cod)"," "..
                       "a.ativ_cod")

   -- ------------------------------------------------------------------
   -- Cadastra as atividades, dependencias e o per_realizado, se tiver 
   -- ------------------------------------------------------------------
   i = 1
   while cativ[i] and cativ[i].ativ_cod do	  
      coluna3 = ""
      valor3 = ""
      if cativ[i].ativ_dt_ini_real and cativ[i].ativ_dt_ini_real ~= '' then
         coluna3 = coluna3..", ativ_dt_ini_real"
         valor3 = valor3..", '"..cativ[i].ativ_dt_ini_real.."' "
      end

      if cativ[i].ativ_dt_fim_real and cativ[i].ativ_dt_fim_real ~= '' then
         coluna3 = coluna3..", ativ_dt_fim_real"
         valor3 = valor3..", '"..cativ[i].ativ_dt_fim_real.."' "
      end
     
      if cativ[i].ativ_perc_real and cativ[i].ativ_perc_real ~= '' then
         coluna3 = coluna3..", ativ_perc_real"
         valor3 = valor3..", "..cativ[i].ativ_perc_real.." "
      end
        
      EscreveBD("atividade","proj_cod, ativ_nome, ativ_dt_ini_prev, ativ_dt_fim_prev "..coluna3..", "..
                " ativ_peso_rel_proj, ativ_alerta ", " "..
                " "..dproj[1].proj_cod..", '"..cativ[i].ativ_nome.."', '"..cativ[i].ativ_dt_ini_prev.."', "..
                " '"..cativ[i].ativ_dt_fim_prev.."' "..valor3..", "..
                " "..cativ[i].ativ_peso_rel_proj..", "..cativ[i].ativ_alerta.." ")
      
      -- -----------------------------------------------------------
      -- Consulta o codigo da atividade para incluir a dependencia 
      -- e o perc_realizado correspondente 
      -- -----------------------------------------------------------
      dativ = ConsultaBD2("atividade","ativ_cod","proj_cod = "..dproj[1].proj_cod.." and ativ_nome = '"..cativ[i].ativ_nome.."'")
      
      -- -------------------------------------
      -- Grava as dependencias que existirem
      -- -------------------------------------
      if cativ[i].dep_area_cod then
         EscreveBD ("dependencia", "dep_proj_cod, dep_ativ_cod, dep_area_cod, dep_descr", " "..dproj[1].proj_cod..", "..
                    " "..dativ[1].ativ_cod..", "..cativ[i].dep_area_cod..", '"..cativ[i].dep_descr.."' ")
      end
      
      -- ---------------------------------------------
      -- Grava os dados existentes na perc_realizado
      -- ---------------------------------------------
      if cativ[i].perc_real then
      	 EscreveBD ("perc_realizado", "proj_cod, ativ_cod, dt_atual, perc_real", " "..dproj[1].proj_cod..", "..
      	            " "..dativ[1].ativ_cod..", '"..date('%c').."', "..cativ[i].perc_real.." ")
      end
      	            
      -- -------------------------------------------------------------------------------------------------
      -- Consulta a tabela de observacoes para verificar se h ou nao observacao para a atividade gravada      
      ----------------------------------------------------------------------------------------------------
      cobs = ConsultaBD2 ("observacoes", "obs_titulo, obs_descr, obs_versao, obs_status, obs_resp, obs_sigla, obs_tipo", "obs_proj_cod = "..codigo_projeto.." and obs_ativ_cod = "..cativ[i].ativ_cod.." ")
      
      -- ---------------------------------------------------------------
      -- Cadastra a observaco relativa a atividade que foi revisada 
      -- ---------------------------------------------------------------
      j = 1
      while cobs[j] and cobs[j].obs_titulo do
         data = gano..gmes..gdia
         EscreveBD ("observacoes", "obs_titulo, obs_proj_cod, obs_ativ_cod, obs_descr, obs_data, obs_versao, "..
                    "obs_status, obs_resp, obs_sigla, obs_tipo", " '"..cobs[j].obs_titulo.."', "..
                    ""..dproj[1].proj_cod..", "..dativ[1].ativ_cod..", '"..cobs[j].obs_descr.."', "..
                    "'"..data.."', "..cobs[j].obs_versao..", "..cobs[j].obs_status..", "..
                    "'"..cobs[j].obs_resp.."', "..cobs[j].obs_sigla..", "..cobs[j].obs_tipo.." ")
         j = j + 1
      end
      i = i + 1
   end
   
   -- ------------------------------------
   -- Consulta a tabela de observacoes 
   -- ------------------------------------
   dobs = ConsultaBD2 ("observacoes", "obs_titulo, obs_descr, obs_data, obs_versao, obs_status, obs_resp, "..
                       "obs_sigla, obs_tipo", "obs_proj_cod = "..codigo_projeto.." and obs_ativ_cod is NULL") 
   
   -- -------------------------------------
   -- Cadastra as observacoes, se tiver 
   -- -------------------------------------
   i = 1
   while dobs[i] and dobs[i].obs_titulo do
      EscreveBD ("observacoes","obs_titulo, obs_proj_cod, obs_descr, obs_data, "..
                 " obs_versao, obs_status, obs_resp, obs_sigla, obs_tipo", " '"..dobs[i].obs_titulo.."', "..
                 " "..dproj[1].proj_cod..",'"..dobs[i].obs_descr.."', '"..dobs[i].obs_data.."', "..
                 " "..dobs[i].obs_versao..","..dobs[i].obs_status..", '"..dobs[i].obs_resp.."', "..
                 " "..dobs[i].obs_sigla..", "..dobs[i].obs_tipo.." ")
      i = i + 1
   end
      
   -- Grava follow_up do projeto que foi revisado
   atualizaFollowUp(codigo_projeto, 1, 'O projeto foi revisado.')  
          
   -- Grava follow_up do novo projeto resultante da revisao           
   atualizaFollowUp(dproj[1].proj_cod, 1, 'Reviso do projeto (nova verso).')      
                        


   -------------------------------------------------------------------------------------
   -- Verifica se projeto anterior(pai) esta em auditoria.--
   -------------------------------------------------------------------------------------
   proj_audi = ConsultaBD2 ("projeto", "proj_aud", "proj_cod = "..codigo_projeto) 
   
   if proj_audi[1] and tonumber(proj_audi[1].proj_aud) == 1 then

      ---------------------------------------------------------------------------------------------------------
      --Se o projeto anterior(pai) estava em auditoria, o projeto novo(filho) deve estar em auditoria tambem.--
      ---------------------------------------------------------------------------------------------------------
      AlteraBD("projeto", "proj_aud = 1" ," proj_cod = "..dproj[1].proj_cod.." ")

      -------------------------------------------------------------------------------------
      -- Copia para o novo(filho) projeto os dados de auditoria do projeto anterior(pai).--
      -------------------------------------------------------------------------------------
      audi = ConsultaBD2 ("auditoria", "*", "proj_cod = "..codigo_projeto) 
   
      -- -------------------------------------
      -- Cadastra as auditorias, se tiver 
      -- -------------------------------------
      i = 1
      while audi[i] and audi[i].proj_cod do
         EscreveBD ("auditoria","proj_cod, aud_data, aud_obs", " "..dproj[1].proj_cod..",'"..audi[i].aud_data.."', '"..audi[i].aud_obs.."' ")
         i = i + 1
      end
   end

   return dproj[1].proj_cod
   
end -- function RevisaProj

-- ---------------------------------------------------------------------------

if cgi.acao == 'Revisa' then 
   
    dofile( 'acessabd.lua' )    
    dofile( 'global.lua' )
    dofile( 'mail.lua' ) 

    if ESTADO_BD ~= 'aberto' then
       AbreBD()
    end
    
    -- ------------------------------------------------------------------------
    -- Verifica se o usuario tem permissao para cadastrar e revisar projetos.
    -- ------------------------------------------------------------------------
    sql = "SELECT b.cad_proj FROM pessoal a, permissao_acesso b WHERE a.pes_chave = '"..
           getuser().."' and a.pes_cod = b.pes_cod and b.ac_cod <> 1"
      
    DBExec(sql)
    local tmp = DBRow()
    if tmp then
       permissao_cadastro = tmp.cad_proj
    end
    
    grava    = " INSERT INTO revisao ( proj_cod ) VALUES ( "..cgi.cod.." ) "
    deleta   = " DELETE FROM revisao WHERE proj_cod = "..cgi.cod
    
    -- --------------------------------------------------------------------------------------- 
    -- A funcao Lua CALL (com MODE setado para 'X') permite que se ocorrer algum erro na 
    -- execucao de alguma funcao, esse erro possa ser tratado (maiores detalhes no manual)
    -- ---------------------------------------------------------------------------------------
    if call( DBExec, {grava}, "x", function() end ) then
                         
       versao = tonumber(cgi.versao) + 1 
       novo_codigo_projeto = RevisaProj(cgi.cod, versao)                 
       DBExec(deleta)               
        
       if temConteudo( novo_codigo_projeto ) then           
          -- ---------------------------------------------------------------------------------------
          -- Chama a funcao EMAIL da pagina MAIL.LUA para enviar os e-mails aos responsaveis
          -- A funcao EMAIL jah fecha o banco
          -- ---------------------------------------------------------------------------------------     
          msgsuc, email = email( cgi.cod , 'revisao')

          if ESTADO_BD == 'fechado' then
             AbreBD()
          end         


          --------------------------------------------------------------------------------------------------
          --A variavel "form_rev" impede que um projeto seja formalizado, sem que ocorra alguma alteracao.--
          --------------------------------------------------------------------------------------------------

          ----------------------------------------------------------------
          --Verifica se projeto a ser revisado contem documentos.       --
          --Se tiver redireciona para a pagina de revisao de documentos.--
          ----------------------------------------------------------------
          sql = "SELECT * FROM documento WHERE proj_cod = "..cgi.cod
          DBExec(sql)

          tmp = DBRow()
          if tmp then
             if permissao_cadastro and permissao_cadastro == 1 then
                tipo_edi = "projeto"
             else
                tipo_edi = "atividades"
             end
             cgilua.redirect('../revisa_doc.html?'..cgilua.encodetable({revisao = 'revisao', msgsuc = msgsuc, email = email, proj_ant = cgi.cod, proj_cod = novo_codigo_projeto, proj_sigla=tmp.proj_sigla, tipo_servico = 'edita', tipo_edi = tipo_edi}))
          elseif permissao_cadastro and permissao_cadastro == 1 then -- Revisao completa para usuarios cadastrados na permissao_acesso
             cgilua.redirect('../cadastra.html?'..cgilua.encodetable({revisao = 'revisao', msgsuc = msgsuc, email = email, proj_cod = novo_codigo_projeto,tipo_servico = 'edita', tipo_edi = 'projeto', form_rev='1'}))
          else -- os outros usurios so poderam revisar as atividades
             cgilua.redirect('../atividades.html?'..cgilua.encodetable({revisao = 'revisao', msgsuc = msgsuc, email = email, proj_cod = novo_codigo_projeto,tipo_servico = 'edita', tipo_edi = 'atividades', form_rev='1'}))
          end
          if ESTADO_BD == 'aberto' then
             FechaBD()
          end         
          
       end
       
    else
       if ESTADO_BD == 'aberto' then
          FechaBD()
       end         

       cgilua.redirect('../erro_acao.html?proj_cod='..cgi.cod)                                              

    end    
               
end