dofile("acessabd.lua")
dofile("global.lua")

function VerificaDesempenho(projeto)
   if ESTADO_BD ~= 'aberto' then
      AbreBD()
   end
   DBExec( ' SELECT desempenho_descr FROM projeto, desempenho WHERE proj_cod = '..projeto..
           ' AND proj_desempenho = desempenho_cod ' )     
   local valor = DBRow()
   if valor and valor.desempenho_descr then      
      return valor.desempenho_descr
   end
   return 'no foi possvel determinar o desempenho do projeto'
end

-- Retorna os percentuais de realizao fsica planejada e realizada,
-- ou nil caso o projeto no tenha atividades.
function Desempenho(codigo_projeto)
  if ESTADO_BD ~= 'aberto' then
    AbreBD()
  end
  local atividades = ConsultaBD2("atividade",
                                 "*",
                                 "proj_cod="..codigo_projeto,
                                 "ativ_nome")
  if (atividades==nil or atividades[1]==nil or atividades[1].ativ_cod==nil) then
    return {}
  end
  DBExec("select tipo_curva from orgao, projeto "..
         "where orgao.org_cod=projeto.org_cod"..
         "      and projeto.proj_cod="..codigo_projeto)
  local tipo_curva = DBRow().tipo_curva
  FechaBD()
  local planejado = 0
  local realizado = 0
  local hoje = date("%Y%m%d")
  local i = 1
  while (atividades[i]) do
    local ativ_planejado = planejadoCurvaS(tipo_curva,
                                           atividades[i].ativ_dt_ini_prev,
                                           atividades[i].ativ_dt_fim_prev,
                                           hoje)
    local peso = tonumber(atividades[i].ativ_peso_rel_proj)/100
    planejado = planejado+ativ_planejado*peso
    if (atividades[i].ativ_perc_real) then
      realizado = realizado + tonumber(atividades[i].ativ_perc_real)
    end
    i = i + 1   
  end
  return {
    ["planejado"] = planejado,
    ["realizado"] = realizado,
  }
end

-- Definio das curvas. Todas devem ter no mnimo dois pontos, sendo o
-- primeiro 0 e o ltimo 100.
local curvas = {
  ["33"] = {0, 0.1, 0.1, 0.2, 0.4, 0.8, 1.3, 2.1, 3.4, 5.1, 7.6, 10.9, 15.1,
            20.3, 26.4, 33.3, 40.9, 48.8, 56.7, 64.4, 71.5, 77.9, 83.4, 87.9,
            91.5, 94.2, 96.2, 97.6, 98.5, 99.1, 100.0},
  ["50"] = {0, 0.3, 0.5, 0.8, 1.4, 2.3, 3.6, 5.5, 8.1, 11.5, 15.9, 21.2, 27.4,
            34.5, 42.1, 50.0, 57.9, 65.5, 72.6, 78.8, 84.1, 88.5, 91.9, 94.5,
            96.4, 97.7, 98.6, 99.2, 99.5, 99.7, 100.0},
  ["67"] = {0, 0.9, 1.5, 2.4, 3.8, 5.8, 8.5, 12.1, 16.6, 22.1, 28.5, 35.6,
            43.3, 51.2, 59.1, 66.7, 73.6, 79.7, 84.9, 89.1, 92.4, 94.9, 96.6,
            97.9, 98.7, 99.2, 99.6, 99.8, 99.9, 99.9, 100.0},
}

-- Recebe uma data no formato [aaaammdd] e retorna o dia, ms e ano.
function quebraData(data)
   data = tonumber(data)
   local ano = floor(data/10000)
   local mes = floor((data-ano*10000)/100)
   local dia = mod(data, 100)
   return dia, mes, ano
end

-- Recebe duas datas no formato [aaaammdd] e retorna o nmero de dias entre
-- elas.
function diasEntre(data_ini, data_fim)
   local dia, mes, ano = quebraData(data_ini)
   local sec_ini = date2sec(ano.." "..mes.." "..dia, "%Y %m %d")
   dia, mes, ano = quebraData(data_fim)
   local sec_fim = date2sec(ano.." "..mes.." "..dia, "%Y %m %d")
   return floor((sec_fim-sec_ini)/(60*60*24))
end

-- Recebe o tipo de uma curva S (que deve estar cadastrados no vetor curvas,
-- caso contrrio  utilizada a curva 50), a data de incio e fim previstas da
-- atividade e uma data de referncia, para a qual  retornada a realizao
-- fsica planejada. As datas devem ter o formato [aaaammdd].
function planejadoCurvaS(tipo_curva, dt_ini, dt_fim, dt_ref)
  if (dt_ini == dt_fim) then
    if (dt_ref <= dt_ini) then
      return 0
    else
      return 100
    end
  end
  local razao = diasEntre(dt_ini, dt_ref)/diasEntre(dt_ini, dt_fim)
  tipo_curva = tipo_curva or "50"
  local curva = %curvas[tipo_curva] or %curvas["50"]
  local amostra = razao * (getn(curva) - 1) + 1
  if (amostra <= 1) then return 0 end
  if (amostra >= getn(curva)) then return 100 end
  local pre_amostra = floor(amostra)
  local pos_amostra = ceil(amostra)
  if (pre_amostra == pos_amostra) then return curva[amostra] end
  local planejado_pre_amostra = curva[pre_amostra]
  local planejado_pos_amostra = curva[pos_amostra]
  local delta_p = planejado_pos_amostra-planejado_pre_amostra
  local planejado_amostra = planejado_pre_amostra+delta_p*(amostra-pre_amostra)
  return planejado_amostra
end