-- -------------------------------------------------------------------------------------------------
-- FUNC_CALCULOFINANCEIRO.LUA
-- -------------------------------------------------------------------------------------------------
-- $Revision:$
--
-- $Update Date:$ $NoKeywords:$
-- 
-- Copyright by TeCGraf (PUC-RIO)
-- -------------------------------------------------------------------------------------------------       
-- Ao se efetuar os calculos financeiros, os percetuais financeiros esperados e realizados, deverao 
-- ser armazenados em uma tabela no Banco de Dados (tabela PROJETO) como:
-- proj_perc_financeiro_esperado e proj_perc_financeiro_realizado
-- -------------------------------------------------------------------------------------------------    

-- -----------------------------------------------------------------------------------------------------
-- Atualizacao dos campos PROJ_PERC_FINANCEIRO_REALIZADO e PROJ_PERC_FINANCEIRO_ESPERADO do pojeto
-- informado
-- -----------------------------------------------------------------------------------------------------      
-- 1.1 - Para obtermos o Percentual ESPERADO de Realizacao Financeira , devera ser obtido o valor total 
--       planejado ateh o ano corrente.
--
-- Ex: Usando um Projeto X, cujas suas datas de inicio  e termino previsto sejam 01/07/2000 e 31/12/2003, 
--     respectivamente e onde suas previsoes financeiras, sejam :
--     2000 - R$ 200.000,00
--     2001 - R$ 700.000,00
--     2002 - R$ 500.000,00
--     2003 - R$ 300.000,00
--
-- Se levarmos em consideracao a data atual 22/07/2002 teremos a indicacao de que o ano que nos interessa 
-- eh o de 2002, nesse caso obteremos o valor total planejado do projeto ateh 2002 ( R$ 200.000,00 + 
-- R$ 700.000,00 + R$ 500.000,00 = R$ 1.400.000,00 )                  
-- -----------------------------------------------------------------------------------------------------
-- 1.2 - O proximo passo serah obter "quantos %" representa o valor obtido no item 1.1 em relacao ao 
--       custo total do projeto (R$ 200.000,00 + R$ 700.000,00 + R$ 500.000,00 + R$ 300.000,00 = 
--       R$ 1.700.000,00), assim teremos a seguinte relacao:
--
--       R$ 1.700.000,00   -   100 %
--       R$ 1.400.000,00   -     X % 
--
-- O que nos d o precentual esperado ateh o momento de 82,35%         
-- -----------------------------------------------------------------------------------------------------
-- 1.3 - Agora que jah temos o % Esperado Financeiro, vamos obter o % de REALIZACAO FINANCEIRA, da 
--       seguinte maneira:
--
-- Consideramos o valor total financeiro ateh a data atual como sendo os 100 % Esperado, conforme 
-- detalhamos no item 1.1 e que ateh a data atual tenhamos acumulado um valor total financeiro realizado, 
-- informado pelo usuario de R$ 1.100.000,00 , teremos a relacao mostrada abaixo:
--
-- R$ 1.400.000,00    -    100 %
-- R$ 1.100.000,00    -      X %
--
-- O que nos dah o percentual de realizacao financeira ateh o momento de 78,57%
-- ----------------------------------------------------------------------------------------------------- 

function atualizaFinanceiro(projeto)
   ano_atual = date('%Y')         
      
   -- Abre conexao com o Banco de Dados   
   if ESTADO_BD ~= 'aberto' then
      AbreBD()
   end 
   
   perc_esperado = 0
   perc_realizado = 0
   
   -- Calculando o custo ESPERADO
   queryTotal   = 'SELECT SUM(custo_prev_ano) AS soma FROM custo WHERE proj_cod = '..projeto
   queryParcial = 'SELECT SUM(custo_prev_ano) AS soma FROM custo WHERE proj_cod = '..projeto..
                  ' AND custo_ano <='..ano_atual
   
   DBExec(queryTotal)
   total = DBRow()
   
   if total and total.soma then   
      total = total.soma  
   end  

   DBExec(queryParcial)
   parcial = DBRow()

   if parcial and parcial.soma then  
      parcial_esperado = parcial.soma       
      if total and tonumber(total) ~= 0 then
         perc_esperado = format( '%.2f',(parcial.soma/total)*100 )
      end    
        
      -- Calculando o custo REALIZADO         
      if tonumber(parcial_esperado) ~= 0 then        
         -- Soh sao inseridos valores para custo realizado para datas menores ou iguais a do dia da insercao     
         DBExec('SELECT SUM(valor_real) AS soma FROM custo_realizado WHERE proj_cod = '..projeto)
         total = DBRow() 
         if total and total.soma then
            perc_realizado = format( '%.2f',(total.soma/parcial_esperado)*100 ) 
         end                   
      end      
   end 
   
   DBExec('UPDATE projeto SET proj_perc_financeiro_esperado = '..perc_esperado..', '..
          'proj_perc_financeiro_realizado = '..perc_realizado..' WHERE proj_cod = '.. projeto)         
end  
 

        