----------------------------- campoEstrututado.lua -----------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	campoEstruturado.lua

-- Descrio:  	Contm a definio e os mtodos da classe CampoEstruturado e de
--		suas subclasses. Esta  uma classe abstrata, criada para agrupar
--		mtodos comuns a objetos que representam estruturas que 
--		relacionam campos de um formulrio html. Esta estrutura pode ser 
--		de vetor (CampoVetor), quando todos os campos tem o mesmo tipo e 
--		semntica ou de 'registro' (CampoComposto), quando os campos tem 
--		tipos e ou semnticas diferentes. 

-- Data:	19/06/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes: 
--     Quem      |   Quando   |   Motivo
-- --------------|------------|-------------------------------------------------
--  
--------------------------------------------------------------------------------

--Quando estiver debugando, retire os comentrios das 2 linhas a seguir
--$debug
--cgilua.htmlheader()

--------------------------------------------------------------------------------
--Carrega funes utilizadas neste mdulo
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Definio da Classe CampoEstruturado, como uma subcalsse da classe Campo
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos estruturados.
--------------------------------------------------------------------------------
--print('<BR>Criao da Classe CampoEstruturado<BR>')
CampoEstruturado = CampoIlimitado:subclass { 
	tipoMsg = nil,
	numComponentes = nil,
	componentes   = nil,
	numCampos = 3,
	
--Variveis estticas
	camposVazios	= nil, 
	camposInvalidos = nil,
	contVazios	= nil,
	contInvalidos   = nil,
	tamanhoValor	= nil
	
}

--------------------------------------------------------------------------------
-- Definio dos mtodos da Classe CampoEstruturado
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Construtor
--------------------------------------------------------------------------------
function CampoEstruturado:new (o)
--------------------------------------------------------------------------------
-- Descrio: Construtor dos objetos da classe CampoEstruturado.

-- Parmetros:	o: tabela com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro tipoMsg.
--		o[5]: argumento a ser armazendado no  membro numComponentes.   
--		o[6]: argumento a ser armazendado no  membro componentes.   

-- Retorno: 	Retorna o objeto da classe CampoEstruturado criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
    --print('Entrando em CampoEstruturado:new()<BR>')
   local p, ultimo_indice --Auxiliares para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Descobre  o ltimo ndice dos membros da superclasse
   ultimo_indice = self.parent.ultimoIndice
   
   --Atribuio dos campos especficos da classe
   p.tipoMsg = o[ultimo_indice+1]
   p.numComponentes = tonumber(o[ultimo_indice+2])
   p.componentes = o[ultimo_indice+3]
   
   --Validao dos atributos
   
   --Se o campo  obrigatrio
   if 	p.obrigatorio then
   	--Devem ser especificadas as restries de tamanho e a
   	--dependncia em relao aos componentes
   	if 	not p.restricoes or ( not p.restricoes.tamanho and 
   		not p.restricoes.subconjunto ) then
   		error([[Se o campo estruturado  obrigatrio, devem ser 
   		especificados os atributos 'restricoes.tamanho' e 
   		'restricoes.subconjunto', para a determinao do conjunto de 
   		componentes obrigatrios. Consulte o manual tcnico.]])
   	end
   end
   --if p.restricoes then
   --	print( 'restricoes.subconjunto:', p.restricoes.subconjunto, '<BR>' )
   --	print( 'restricoes.tamanho:',p.restricoes.tamanho, '<BR>')
   --end

   p:atribuiDefaultRestricoes()
	
   if 	not p.tipoMsg or 
   	(p.tipoMsg ~= 'global' and p.tipoMsg ~= 'global_local' and 
   	p.tipoMsg ~= 'local') then
   	error("Parmetro tipoMsg invlido! Deve ser 'global', 'global_local' ou 'local'.")
   end 
   
   if 	not p.numComponentes then
	error("Parmetro numComponentes invlido! Deve ser um nmero inteiro.")
   end
   
   if 	not p.componentes or type(p.componentes) ~= 'table' then
	error("Parmetro componentes invlido! Deve ser uma tabela.")
   end
     
   --Variveis estticas
   p.camposVazios = {}
   p.contVazios = 0
   p.contInvalidos = 0
   p.camposInvalidos = {}

   --Para debugar
   --p:print()
   --print('Saindo de CampoEstruturado:new()<BR>')
   return p
end

--------------------------------------------------------------------------------
function CampoEstruturado:atribuiDefaultRestricoes()
--------------------------------------------------------------------------------
-- Descrio:	Quando no so especificadas todas as restries, esta funo,
--		atribui valores default as restries no especificadas.
-- Autora:	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --Se foi especificada apenas a restrio de tamanho 
   --o subconjunto de campos a criticar default  '#indeterminado'
   if 	self.restricoes and self.restricoes.tamanho and 
   	not self.restricoes.subconjunto then
   	self.restricoes.subconjunto = '#indeterminado'
   	--print("restricoes.subconjunto = '#indeterminado'<BR>")
   end
   
   --Se foi especificada apenas a restricao de subconjunto 
   --a restrio de tamanho default  ['=='] ao nmero de elementos 
   --do subconjunto
   if 	self.restricoes and self.restricoes.subconjunto and 
   	not self.restricoes.tamanho then
 	--Quando o subconjunto  indetermindado o nmero de elementos do 
 	--subconjunto  o prprio nmero de componentes do campo 
 	if	self.restricoes.subconjunto == '#indeterminado' then
   		self.restricoes.tamanho = { ['=='] = self.numComponentes }
   		--print("restricoes.tamanho = { ['=='] = "..self.numComponentes.." }<BR>")
   	
 	--Quando o subconjunto  determinado devemos calcular o seu nmero de elementos
   	else
   		local str, num_elementos = gsub(self.restricoes.subconjunto..",", ",", ",")
   		self.restricoes.tamanho = { ['>='] = num_elementos }
   		--print("restricoes.tamanho = { ['>='] = "..
   		--self.restricoes.tamanho['>='].." }<BR>")
   	end
   end
end

--------------------------------------------------------------------------------
function CampoEstruturado:critica()
--------------------------------------------------------------------------------
-- Descrio:	Critica o preenchimento de um campo estruturado, 
--		validando os valores preenchidos em seus componentes e verifi- 
--		cando se as restries semnticas (subconjunto de componentes 
--		obrigatrio) do prprio campo estruturado foram satisfeitas. 
--		Quando a configurao para a exibio de mensagens de erro indi-
--		ca a exibio de mensagens 'locais', gera as mensagens associa-
--		das aos componentes invlidos.

-- Retorno: 	Retorna nil caso o preenchimento do campo esteja correto. Em ca-
--		so contrrio, retorna uma string que identifica o erro encontra- 
--		do e, quando necessrio, uma tabela com os parmetros para a ge-
--		rao da mensagem associada a este erro. 
--		Obs: quando  detectado erro, mas a configurao para a exibio 
--		de mensagens de erro no indica a exibio de mensagens 
--		'globais', a string retornada  um identificador especial que in
--		dica que ocorreu erro no preenchimento do campo estruturado, mas 
--		que no est associado a nenhuma mensagem.

-- Autora:	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local erro, parametros
	local restricao_tamanho_insatisfeita = nil
	local sufixo_invalido = ""
		
	------------------------------------------------------------------------
	--PASSO1: Criticar os componentes do campo estruturado, gerando suas
	--respectivas mensagens de erro
	------------------------------------------------------------------------
	self:criticaComponentes()
	-----------------------------------------------------------------------
	--PASSO2: Analisar os campos vazios, para determinar se o campo estru-
	--turado est incompleto, de acordo com as restries de subconjunto e 
	--de tamanho especificadas
	-----------------------------------------------------------------------	
	--print( 'contVazios', self.contVazios, '<BR>' )  	
	
  	--Se o campo no  obrigatrio e todos os componentes esto
  	--vazios, isto no  um erro
  	if 	not self.obrigatorio and 
  		self.contVazios == self.numComponentes then
  		return nil
  	end
  		
	--Se existem restries de tamanho, verificar se so satisfeitas
	if 	self.restricoes then
		--Calcula o tamanho do valor do campo estruturado entrado
		--pelo usurio (nmero de campos preenchidos)

		if 	self.restricoes.subconjunto == '#indeterminado' then
			self.tamanhoValor = self.numComponentes - self.contVazios
		else
		       	local str
		       	str, self.numComponentes = gsub(self.restricoes.subconjunto..",", ",", ",")
			self.tamanhoValor = self.numComponentes - self.contVazios
		end		
		erro, parametros = self:criticaTamanho()

		if 	erro then
        		restricao_tamanho_insatisfeita = 1
 			--Guarda os parmetros necessrios para 
 			--a gerao da mensagem de erro
 		end			
	end

	-----------------------------------------------------------------------
	--PASSO 3: Gerao da mensagem do campo estruturado e de seus componen-
	--tes de acordo com a crtica dos componentes vazios e a configurao 
	--especificada
	------------------------------------------------------------------------
	
	-----------------------------------------------------------------------
	---Se o campo estruturado estiver incompleto, a mensagem relativa a este 
	--fato, deve ser exibida de forma 'global', independentemente da configu
	--rao especificada. Caso o campo tenha componentes tambm estruturados
	--As mensagens locais de campos vazios, de seus componentes, devem
	--ser apagadas, caso tenham sido geradas
	-----------------------------------------------------------------------
	if 	restricao_tamanho_insatisfeita then
		self:apagaMensagensLocais('camposVazios')
		erro = 'restricao_tamanho_insatisfeita'
	end
	
	-----------------------------------------------------------------------
	--Anlise do erro de componentes invlidos e da configurao para a 
	--exibio de mensagens
	-----------------------------------------------------------------------
	if	self.contInvalidos > 0 then
		--------------------------------------------------------------
		--Se a configurao requer a exibio de mensagens locais
		--------------------------------------------------------------
		if 	self.tipoMsg == 'local' or self.tipoMsg == 'global_local' then
		--------------------------------------------------------------
			--Gerar as mensagens locais dos componentes invlidos
			self:geraMensagensLocais()
			--------------------------------------------------------
			--Se a congfigurao determina somente a gerao de 
			--mensagens locais
			--------------------------------------------------------
			if 	self.tipoMsg == 'local' then
				------------------------------------------------
				--Se no foi detectado o erro de campo incomple-
				--to no ser gerada nenhuma mensagem global. 
				--Mas a funo precisa retornar um identificador  
				--para sinalizar a ocorrncia de erro no campo 
				--estruturado. Para isto  usado o identificador 
				--especial 'erro_campo_estruturado', que no 
				--est associado a nenhuma mensagem
				------------------------------------------------
				if 	not erro then
					erro = 'erro_campo_estruturado'
				end
			end
		end	
		
		----------------------------------------------------------------
		--Se a congfigurao determina a gerao da mensagem global
		---------------------------------------------------------------
		if 	self.tipoMsg == 'global' or self.tipoMsg == 'global_local' then
		---------------------------------------------------------------
			--Se no foi detectado o erro de campo incomple-
			--to, cria o identificador do erro de campo inv
			--lido, em caso contrrio, simplesmente concatena
			--a palavra '_invalido' neste identificador
			------------------------------------------------
			if 	not erro then
				erro = 'componentes_invalidos' 
			else
				erro = erro.."_componentes_invalidos"
			end
		end
		
	end
	
	--print( self.identificador, erro, '<BR>')	
	return erro, parametros
end

--------------------------------------------------------------------------------
function CampoEstruturado:geraMensagensLocais()
--------------------------------------------------------------------------------
-- Descrio:	Gera as mensagens associadas aos componentes invlidos de um 
--		campo composto, a partir das informaes armazenadas na 
--		tabela membro camposInvalidos.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local i = 1
	local camposInvalidos = self.camposInvalidos
	local campo
	while 	camposInvalidos[i] do
		campo = camposInvalidos[i]
		cgi['msg_'..campo.identificador] = 
		campo.componente:geraMensagem(campo.erro, 
		campo.parametros )
		i = i + 1
	end
end


--------------------------------------------------------------------------------
function CampoEstruturado:trataErroComponente( componente, nome, erro, 
						parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Armazena as informaes necessrias para a gerao da mensagem 
--		associada a um erro detectado para um componente de um campo 
--		estruturado. Se o erro detectado foi a falta de preenchimento de
--		um campo obrigatrio, estas informaes so armazenadas no mem
--		bro self.camposVazios, se o erro  relativo ao preenchimento 
--		invlido, so armazenadas em self.camposInvalidos.

-- Parmetros:	id: string que identifica o componente no formulrio html, ou 
--		seja,  o valor especificado no atributo NAME da TAG HTML que 
---		gera a sua interface.

--		nome: string indicando o nome ou apelido, pelo qual o usu-
--		rio identifica o componente na interface. Para um campo da 
--		classe CampoComposto, esta string  o prprio membro nome do 
--		componente. Para um campo do tipo CampoVetor, esta string 
--		da forma i<SUP><U>genero</U></SUP>, onde i  o ndice do 
--		componente e genero  o genero do membro nome do componente. 
--		
--		componentes: membro componentes do componente, caso este seja 
--		estruturado ou nil em caso contrrio. 

--		erro:  string que identifica o erro detectado para o componente.

-- 		parametros: tabela com os parmetros necessrios para a gerao 
--		da mensagem associada ao erro.  

-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	--print( componente.identificador, erro, '<BR>' )
	--Se o componente  estruturado 
	if 	componente.componentes then
		if 	erro == 'restricao_tamanho_insatisfeita' then
			--Verificar se o componente como um todo est vazio,
			--ou seja, se todos os seus componentes esto vazios
			if 	componente.contVazios == componente.numComponentes then
				erro = 'campo_obrigatorio_vazio'
			end
		end			
	end

	--Se o componente est vazio
	if 	erro == 'campo_obrigatorio_vazio' then 
	        --Verificar se ele realmente  um elemento obrigatrio
		--print( 'Campo Vazio<br>' )		
	        
	        if 	self.restricoes then 
			local subconjunto = ","..
			strlower(self.restricoes.subconjunto).."," 
	        	if 	self.restricoes.subconjunto == '#indeterminado' 
	        		or strfind(subconjunto, "%,%s*"..
	        			strlower(componente.identificador).."%s*%,") then
	        		--Guarda o ndice do campo na lista de campos vazios
				self.contVazios = self.contVazios + 1
	        		self.camposVazios[self.contVazios] =  
	        		{ nome = nome }
				--print( componentes.identificador, ' est vazio<BR>' )
			end
		end
	else
		 --Guarda o ndice do campo na lista de campos invlidos
		 self.contInvalidos = self.contInvalidos + 1
		 self.camposInvalidos[self.contInvalidos] = 
		{ nome = nome, identificador = componente.identificador, componente = componente, erro = erro, 
		parametros = parametros }
		 --print( componente.identificador, erro, parametros.formato_campo, '<BR>' )
	end
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoComposto, como subcalsse da classe CampoEstruturado
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos compostos.
--------------------------------------------------------------------------------
CampoComposto = CampoEstruturado:subclass { numCampos = 0 }

--------------------------------------------------------------------------------
-- Construtor
--------------------------------------------------------------------------------
function CampoComposto:new (o)
--------------------------------------------------------------------------------
-- Descrio: Construtor dos objetos da classe CampoComposto.

-- Parmetros:	o: tabela com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro tipoMsg.
--		o[5]: argumento a ser armazendado no  membro numComponentes.   
--		o[6]: argumento a ser armazendado no  membro componentes.   

-- Retorno: 	Retorna o objeto da classe CampoComposto criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
    --print('Entrando em CampoComposto:new()<BR>')
   local p, ultimo_indice --Auxiliares para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'composto'
    
   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   p.nomeComponente = "campo"
   --p:imprime()		
   --Para debugar
   --p:print()
   --print('Saindo de CampoComposto:new()<BR>')
   return p
end

--------------------------------------------------------------------------------
function CampoComposto:apagaMensagensLocais(tipo_msg)
--------------------------------------------------------------------------------	
-- Descrio: 	Quando um componente de um campo composto tambm  estrutura
--		do, sua critica pode ter gerado mensagens de erro locais para
--		seus componentes. Se a configurao de mensagem do campo 
--		composto no possuir mensagem local, as mensagens locais, 
--		geradas para  os componentes deste componente devem ser 
--		apagadas. Isto  feito atravs deste mtodo.

-- Parmetros:	
--		tipo_msg: string que identifica o nome da tabela que armazena
--		informaes sobre a gerao das mensagens de erro a serem 
--		apagadas. 

--		'CamposVazios': mensagens sobre falta de preenchimento
--		de campos obrigatorios. 
--		'CamposInvalidos': mensagens sobre preenchimento de campos com
--		valores invlidos. 

-- Autora: 	Gilda Dahis
-- Data: 	03/11/1998
--------------------------------------------------------------------------------
  local tabela
  tabela = self[tipo_msg]

  local id, campo = next( tabela, nil )
 	
  while campo do
	--print( id )  
 	cgi['msg_'..id] = nil
 	--Se o componente tambm  um campo estruturado, as mensagens
 	--locais de seus componentes tambm devem ser apagadas
 	if 	campo.componentes then
 		campo.componentes:apagaMensagensLocais(tipo_msg) 	
      	end
      	id, campo = next( tabela, id )
  end
end

--------------------------------------------------------------------------------
function CampoComposto:criticaComponentes()
--------------------------------------------------------------------------------
-- Descrio: 	Critica os componentes de um campo composto, armazenando nas
--		tabelas self.camposVazios e self.camposInvalidos 
--		informaes necessrias a posterior gerao de mensagens de 
--		erro.

-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local numComponentesObrigatorios = 0
	local componentes = self.componentes
	local i = 1
	local erro, parametros
	--Para cada componente do campo
	self.numComponentes = 0
	while( componentes[i] ) do
		  --Conta o nmero de componentes do campo composto
		  self.numComponentes = self.numComponentes + 1
		
		  --Critica o componente
		  componentes[i].obrigatorio = 'sim'
		  erro, parametros = componentes[i]:critica()
		  --print( componentes[i].identificador, erro, '<BR>' )	
		  --Se houve erro
		  if 	erro then
			self:trataErroComponente( componentes[i], 
			componentes[i].nome, erro, parametros )
		  end
		  i = i + 1
	end
end

--------------------------------------------------------------------------------
function CampoComposto:identifica( prefixos, i )
--------------------------------------------------------------------------------
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	self.contVazios = 0
	self.identificador  = prefixos.elemento..i

	local componentes = self.componentes	
	local prefixos_componentes = prefixos.componentes
	local j = 1
	while( componentes[j] ) do
		componentes[j]:identifica( prefixos_componentes[j], i )	
		j = j + 1
	end
end

--------------------------------------------------------------------------------
function CampoComposto:limpa()
--------------------------------------------------------------------------------
-- Descrio: 	Limpa o valor dos componentes do campo, deixando-os 'vazios'.
-- Autora: 	Gilda Dahis
-- Data: 	10/08/1999
--------------------------------------------------------------------------------
	local componentes = self.componentes
	local i = 1
	while( componentes[i] ) do
		componentes[i]:limpa()		
		i = i + 1
	end
end

function CampoComposto:prefixoIdentificador()
	local ids = 
	{ elemento = self.identificador, componentes = {} }
	local componentes, i
	componentes = self.componentes
	i = 1
	while 	componentes[i] do
		ids.componentes[i] = componentes[i]:prefixoIdentificador()
		i = i + 1
	end
	return ids
end


--------------------------------------------------------------------------------
-- Definio da Classe CampoVetor, como subcalsse da classe CampoEstruturado
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos do tipo vetor.
--------------------------------------------------------------------------------
CampoVetor = CampoEstruturado:subclass { 
	numCampos = 0
}

--------------------------------------------------------------------------------
function CampoVetor:new (o)
--------------------------------------------------------------------------------
-- Descrio: Construtor dos objetos da classe CampoVetor.

-- Parmetros:	o: tabela com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro tipoMsg.
--		o[5]: argumento a ser armazendado no  membro numComponentes.   
--		o[6]: argumento a ser armazendado no  membro componentes.   
--		o[7]: argumento a ser armazenado no membro id_elemento.
-- Retorno: 	Retorna o objeto da classe CampoEstruturado criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   local p --Auxiliar para a criao do objeto
   p = self.parent:new (o, self)
   p.tipo = 'vetor'
   p.parent 		= self

   --Descobre  o ltimo ndice dos membros da superclasse
   ultimo_indice = self.parent.ultimoIndice
   
   --p.idElemento 	= p.componentes.identificador
   --Se o elemento   estruturado, guarda os prefixos dos seus componentes
   p.prefixosComponentes = p.componentes:prefixoIdentificador()   	
   p.nomeComponente 	= p.componentes.nome
   --p:print()
   return p
end

--------------------------------------------------------------------------------
function CampoVetor:apagaMensagensLocais(tipo_msg)
--------------------------------------------------------------------------------	
-- Descrio: 	Quando um componente de um campo vetor tambm  estrutura
--		do, sua critica pode ter gerado mensagens de erro locais para
--		seus componentes. Se a configurao de mensagem do campo vetor 
--		no possuir mensagem local, as mensagens locais, geradas para  
--		os componentes deste componente devem ser apagadas. Isto  feito 
--		atravs deste mtodo.

-- Parmetros:	
--		tipo_msg: string que identifica o nome da tabela que armazena
--		informaes sobre a gerao das mensagens de erro a serem 
--		apagadas. 

--		'CamposVazios': mensagens sobre falta de preenchimento
--		de campos obrigatorios. 
--		'CamposInvalidos': mensagens sobre preenchimento de campos com
--		valores invlidos. 

-- Autora: 	Gilda Dahis
-- Data: 	03/11/1998
--------------------------------------------------------------------------------
  local tabela
  tabela = self[tipo_msg]

  local id, campo = next( tabela, nil )
  while campo do
 	cgi['msg_'..id] = nil
 	--Se o componente tambm  um campo estruturado, as mensagens
 	--locais de seus componentes tambm devem ser apagadas
 	if 	campo.componentes then
 		--indice = gsub( id, self.idElemento, "" )
 		--self.componentes:identifica( self.idElemento, indice )
 		campo.componentes:apagaMensagensLocais(tipo_msg) 	
 	end
	id, campo = next( tabela, id )
  end
end

--------------------------------------------------------------------------------
function CampoVetor:criticaComponentes()
--------------------------------------------------------------------------------
-- Descrio: 	Critica os componentes de um campo vetor, armazenando nas
--		tabelas self.camposVazios e self.camposInvalidos 
--		informaes necessrias a posterior gerao de mensagens de 
--		erro.

-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local i, num_elementos
	local erro, parametros
	local ord = ordinal(genero(self.nomeComponente))
	num_elementos = self.numComponentes

	self.componentes.obrigatorio = 'sim'

	i = 0
	while 	i < num_elementos do	
		i = i + 1
		self.componentes:identifica( self.prefixosComponentes, i)
		erro, parametros = self.componentes:critica()

		--print( '<DIR><DIR>',self.idElemento..i, "-", cgi[self.idElemento..i],"-", erro, '</DIR></DIR>' )
		--Se houve erro
		if 	erro then
			self:trataErroComponente( self.componentes, i..ord, 
				erro, parametros )
		end
	end
end

--------------------------------------------------------------------------------
function CampoVetor:identifica( prefixos, i )
--------------------------------------------------------------------------------
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	self.contVazios = 0
	self.identificador  = prefixos.elemento..i
	self.componentes:identifica( prefixos_componentes, i )	
end

--------------------------------------------------------------------------------
function CampoVetor:limpa()
--------------------------------------------------------------------------------
-- Descrio: 	Limpa o valor dos componentes do campo, deixando-os 'vazios'.
-- Autora: 	Gilda Dahis
-- Data: 	10/08/1999
--------------------------------------------------------------------------------
	local i, num_elementos
	num_elementos = self.numComponentes

	i = 0
	while 	i < num_elementos do	
		i = i + 1
		self.componentes:identifica( self.prefixosComponentes, i)
		self.componentes:limpa()
	end
end

--------------------------------------------------------------------------------
function CampoVetor:prefixoIdentificador()
--------------------------------------------------------------------------------
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local ids = 
	{ 	
		elemento = self.identificador, 
		componentes = self.componentes:prefixoIdentificador()  
	}
	return ids
end
