----------------------------- campoIlimitado.lua -------------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	campoIlimitado.lua
-- Descrio:  	Contm a definio e os mtodos da classe CampoIlimitado. 
--		Esta classe  uma classe abstrata, criada para agrupar membros e 
--		mtodos comuns a todas as classes de campos cujos valores poss-
--		veis no so limitados pela interface. 
-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes: 
--     Quem      |   Quando   |   Motivo
-- --------------|------------|-------------------------------------------------
--  
--------------------------------------------------------------------------------

--Quando estiver debugando, retire os comentrios das 2 linhas a seguir
--$debug
--cgilua.htmlheader()

--------------------------------------------------------------------------------
-- Definio da Classe CampoIlimitado
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos cujos valores possveis no 
-- so limitados pela interface. Exempo: campos representados no formulrio 
-- pelos seguintes widgets para entrada de text. (em html, representa
-- dos pelas tags <INPUT TYPE = "text" ...> e <TEXTAREA>.
--------------------------------------------------------------------------------
--print('Criao da classe CampoIlimitado<BR>')
CampoIlimitado = Campo:subclass {	
	restricoes  = nil,	--tabela com as restries do campo, de tamanho 
				--e de limite 
	numCampos = 1			
}

--------------------------------------------------------------------------------
function CampoIlimitado:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoIlimitado.

-- Parmetros:	o: tabela com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.

-- Retorno: 	Retorna o objeto da classe CampoLimitadoInterface criado.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
   --print('Entrando em CampoIlimitado:new()<BR>')
   local p, ultimo_indice --Auxiliares para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'ilimitado'
   --print( p.nome )	
   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Descobre  o ltimo ndice dos membros da superclasse
   ultimo_indice = self.parent.ultimoIndice
   
   
   --Atribuio dos campos especficos da classe
   p.restricoes = o[ultimo_indice+1]

   --Validao dos atributos
   if 	p.restricoes then
	if 	type(p.restricoes) ~= 'table' then
		local num_param = ultimo_indice+1
		error( 'Erro na construo do campo '..
		p.identificador..': o '..num_param..ordinal('o')..
		' parmetro (restricoes) deve ser uma tabela. Consulte o manual.') 
	end
   	if 	p.restricoes.tamanho then
		p:validaRestricoesTamanho()
	end
   end
   
   --Para debugar
   --p:print()
   --print('Saindo de CampoIlimitado:new()<BR>')
   return p
end

--------------------------------------------------------------------------------
function CampoIlimitado:criticaTamanho()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica se o campo satisfaz as restries de tamanho especifi-
--		cadas no membro self.restricoes.tamanho.
--
-- Retorno: 	retorna o identificador do erro encontrado, junto com os 
--		parmetros para a gerao da mensagem de erro, ou nil caso o 
--		valor do campo satisfaa s restries de tamanho.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	--print( 'CampoIlimitado:criticaTamanho()<BR>' )
	local restricao = self.restricoes.tamanho
	local operador, operando = next( restricao, nil )
		
	while( operador ) do
		--print( self.nome, self.tamanhoValor, operador, operando, '<br>' )
		if 	not dostring( 'return '..self.tamanhoValor..operador..
					operando ) then
		  	return 'restricao_tamanho_insatisfeita', 
		  		{ operador = operador } 
		end
	
		operador, operando = next( restricao, operador )
	end
	return nil
end

--------------------------------------------------------------------------------
function CampoIlimitado:validaRestricoesTamanho()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica a consistncia das restries de tamanho recebidas 
--		como parmetros no construtor de um campo cujos valores so 
--		ilimitados pela interface.

-- Retorno: 	aborta a execuo, caso as restries de tamanho estejam incon-
--		sistentes, exibindo uma mensagem com a inconsistncia 
--		identificada. Caso no seja detetada inconsistncuia, retorna 
--		nil.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local tabela_restricoes = self.restricoes.tamanho

	if 	tabela_restricoes['<'] then
		error("A restrio de tamanho correta  '<='.")
	end

	if 	tabela_restricoes['>'] then
		error("A restrio de tamanho correta  '>='.")
	end

	--Se foi especificada a restrio '==', ela deve ser a nica restrio
	--especificada
	if 	tabela_restricoes['=='] and 
		(tabela_restricoes['<='] or tabela_restricoes['=>']) then
		error([[Restries de tamanho ambguas: a restrio "==" deve 
			ser nica.]]) 
	end
end