----------------------------- campoLimitadoInterface.lua -----------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	campoLimitadoInterface.lua
-- Descrio:  	Contm a definio e os mtodos da classe abstrata 
--		CampoLimitadoInterface e de suas subclasses. Um objeto desta 
--		classe representa um campo cujos valores possveis so limitados 
--		pela prpria interface do campo, como  o caso de radio buttons, 
--		checkbox e dropdownlists. 

-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes: 
--     Quem      |   Quando   |   Motivo
-- --------------|------------|-------------------------------------------------
--  Gilda	  10/08/1999   Incluso do mtodo CampoLimitadoInterface:limpa()
--------------------------------------------------------------------------------

--Quando estiver debugando, retire os comentrios das 2 linhas a seguir
--$debug
--cgilua.htmlheader()

--------------------------------------------------------------------------------
-- Definio da Classe CampoLimitadoInterface, como subclasse da classe 
-- Campo
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos simples cujos valores poss-
-- veis so limitados pela interface: Exempo: radio buttons, checkboxes, 
-- drop down lists, etc. 
--------------------------------------------------------------------------------
--print( 'Criao da classe CampoLimitadoInterface<BR>' )
CampoLimitadoInterface = Campo:subclass { valor = nil, numCampos = 1 }

--------------------------------------------------------------------------------
-- Construtor
--------------------------------------------------------------------------------
function CampoLimitadoInterface:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoLimitadoInterface.

-- Parmetros:	o: tabela com os valores a serem armazenados nos membros do 
--		objeto a ser construdo. 
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro valor.
-- Retorno: 	Retorna o objeto da classe CampoLimitadoInterface criado.

-- Autora: 	    Gilda Dahis
-- Data: 	      05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   ----print('Entrando em CampoLimitadoInterface:new()<BR>')
   local p, ultimo_indice --Auxiliares para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Descobre  o ltimo ndice dos membros da superclasse
   ultimo_indice = self.parent.ultimoIndice
   
   
   --Atribuio dos campos especficos da classe
   p.valor 	= o[ultimo_indice+1]
   
   --Para debugar
   ----print('Saindo de CampoLimitadoInterface:new()<BR>')
   --p:print()
   return p
end

--------------------------------------------------------------------------------
function CampoLimitadoInterface:critica()
--------------------------------------------------------------------------------
-- Descrio: 	Faz a crtica de um campo cujos valores so limitados pela 
--		interface, que portanto, consiste apenas em verificar se o campo 
--		 obrigatrio e neste caso, se ele foi preenchido.
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o preen-
--    		chimento do campo esteja correto.

-- Autora:	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	--Verifica se o campo  obrigatrio e se foi preenchido
	--print('CampoLimitadoInterface:critica()<BR>')
	if 	self.obrigatorio then
		--print( 'O campo limitado pela interface  obrigatrio.<BR>' )
		if 	self:vazio() then
			--print( 'O campo limitado pela interface est vazio.<BR>' )
			return 'campo_obrigatorio_vazio'
		end
	end
end

--------------------------------------------------------------------------------
function CampoLimitadoInterface:limpa()
--------------------------------------------------------------------------------
-- Descrio: 	Limpa o valor do campo, deixando-o 'vazio'.
-- Autora: 	Gilda Dahis
-- Data: 	10/08/1999
--------------------------------------------------------------------------------
	cgi[self.identificador] = nil
end

--------------------------------------------------------------------------------
function CampoLimitadoInterface:vazio()
--------------------------------------------------------------------------------
-- Descrio: 	verifica se um campo "marcvel" foi preenchido ou no.
-- Retorno: 	retorna um valor diferente de nil, se o campo foi preenchido e 
--    		nil, em caso contrrio.
-- Autora: 	Gilda Dahis
-- Data:       	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	return self.valor == nil
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoCheckbox, como subclasse da classe 
-- CampoLimitadoInterface
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos cuja enntrada  feita por
-- widgets do tipo checkbox.
--------------------------------------------------------------------------------
--print( 'Criao da classe CampoCheckbox<BR>' )
CampoCheckbox = CampoLimitadoInterface:subclass { numCampos = 0 }

--------------------------------------------------------------------------------
-- Definio dos mtodos da Classe CampoCheckbox
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoCheckbox:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoCheckbox.

-- Parmetros:	o: tabela com os valores a serem armazenados nos membros do 
--		objeto a ser construdo. 
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro valor.
-- Retorno: 	Retorna o objeto da classe CampoCheckbox criado.

-- Autora: 	    Gilda Dahis
-- Data: 	      05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   ----print('Entrando em CampoRadio:new()<BR>')
   local p --Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'checkbox' 
   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   ----print('Saindo de CampoRadio:new()<BR>')
   return p
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoLista, como subclasse da classe 
-- CampoLimitadoInterface
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos cujos valores possveis
-- so limitados pela interface e cuja entrada de dados  feita atravs de sele-
-- o em uma lista de valores: Exempo: dropdown list (multiple e single).
-- Representados em html pela tag <SELECT>. 
--------------------------------------------------------------------------------
--print( 'Criao da classe CampoLista<BR>' )
CampoLista = CampoLimitadoInterface:subclass { numCampos = 0 }

--------------------------------------------------------------------------------
function CampoLista:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoLista.

-- Parmetros:	o: tabela com os valores a serem armazenados nos membros do 
--		objeto a ser construdo. 
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro valor.
-- Retorno: 	Retorna o objeto da classe CampoLista criado.

-- Autora: 	    Gilda Dahis
-- Data: 	      05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   ----print('Entrando em CampoLista:new()<BR>')
   local p --Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'lista'

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   ----print('Saindo de CampoLista:new()<BR>')
   return p
end


--------------------------------------------------------------------------------
function CampoLista:vazio()
--------------------------------------------------------------------------------
-- Descrio: 	verifica se um campo do tipo lista foi preenchido ou no.
-- Retorno: 	retorna um valor diferente de nil, se o campo foi preenchido e 
--    		nil, em caso contrrio.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	if 	self.valor == nil then
		return 1
	else
		local ini --Para guardar o incio da captura na funo strfind 
		local fim --Para guardar o fim da captura na funo strfind 
		ini, fim, self.valor = strfind( self.valor, "%s*(.*)%s*")
		--print( self.valor )
		--print( self.valor == "")
		
		return self.valor == ""
	end
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoRadio, como subclasse da classe 
-- CampoLimitadoInterface
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos cuja enntrada  feita por
-- widgets do tipo radio button.
--------------------------------------------------------------------------------
--print( 'Criao da classe CampoRadio<BR>' )
CampoRadio = CampoLimitadoInterface:subclass { numCampos = 0 }

--------------------------------------------------------------------------------
-- Definio dos mtodos da Classe CampoRadio
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoRadio:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoRadio.

-- Parmetros:	o: tabela com os valores a serem armazenados nos membros do 
--		objeto a ser construdo. 
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro valor.
-- Retorno: 	Retorna o objeto da classe CampoRadio criado.

-- Autora: 	    Gilda Dahis
-- Data: 	      05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   ----print('Entrando em CampoRadio:new()<BR>')
   local p --Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'radio'
 
   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   ----print('Saindo de CampoRadio:new()<BR>')
   return p
end


--------------------------------------------------------------------------------
-- Definio da Classe CampoListaComOutro, como subclasse da classe 
-- CampoLimitadoInterface
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos cujos valores possveis
-- so limitados pela interface e cuja entrada de dados  feita atravs de sele-
-- o em uma lista de valores: Exempo: dropdown list (multiple e single).
-- Representados em html pela tag <SELECT>. 
--------------------------------------------------------------------------------
--print( 'Criao da classe CampoListaComOutro<BR>' )
CampoListaComOutro = CampoLista:subclass { numCampos = 0 }

--------------------------------------------------------------------------------
function CampoListaComOutro:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoLista.

-- Parmetros:	o: tabela com os valores a serem armazenados nos membros do 
--		objeto a ser construdo. 
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro valor.
-- Retorno: 	Retorna o objeto da classe CampoLista criado.

-- Autora: 	    Milene
-- Data: 	      05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   ----print('Entrando em CampoLista:new()<BR>')
   local p --Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'lista_com_outro'

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   ----print('Saindo de CampoLista:new()<BR>')
   return p
end
