----------------------------- campoSimples.lua ---------------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	campoSimples.lua
-- Descrio:  	Contm a definio e os mtodos da classe abstrata CampoSimples 
--		e de suas subclasses. Um objeto desta classe representa um campo 
--		associado a um widget do tipo 'caixa de texto' (simples ou mul-
--		tipla) de um formulrio html. Ele contem informaes semnticas 
--		sobre o campo e o valor digitado dinamicamente pelo usurio. 

-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes: 
--     Quem      |   Quando   |   Motivo
-- --------------|------------|-------------------------------------------------
--  Gilda	  10/08/1999    Incluso dos mtodos CampoSimples:limpa(), 
--				CampoData:limpa().
--------------------------------------------------------------------------------

--Quando estiver debugando, retire os comentrios das 2 linhas a seguir
--$debug
--cgilua.htmlheader()

--------------------------------------------------------------------------------
--Carrega funes utilizadas neste mdulo
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function AbreBD( dsn )
--------------------------------------------------------------------------------
-- Funo que faz conexo com o Banco de Dados. Em caso de erro, retorna a 
-- mensagem a ser apresentada 
--------------------------------------------------------------------------------
  erro = DBOpen('DSN='..dsn..';')
  if erro then 
     error('No foi possvel estabelecer conexo com o  Banco de Dados. '..
	    'Entre em contato com suporte.\n<BR>Error='..erro)
  end
end 

--------------------------------------------------------------------------------
function escapa_caracteres_magicos( str )
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoSimples.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--
-- Retorno: 	Retorna o objeto da classe CampoSimples criado.

-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	caracteres_magicos = { "(", ")", "%", ".", "[", "*", "-", "?" }
	local retorno = str
	local i = 1
	while 	caracteres_magicos[i] do
		retorno = gsub( retorno, "%"..caracteres_magicos[i],
				"%%"..caracteres_magicos[i])
		i = i + 1
	end
	return retorno
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoSimples, como uma subcalsse da classe Campo
--------------------------------------------------------------------------------
-- Os objetos desta classe, representam os campos no estruturados, ou seja, 
-- o valor digitado no campo  considerado o valor do campo.
--------------------------------------------------------------------------------
--print('Criao da classe CampoSimples<BR>')
CampoSimples = CampoIlimitado:subclass { 
	atributosChave = nil,	--Quando atravs do campo so informados valores
				--que sero armazenados como chave primria este
				--parmetro conter uma tabela com os atributos
				--'tabela' e 'campo', strings que informam, 
				--respectivamente, quais os nomes da tabela e do
				--campo chave do BD, ao qual o campo est associ
				--ado
	valor = nil,	--string representando o valor dinmico do campo, aquele
			--que foi digitado pelo usurio no formurio html, ou al
			--guma formatao especial deste valor (exemplo: uma data 
			--convertida para um outro formato.
	numCampos = 2
}

--------------------------------------------------------------------------------
-- Definio dos mtodos da Classe CampoSimples
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Construtor
--------------------------------------------------------------------------------
function CampoSimples:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoSimples.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--
-- Retorno: 	Retorna o objeto da classe CampoSimples criado.

-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em CampoSimples:new()<BR>')
   local p, ultimo_indice --Auxiliares para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Descobre  o ltimo ndice dos membros da superclasse
   ultimo_indice = self.parent.ultimoIndice
   
   
   --Atribuio dos campos especficos da classe
   p.atributosChave = o[ultimo_indice+1]
   if 	o[ultimo_indice+2] then
   	--print( 'valor carregado' )
   	p.valor = o[ultimo_indice+2]
   	p:retiraEspacosExternos()
   	p.tamanhoValor = strlen( p.valor )
   end
   
  if 	p.restricoes and p.restricoes.limite then
	--print( p.nome )
	p:validaRestricoesLimite()
  end
     
   --Para debugar
   --p:print()
   --print('Saindo de CampoSimples:new()<BR>')
   return p
end

--------------------------------------------------------------------------------
function CampoSimples:critica()
--------------------------------------------------------------------------------
-- Descrio:	Faz a crtica de um campo simples, verificando primeiramente se 
--		ele  obrigatrio e est vazio, em seguida verificando se o 
--		valor digitado  do tipo requerido, depois verificando se alguma 
--		das restries  violada (tamanho, limite, valores invlidos) e 
--		por fim verificando se o campo  chave e se o valor no  um va-
--		lor j cadastrado.
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o 
--		preenchimento do campo esteja correto.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	--Teste para facilitar a debugao
	if 	not self.valor then
			local nome_var 
			if 	self.identificador then
					nome_var = ", provavelmente cgi."..self.identificador
			end
			error( 'O atributo valor, do campo '..self.nome..
			[[  nil. Ele deve conter o contedo digitado no widget que representa 
			o campo no formulrio HTML]]..nome_var..'.') 
	end
	
	local erro, parametros
	
	--Verifica se o campo est vazio e se  obrigatrio
	if 	self:vazio() then
		--print( self.nome, 'est vazio.' )
		if 	self.obrigatorio then
			return 	'campo_obrigatorio_vazio'
		else
			return nil
		end
	end
	
	--Verifica se o valor digitado no campo  de um tipo vlido
	erro, parametros = self:criticaTipo()
	if 	erro then	
		return erro, parametros
	end
	
	--Caso hajam restries, verifica se o valor digitado 
	--satisfaz as mesmas
	if 	self.restricoes then
		erro, parametros = self:criticaRestricoes()
	end	
		
	if 	not erro then
		--Se o sistema possui um banco de dados
		if 	DSN then
			--Caso o campo esteja relacionado a um campo 
			--'chave primria', de uma tabela do BD, 
			--verifica se o valor digitado j no est 
			--cadastrado
			if 	self.atributosChave then
				erro = self:criticaChave()
			end
		end
	end
	--print( self.identificador, erro )
	return erro, parametros
end

--------------------------------------------------------------------------------
function CampoSimples:criticaChave()
--------------------------------------------------------------------------------
-- Descrio:	Verifica se o valor digitado em um campo j est cadastrado na 
--		tabela especificada no membro atributosChave.
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o valor 
--		no esteja cadastrado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local	sql		--auxiliar para armazenar a string a consulta 
				--sql para a recuperao do valor digitado na 
				--tabela onde o campo  chave
	local atributos_chave	--auxiliar para armazenar os atributos da tabela
				--self.atributos_chave 			
	local 	registro	--auxiliar para armazenar a tabela com o resulta
				--do da consulta
	
	atributos_chave = self.atributosChave		
	--Abre o Banco de Dados
	AbreBD( DSN )
	sql = 	'SELECT '..atributos_chave.campo..' FROM '..atributos_chave.tabela..
		' WHERE '..atributos_chave.campo..' = '..self:formataParaConsulta()
	--cgilua.htmlheader()
	--print( sql )
	--Executa a consulta para verificar se j existe na tabela um registro 
	--cuja chave seja o valor digitado
	DBExec( sql ) 
	registro = DBRow()
	--print( registro )
	--Fecha o Banco de Dados
	DBClose()
	
	--Caso exista tal registro, retorna o erro 'chave_ja_cadastrada'		
	if	registro then
		return 'chave_ja_cadastrada'
	end
end

--------------------------------------------------------------------------------
function CampoSimples:criticaRestricoes()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica se o valor do campo satisfaz a todas as restries 
--		especificadas no membro restricoes. (limite, tamanho, 
--		valores_invalidos).
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso nenhuma 
--		restrio tenha sido violada.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	--print( self.nome, '<BR>' )
	local operador, operando, restricao, erro, parametros, cont
	local valor_digitado, valor_limite, valores_invalidos
	
	--Se existem restries de limite, verificar se o valor do campo 
	--as satisfaz
	if 	self.restricoes.limite then
		restricao = self.restricoes.limite
		operador, operando = next( restricao, nil )
		
		while( operador ) do
			--print( self.valor..operador..operando.valor )
			if 	self.parent.parent ~= CampoNumerico then
				valor_digitado = "'"..strlower(self.valor).."'"
				valor_limite = "'"..strlower(operando.valor).."'"	
			else
				valor_digitado = self.valor
				valor_limite = operando.valor	
			end
			if not dostring( 'return '..valor_digitado..operador..valor_limite ) then
			   --print('Falso<BR>')
			   return 'limite_ultrapassado', { operador = operador } 
			end
			operador, operando = next( restricao, operador )
		end
	end
	
	--Se existem restries de tamanho, verificar se o valor do campo 
	--as satisfaz

	if 	self.restricoes.tamanho then
		erro, parametros = self:criticaTamanho()
		
		if 	erro then
			return erro, parametros
		end
	end	

	--Se existem restries quanto a valores invalidos, verificar se o 
	--valor do campo as satisfaz
	if 	self.restricoes.valores_invalidos then
		valores_invalidos = ","..self.restricoes.valores_invalidos..","
		if 	strfind(strlower(valores_invalidos), 
			"%,%s*"..strlower(escapa_caracteres_magicos(self.valor)).."%s*%,") then
			return 'valor_invalido'
		end
	end
end

--------------------------------------------------------------------------------
function CampoSimples:formataParaConsulta()
--------------------------------------------------------------------------------
-- Descrio: 	Formata o valor do campo para ser utilizado em um comando SQL. 
--		Pois, por exemplo, campos do tipo string tem que ser envolvidos 
--		por " ' ",  para no violar a sintaxe SQL.

-- Retorno: 	retorna uma string com o valor do campo formatado para ser 
--		utilizado em um comando SQL.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	return cgilua.dblua_escape(self.valor)
end

--------------------------------------------------------------------------------
function CampoSimples:limpa()
--------------------------------------------------------------------------------
-- Descrio: 	Limpa o valor do campo, deixando-o 'vazio'.
-- Autora: 	Gilda Dahis
-- Data: 	10/08/1999
--------------------------------------------------------------------------------
	cgi[self.identificador] = ""
end

--------------------------------------------------------------------------------
function CampoSimples:retiraEspacosExternos()
--------------------------------------------------------------------------------
-- Descrio: 	modifica o valor de um campo simples, retirando os espaos 
--    		externos deste valor. 
--    		Exemplo: seja o valor do campo = "   Maria Pereira da Silva   "
--    		transforma este valor em "Maria Pereira da Silva"

-- Retorno: 	-
-- Autora: 	    Gilda Dahis
-- Data: 	      05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local ini --Para guardar o incio da captura na funo strfind 
	local fim --Para guardar o fim da captura na funo strfind 
	ini, fim, self.valor = strfind( self.valor, "%s*(.*)%s*")
end

--------------------------------------------------------------------------------
function CampoSimples:validaRestricoesLimite()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica a consistncia das restries de limite recebidas 
--		como parmetros no construtor de um campo simples.

-- Retorno: 	aborta a execuo, caso as restries de limite estejam incon-
--		sistentes, exibindo uma mensagem com a inconsistncia 
--		identificada. Caso no seja detectada inconsistncia, retorna 
--		nil.

-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local tabela_restricoes = self.restricoes.limite
	local str_erro = 'Restries de limite ambguas: '
	
	--Se foi especificada a restrio '==', ela deve ser a nica restrio
	--especificada
	if 	tabela_restricoes['=='] and 
		(tabela_restricoes['<='] or tabela_restricoes['=>'] or
		tabela_restricoes['<'] or tabela_restricoes['>']) then 
			error(str_erro..'a restrio "==" deve ser nica.') 
	elseif 	tabela_restricoes['<'] and tabela_restricoes['<='] then
		error(str_erro.."escolha uma das restries '<' ou '<='.") 
	elseif  tabela_restricoes['>'] and tabela_restricoes['>='] then
		error(str_erro.."escolha uma das restries '>' ou '>='.")
	end
end

--------------------------------------------------------------------------------
function CampoSimples:vazio()
--------------------------------------------------------------------------------
-- Descrio:	Verifica se um campo simples foi preenchido ou no.
-- Retorno: 	retorna um valor diferente de nil, se o campo foi preenchido e 
--		nil, em caso contrrio.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	--print( 'CampoSimples:vazio()<BR>' )
	return self.valor == "" 
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoAlfabetico
--------------------------------------------------------------------------------
--print('Criao da classe CampoAlfabetico<BR>')
CampoAlfabetico = CampoSimples:subclass { numCampos = 0 }

--------------------------------------------------------------------------------
function CampoAlfabetico:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoAlfabetico.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--
-- Retorno: 	Retorna o objeto da classe CampoAlfabetico criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em CampoAlfabetico:new()<BR>')
   local p --Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'alfabtico'

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Para debugar
   --p:print()
   --print('Saindo de CampoAlfabetico:new()<BR>')
   return p
end


--------------------------------------------------------------------------------
function CampoAlfabetico:criticaTipo()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica se o valor do campo  do tipo 'alfabtico', ou seja,  
--		formado apenas por letras.
--		 
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o valor 
--		seja do tipo requerido.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	if 	strfind(gsub(self.valor, "%s", ""), "%A") then
		return 'tipo_invalido'
	end
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoAlfanumerico
--------------------------------------------------------------------------------
--print('Criao da classe CampoAlfanumerico<BR>')
CampoAlfanumerico = CampoSimples:subclass { numCampos = 0 }

--------------------------------------------------------------------------------
function CampoAlfanumerico:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoAlfanumerico.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--
-- Retorno: 	Retorna o objeto da classe CampoAlfanumerico criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em CampoAlfanumerico:new()<BR>')
   local p --Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'alfanumerico'

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self

   --Para debugar
   --p:print()
   --print('Saindo de CampoAlfanumerico:new()<BR>')
   return p
end


--------------------------------------------------------------------------------
function CampoAlfanumerico:criticaTipo()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica se o valor do campo  do tipo 'alfabtico', ou seja,  
--		formado apenas por letras e nmeros.
--		 
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o valor 
--		seja do tipo requerido.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	if 	not strfind(self.valor, "%w") then
		return 'tipo_invalido'
	end
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoCaracter
--------------------------------------------------------------------------------
--print('Criao da classe CampoCaracter<BR>')
CampoCaracter = CampoSimples:subclass { numCampos = 0 }


--------------------------------------------------------------------------------
function CampoCaracter:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoCaracter.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--
-- Retorno: 	Retorna o objeto da classe CampoCaracter criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em CampoCaracter:new()<BR>')
   local p --Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'caracter'

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Para debugar
   --p:print()
   --print('Saindo de CampoCaracter:new()<BR>')
   return p
end

--------------------------------------------------------------------------------
function CampoCaracter:criticaTipo()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica se o valor do campo  do tipo 'caracter', ou seja,  
--		formado por ao menos uma letra.
--		 
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o valor 
--		seja do tipo requerido.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	--print( self.identificador )
	if 	not strfind(self.valor, "%a") then
		return 'tipo_invalido'
	end
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoFormatado
--------------------------------------------------------------------------------
--print('Criao da classe CampoFormatado<BR>')
CampoFormatado = CampoSimples:subclass { 
	formato = nil, 	--string representando o padro a ser respeitado, com a
			--sintaxe do lua (opcional) 
	--formatoLua = nil --string representando o padro a ser respeitado com a
			--sintaxe do VALIDAFORM
			--9 qualquer nmero de 0 a 9
			--a qualquer caracter alfanumrico
			--l qualquer letra
			--x qualquer caracter
			
			--c onde c  um caracter, representa ele mesmo
			--para representar os prprios caracteres utilizados na 
			--sintaxe, (9, a, l, x) usar '%',
	numCampos = 3
}

--------------------------------------------------------------------------------
function CampoFormatado:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoFormatado.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--		o[6]: argumento a ser armazendado no  membro formato.
--		o[7]: argumento a ser armazendado no  membro exemplo.
--
-- Retorno: 	Retorna o objeto da classe CampoFormatado criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em CampoFormatado:new()<BR>')
   local p, ultimo_indice --Auxiliares para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'formatado'
 
   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Descobre  o ltimo ndice dos membros da superclasse
   ultimo_indice = self.parent.ultimoIndice
   
   --Atribuio dos membros especficos da classe
   p.formato 	= o[ultimo_indice+1]
	 p.formatoLn = o[ultimo_indice+2]	
	 p.exemplo = o[ultimo_indice+3]	

   --Para debugar
   --p:print()
   --print('Saindo de CampoFormatado:new()<BR>')
   return p
end

--------------------------------------------------------------------------------
function CampoFormatado:criticaTipo()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica se o valor do campo est no formato especificado.
--		 
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o valor 
--		esteja no formato requerido.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	--Verifica se o campo est vazio e se  obrigatrio
		--print(self.nome, self.obrigatorio, '-',self.valor,'-', '<BR>' )
	if 	self:vazio() then
		if 	self.obrigatorio then
			return 'campo_obrigatorio_vazio'
		else
			return nil
		end
	end
	if 	not strfind(self.valor, self.formato) then
		return 'tipo_invalido'
	end
end

--------------------------------------------------------------------------------
function CampoFormatado:formatoInterface()
--------------------------------------------------------------------------------
-- Descrio: 	Retorna uma string com o formato do campo, tal como deve ser 
--		apresentado ao usurio na mensagem de erro.
-- 		Exemplo: se o membro formato for "^%d%d%d%d%d%%-d%d%d$", o 
--		formato que deve aparecer na mensagem de erro  "99999-999".
-- Retorno: 	Retorna uma string com o formato do campo, tal como deve ser 
--		apresentado ao usurio na mensagem de erro.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	if 	self.formatoLn then
			return self.formatoLn
	end
	local lua_para_interface = { ['d'] = '9', ['a'] = 'X'}
	local retorno = self.formato
	local num_sub
	
	sintaxe_lua, sintaxe_interface = next( lua_para_interface, nil )
	while 	sintaxe_lua do
		retorno = gsub( retorno, "%%%%"..sintaxe_lua, sintaxe_lua ) 
		retorno = gsub( retorno, "%%"..sintaxe_lua, sintaxe_interface )
		sintaxe_lua, sintaxe_interface = 
		next( lua_para_interface, sintaxe_lua )
	end
	retorno, num_sub = gsub( retorno, '%^', '' )
	if 	num_sub == 0 then
		retorno = "... "..retorno
	end	

	--print( retorno )
	retorno, num_sub = gsub( retorno, '%$', '' )
	--print( retorno )
	--print( 'num_sub', num_sub )
	if 	num_sub == 0 then
		retorno = retorno..' ...'
	end	

	retorno = gsub( retorno, "%%", '' ) 	
	return retorno
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoNumerico
--------------------------------------------------------------------------------
--print('Criao da classe CampoNumerico<BR>')
CampoNumerico = CampoSimples:subclass { numCampos = 0 }
--------------------------------------------------------------------------------
function CampoNumerico:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoNumerico.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--
-- Retorno: 	Retorna o objeto da classe CampoNumerico criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em CampoNumerico:new()<BR>')
   local p --Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Para debugar
   --p:print()
   --print('Saindo de CampoNumerico:new()<BR>')
   return p
end


--------------------------------------------------------------------------------
function CampoNumerico:formataParaConsulta()
--------------------------------------------------------------------------------
-- Descrio: 	Formata o valor do campo para ser utilizado em um comando SQL. 
--		A formatao vai depender do atributo tipoBD do membro 
--		self.atributosChave. Se o tipo do campo no BD for string, a re-
--		presentao de seu valor na string da consulta SQL deve vir 
--		envolvida por " ' ",  para no violar a sintaxe.

-- Retorno: 	retorna uma string com o valor do campo formatado para ser 
--		utilizado em um comando SQL.
--------------------------------------------------------------------------------


	if 	self.atributosChave.tipoBD == 'string' then
		return cgilua.dblua_escape(self.valor)
	elseif self.atributosChave.tipoBD == 'nmero' then
		return gsub(self.valor, ",", ".")
	else
		error('Parmetro "atributoChave.tipoBD" invlido.')
	end
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoData
--------------------------------------------------------------------------------
--print('Criao da classe CampoData<BR>')
CampoData = CampoNumerico:subclass { interface = nil, numCampos = 1 }

--------------------------------------------------------------------------------
function CampoData:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoData.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--		o[6]: argumento a ser armazendado no  membro interface.
--
-- Retorno: 	Retorna o objeto da classe CampoData criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em CampoData:new()<BR>')
   local p --Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'data'

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self

   --Descobre  o ltimo ndice dos membros da superclasse
   ultimo_indice = self.parent.ultimoIndice
   
   
   --Atribuio dos campos especficos da classe
   p.interface = o[ultimo_indice + 1]

   --Para debugar
   --p:print()
   --print('Saindo de CampoData:new()<BR>')
   return p
end

--------------------------------------------------------------------------------
function CampoData:converteParaInteiro()
--------------------------------------------------------------------------------
-- Descrio:	Transforma o valor da data em um valor numrico, para que possa 
--		ser feita a comparao com os valores associados s restries 
--		de limite.
--		Exemplo: se  o membro valor  '18/18/1999', ele passa a ser 
--		'19991218'. 
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
 	local achou, lixo, dia, mes, ano = strfind(self.valor,"^(.-)/(.-)/(.*)")
 	self.valor = ano..mes..dia
end

--------------------------------------------------------------------------------
function CampoData:criticaInterfaceLista()
--------------------------------------------------------------------------------
-- Descrio:	Faz a crtica de um campo Data, cuja interface de entrada 
--		consiste de 3 listas. Neste caso, no so feitas crticas de 
--		formatao e de limites para o dia o ms e ano, pois estas j 
--		esto implcitas na interface. Sobram a verificao do 
--		preenchimento completo do campo e a validao de dias para meses 
--		especiais, como os que s possuem 30 dias ou fevereiro que, 
--		dependendo do ano, tem 28 ou 29 dias.

-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o 
--		preenchimento do campo esteja correto.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
 
	if 	strlen( self.valor ) ~= 8 then
		return 'data_incompleta'
	end
	self:separa()
	local erro, parametros = self:validaDiaMes()
  return erro, parametros
end

--------------------------------------------------------------------------------
function CampoData:criticaInterfaceTexto()
--------------------------------------------------------------------------------
-- Descrio: 	Faz a crtica de um campo Data, cuja interface de entrada  uma 
--		simples caixa de texto. Neste caso, so necessrias todas as 
--		crticas: verificao do preenchimento do campo, do formato 
--		(a data deve estar no formato 'dd/mm/aaaa'), de limites para o 
--		dia (1 <= dia <= 31) e ms (1 <= ms <= 12), alm da validao 
--		de dias para meses especiais, como os que s possuem 30 dias ou 
--		fevereiro que, dependendo do ano, tem 28 ou 29 dias.

-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o 
--		preenchimento do campo esteja correto.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	--Verifica se o formato da data est correto
	--print( 'data_texto', self.valor )
	if 	not strfind(self.valor, "^%d%d/%d%d/%d%d%d%d$") then
		return 'tipo_invalido'
	end
	
	self:converteParaInteiro()
	self:separa()
	--Verifica os limites do dia e do mes
	if 	(self.dia >= 1 and self.dia <= 31) then
      		if 	(self.mes >=1 and self.mes <=12) then
	    local erro, parametros = self:validaDiaMes()
      return erro, parametros
	  	else
	    		return 'mes_invalido'
	  	end
   	else
   	  	return 'dia_invalido' 
   	end
 	return nil
end

--------------------------------------------------------------------------------
function CampoData:criticaTipo()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica se o valor do campo  do tipo 'data'. Como so suporta-
--		das duas interfaces para a entrada de datas, a verificao dos
--		valores  feita de acordo com a interface associada ao objeto,
--		armazenada no membro self.interface.
--		 
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o valor 
--		seja do tipo requerido.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local erro, parametros 
	if 	self.interface == 'lista' then
      		erro, parametros = self:criticaInterfaceLista()
	elseif 	self.interface == 'texto' then
      		erro, parametros = self:criticaInterfaceTexto()
	end	
--	print( self.interface, parametros.numero_dias )
  	return erro, parametros
end

--------------------------------------------------------------------------------
function CampoSimples:limpa()
--------------------------------------------------------------------------------
-- Descrio: 	Limpa o valor do campo, deixando-o 'vazio'.
-- Autora: 	Gilda Dahis
-- Data: 	10/08/1999
--------------------------------------------------------------------------------
	if 	self.interface == 'lista' then
		local inicio, fim, sufixo = strfind(self.identificador, '_(..*)')
		cgi['dia_'..sufixo] = ""
		cgi['mes_'..sufixo] = ""
		cgi['ano_'..sufixo] = ""
	else
		cgi[self.identificador] = ""
	end
end

--------------------------------------------------------------------------------
function CampoData:separa()
--------------------------------------------------------------------------------
-- Descrio: 	A partir do membro self.valor, calcula os membros dia, mes e ano  
--		do campo. Esta separao  til para a validao dos valores do  
--		dia e do ms.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
 self.ano = tonumber(strsub (self.valor,1,4))
 self.mes = tonumber(strsub (self.valor,5,6))
 self.dia = tonumber(strsub (self.valor,7,8))
end

--------------------------------------------------------------------------------
function CampoData:validaDiaMes()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica a validade do dia da data em relao ao seu ms, 
--		considerando os meses especiais que s possuem 30 dias e feve-
--		reiro que, dependendo do ano, tem 28 ou 29 dias. Este mtodo   
--		chamado por ambos os mtodos de crtica associados s duas in-
--		terfaces disponveis para a entrada de datas.

-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o dia 
--		seja vlido em relao ao ms.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
 	local parametros
	if (self.mes==4 or self.mes==6 or self.mes==9 or self.mes==11) and (self.dia > 30) then
		return 'dia_maior_que_30'
        elseif self.mes==2 then 
        	parametros = {}
        	if 	mod(self.ano, 4)~= 0 then
        		parametros.numero_dias = 28
        	else
        		parametros.numero_dias = 29
        	end	
        		
	 	if 	self.dia > parametros.numero_dias then
	  		return 'dia_fevereiro_acima_limite', parametros 
		end
	end
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoInteiro
--------------------------------------------------------------------------------
--print('Criao da classe CampoInteiro<BR>')
CampoInteiro = CampoNumerico:subclass { numCampos = 0 }

--------------------------------------------------------------------------------
function CampoInteiro:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoInteiro.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--
-- Retorno: 	Retorna o objeto da classe CampoInteiro criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em CampoInteiro:new()<BR>')
   local p--Auxiliar para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'inteiro'

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Para debugar
   --p:print()
   --print('Saindo de CampoInteiro:new()<BR>')
   return p
end


--------------------------------------------------------------------------------
function CampoInteiro:criticaTipo()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica se o valor do campo  um nmero inteiro.
--		 
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o valor 
--		seja do tipo requerido.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	if not (  
		--verifica se o nmero  da forma "999..."
		strfind( self.valor, "^%d%d*$" ) or 
	
		--verifica se o nmero  da forma "-999..."
		strfind(self.valor, "^-%d%d*$") or 
	
		--verifica se o nmero  da forma "+999..."
		strfind(self.valor, "^+%d%d*$") 	   ) then
		
		return 'tipo_invalido'
	end
end

--------------------------------------------------------------------------------
-- Definio da Classe CampoReal
--------------------------------------------------------------------------------
--print('Criao da classe CampoReal<BR>')
CampoReal = CampoNumerico:subclass { numCampos = 0 }

--------------------------------------------------------------------------------
function CampoReal:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe CampoReal.
--
--
-- Parmetros:	o: tabela lua com os parametros para a criao do objeto, onde:
--		o[1]: argumento a ser armazenado no membro nome.
--		o[2]: argumento a ser armazenado no membro obrigatorio.	
--		o[3]: argumento a ser armazendado no  membro restricoes.
--		o[4]: argumento a ser armazendado no  membro atributosChave.
--		o[5]: argumento a ser armazendado no  membro valor.
--
-- Retorno: 	Retorna o objeto da classe CampoReal criado.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
   --print('Entrando em CampoReal:new()<BR>')
   local p --Auxiliares para a criao do objeto
   --Cria o objeto como objeto da SuperClasse
   p = self.parent:new (o, self)
   p.tipo = 'real'

   --O pai do objeto  um ponteiro para sua classe
   p.parent 	= self
   
   --Para debugar
   --p:print()
   --print('Saindo de CampoReal:new()<BR>')
   return p
end


--------------------------------------------------------------------------------
function CampoReal:criticaTipo()
--------------------------------------------------------------------------------
-- Descrio: 	Verifica se o valor do campo  um nmero real.
--		 
-- Retorno: 	retorna o identificador do erro encontrado, ou nil caso o valor 
--		seja do tipo requerido.
-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	local inteiroAux = CampoInteiro:new({self.nome, self.identificador, 
				self.obrigatorio, nil, nil, self.valor} )
	if not  (
		--Verifica todos os formatos possiveis para numero inteiro
		not inteiroAux:criticaTipo() or
	 
		--Verifica os outros formatos possveis para nmero real,com "."
		--Formato: "999.99..."
		strfind(self.valor, "^%d%d*%.%d%d*$" ) or
	
		--Formato: "-999.99..."
		strfind(self.valor, "^-%d%d*%.%d%d*$") or 
	
		--Formato: "+999.99..."
		strfind(self.valor, "^+%d%d*%.%d%d*$") or
	
		--verifica os formatos com ","
		--Formato: "999,99..."
		strfind(self.valor, "^%d%d*%,%d%d*$" ) or

		--Formato: "-999,99..."
		strfind(self.valor, "^-%d%d*%,%d%d*$") or 
		
		--Formato: "+999,99..."
		strfind(self.valor, "^+%d%d*%,%d%d*$") ) then
		return 'tipo_invalido'
	end
end