----------------------------- formulario.lua -----------------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	formulario.lua
-- Descrio:  	Contm a definio e os mtodos da classe Formulario. 
--		Um objeto desta classe representa um formulrio html e contm 
--		uma tabela com os objetos que representam seus campos.
-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes: 
--     Quem      |   Quando   |   Motivo
-- --------------|------------|-------------------------------------------------
--  Gilda	  07/08/1999   Mudana na estrutura do membro campos, agora 
--				indexado por nmeros.
--  Gilda	  10/08/1999   Incluso do mtodo limpa().
--------------------------------------------------------------------------------

-- Definicao da Classe Formulario
--print('Criao da Classe Formulario<BR>')
Formulario = Object:subclass { 	campos = nil,	--tabela	
				numCampos = 1 }

--------------------------------------------------------------------------------
function Formulario:new (o)
--------------------------------------------------------------------------------
-- Descrio: 	Construtor dos objetos da classe Formulario.
-- Parmetros:	o: tabela com os parametros para a criao do objeto, onde:
--		o[1]: tabela onde cada ndice  o identificador de um 
--		widget no formulrio html e onde cada valor  um objeto 
--		representando o campo relacionado ao widget.
-- Retorno: 	Retorna o objeto da classe Formulario criado.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
   p = self.parent:new (o, self)
   p.parent 	= self
   p.campos 	= o[1]
   
   local i = 1
   local campo
   --Para cada campo do formulrio
   while p.campos[i] do
   	--Cria um ndice string, a partir do identificador do campo
   	campo = p.campos[i]
   	p.campos[ campo.identificador ] = campo
   	i = i + 1
   end
   p.numCampos = i-1
   return p
end

--------------------------------------------------------------------------------
function Formulario:critica ()
--------------------------------------------------------------------------------
-- Descrio: 	Critica os campos do forumlrio, gerando suas respectivas mensa-
--		gens de erro e uma mensagem geral que indica a ocorrncia de er-
--		ro no formulrio.
-- Retorno: 	Retorna nil caso nenhum erro seja encontrado ou uma mensagem ge- 
--		ral que indica a ocorrncia de erro no formulrio, em caso con-
--		trrio.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
-- Alterado em: 07/08/1999 -Mudana na estrutura da tabela campos. Agora  inde-
--		xada por nmeros e o identificador  um parmetro no construtor
--		de cada campo do formulrio
--------------------------------------------------------------------------------
   	local campos, i, id_erro, parametros, erro
	campos = self.campos
	i = 1
	while( campos[i] ) do
		--Critica o campo
		  --print('<BR>',campos[i].identificador)
		 id_erro, parametros = campos[i]:critica() 
		  --print('<BR>',campos[i].identificador,campos[i].valor,id_erro,'<BR>')

		--Cria as tags da mensagem do campo
		if 	id_erro then
			erro = 1
			cgi['msg_'..campos[i].identificador] = 
				campos[i]:geraMensagem(id_erro,	parametros)
		end
		i = i + 1
	end
	if 	erro then
		return self:msgErro()
	else
		return nil
	end
end

--------------------------------------------------------------------------------
function Formulario:insereCampo( campo, posicao )
--------------------------------------------------------------------------------
-- Descrio: 	Insere um campo no formulrio.
-- Parmetros:  campo: objeto que representa o campo a ser inserido no 
--		formulrio.
--		posicao: parmetro opcional. inteiro indicando a posio do 
--		campo no formulrio.
--		o campo  inserido exatamente na posio especificada em 
--		'posicao', empurrando os campos a partir desta posio para a 
--		frente: o campo que estava em 'posicao' passa para 'posicao'+1
--		e assim por diante. Se este parmetro no for especificado, o  
--		campo  inserido aps o ltimo campo do formulrio.
--		
--		
-- Autora: 	Gilda Dahis
-- Data: 	10/08/1999
--------------------------------------------------------------------------------
	if	posicao then
		--Empurra os campos para a frente
		local p = self.numCampos
		while 	p >= posicao do
			self.campos[p+1] = self.campos[p] 
			p = p - 1
		end
	else
		posicao = self.numCampos+1
	end		

	self.campos[ posicao ] = campo	
   	self.campos[ campo.identificador ] = campo
	self.numCampos = self.numCampos+1
end

--------------------------------------------------------------------------------
function Formulario:deletaCampo( identificador )
--------------------------------------------------------------------------------
-- Descrio: 	Limpa o valor dos campos do formulrio, deixandos 'vazios'.
-- Parmetros:  campo: objeto que representa o campo a ser inserido no 
--		formulrio.
--		posicao: parmetro opcional. inteiro indicando a posio do 
--		campo no formulrio.
--		o campo  inserido exatamente na posio especificada em 
--		'posicao', empurrando os campos a partir desta posio para a 
--		frente: o campo que estava em 'posicao' passa para 'posicao'+1
--		e assim por diante. Se este parmetro no for especificado, o  
--		campo  inserido aps o ltimo campo do formulrio.
--		
--		
-- Autora: 	Gilda Dahis
-- Data: 	10/08/1999
--------------------------------------------------------------------------------
  --cgilua.htmlheader()
  --print( identificador, ':', self.campos[identificador], '<br>' ) 
  
  --Encontra a posicao do campo a ser deletado
  local campos, pos
	campos = self.campos
	local pos = 0
	local achei = nil
	while( not achei and pos < self.numCampos ) do
		pos = pos + 1 
		if  campos[pos].identificador == identificador then
		    achei = 1
		end
	end

  if  not achei then
      error('Tentativa de deletar campo inexistente no formulrio: '..cgilua.tostring(identificador))
  end

	--Deleta o campo 
	while campos[pos] do
			campos[pos] = campos[pos+1] 
			pos = pos+1
	end
  --print( self.campos[identificador], '<br>' ) 
end

--------------------------------------------------------------------------------
function Formulario:limpa()
--------------------------------------------------------------------------------
-- Descrio: 	Limpa o valor dos campos do formulrio, deixandos 'vazios'.
-- Autora: 	Gilda Dahis
-- Data: 	10/08/1999
--------------------------------------------------------------------------------
   	local campos, i
	campos = self.campos
	i = 1
	while( campos[i] ) do
		--limpa o campo	
		campos[i]:limpa()
		i = i + 1 
	end
end
