------------------------------- funcoes_geracao.lua ---------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	funcoes_geracao.lua
-- Descrio:  	Contm mtodos e estruturas das subclasses da classe Campo, 
--		que retornam subtextos em linguagem natural, os quais so 
--		componentes comuns as mensagens de diagnstico e de ao 
--		corretiva.

-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes:
--     Quem    |   Quando   |   Motivo
-- ------------|------------|---------------------------------------------------
--   
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--Carregamento de funes de outros mdulos
--------------------------------------------------------------------------------
--Funes lingsticas para a gerao de texto em portugus.
dofile( 'funcoes_linguisticas.lua')

--Funes lingsticas para a gerao de texto em portugus.
dofile( 'funcoes_estilisticas.lua')


--Funes que realizam transformaes em sentenas ou subsentenas bsicas, como 
--coordenaes, etc.
dofile( 'regras_transformacao.lua' )

--------------------------------------------------------------------------------
--Mtodos e estruturas utilizados pelos mtodos geradores de mensagens de erro
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Classe Campo
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function Campo:mesmaMsg( param1, campo2, param2 )
--------------------------------------------------------------------------------
-- Descrio: 	Compara a mensagem de erro associada ao campo com a associada a 
--		um outro campo. 

-- Parmetros:	param1: tabela com os parmetros para a gerao da mensagem de
--		erro associada ao campo. 
--
--		campo2: objeto que representa o campo cuja mensagem associada  
--		ser comparada com a do campo. 
--
--		param2: tabela com os parmetros para a gerao da mensagem de 
--		erro associada ao objeto campo2. 
--
-- Retorno:	Retorna 1 se as mensagens associadas aos dois campos so iguais 
--		e nil em caso contrrio.
--------------------------------------------------------------------------------
	if 	param1.erro ~= param2.erro then
		return nil
	end
	------------------------------------------------------------------------
	if 	param1.erro == 'campo_obrigatorio_vazio' then 
	------------------------------------------------------------------------
		return 1
	------------------------------------------------------------------------
	elseif param1.erro == 'tipo_invalido' then 
	------------------------------------------------------------------------
		if 	self.tipo == campo2.tipo then
			if 	self.tipo == 'formatado' then
				if 	self.formato == campo2.formato then
					return 1
				else
					return nil
				end
			else
				return 1		
			end
		else
			return nil
		end
	------------------------------------------------------------------------
	elseif	param1.erro == 'limite_ultrapassado' then
	------------------------------------------------------------------------
		if 	self.tipo ~= campo2.tipo then
			return nil
		else 
			return 	param1.operador == param2.operador and
				self:operandoLn(param1.operador) ==
				campo2:operandoLn(param1.operador)
		end		
	------------------------------------------------------------------------
	elseif	param1.erro == 'restricao_tamanho_insatisfeita' then
	------------------------------------------------------------------------
		if 	self.tipo ~= campo2.tipo then
			return nil
		else 
			return 	param1.operador == param2.operador and
				self:operandoLn(param1.operador) ==
				campo2:operandoLn(param1.operador)
		end		
	------------------------------------------------------------------------
	elseif	param1.erro == 'valor_invalido' then
	------------------------------------------------------------------------
		--return self:mesmosValoresInvalidos(campo2)
	------------------------------------------------------------------------
	elseif	param1.erro == 'chave_ja_cadastrada' then
	------------------------------------------------------------------------
		-----------
	end
end

--------------------------------------------------------------------------------
-- Classe CampoIlimitado
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function CampoIlimitado:formataCoordenacao( coordenacao )
--------------------------------------------------------------------------------
-- Descrio: 	Dada uma string contendo valores uma lista de valores, formata a 
--		string para conter uma coordenao bem escrita em linguagem 
--		natural.  usada para formatar membros de objetos das subclasses
--		desta classe, que podem vir a ser utilizados nas mensagens de 
--		erro, mas cujos valores no esto com um formato esttico.
--		Por exemplo, a lista de valores invlaidos de um campo simples ou 
--		a lista de componentes obrigatorios de um campo estruturado.

--		 
-- Parmetros:	string contendo uma lista de valores. Se houver mais de um valor,
--		devem estar separados por virgulas.

-- Retorno:	Retorna a string 'coordenacao' modificada pelas seguintes 
--		transformaes:		
--		
--		- eliminacao de espaos anteriores a cada ','.
--		- fixacao de somente um espao depois de cada ','.
--		- transformacao da ltima ',' em 'e'.
--		Exemplo: 'valor1  ,  valor2, valo3' => 'valor1, valor2 e valor3'

-- Autora: 	Gilda Dahis
-- Data: 	05/01/1998
-- Alterado em:  
--------------------------------------------------------------------------------
	--Formata a coordenao para que os espaos anteriroes as virgulas sejam
	--eliminados e para que haja sempre um e somente um espao depois de 
	--cada ','
	--print( 'coordenacao = ', coordenacao, '<BR>' )
	local coordenacao, num_virgulas = gsub( coordenacao, "%,%s*", ", ")
	--print( "para que haja sempre um e somente um espao depois de cada ',
	--':", coordenacao,'<BR>' )
	
	--Substitui a ltima ',' por um 'e', caso exista ao menos uma vrgula 	
	
	local inicio, fim 
	local cont = 0
	
	fim = 1
	while( cont < num_virgulas ) do
		cont = cont  + 1
		inicio, fim = strfind( coordenacao, "%,%s", fim )
	end
	
	--Se havia mais de um elemento (foi encontrada pelo menos uma ','), 
	--substituir a ltima vrgula por 'e'
	if 	inicio then
		coordenacao = strsub( coordenacao, 1, inicio-1 ).." e "..
		strsub( coordenacao, fim )
	end	
	
	--print("Substitui a ltima ',' por um 'e': ", coordenacao, '<BR>' ) 	
	return coordenacao
end

--------------------------------------------------------------------------------
CampoIlimitado.operadorRestricaoTamanhoLn = 
--------------------------------------------------------------------------------
--Tabela que mapeia uma string indicando um operador de comparao para restri-
--es de tamanho ('<=', '=', '>=')  expresso deste operador em linguagem 
--natural.
--------------------------------------------------------------------------------
{
	['>='] = 'ao menos',
	['<='] = 'no mximo',
	['==']  = 'exatamente'
}

--------------------------------------------------------------------------------
-- Classe CampoSimples
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoSimples:detalhePreenchimento( genero )
--------------------------------------------------------------------------------
-- Descrio: 	Utilizada por funes geradoras de mensagens que descrevem er-
--		ros relacionados a uma restrio de tamanho insatisfeita por um
--		valor preenchido em um campo, descreve em linguagem natural o 
--		detalhamento do erro, que  informa qual o tamanho do valor
--		preenchido. 

-- Parmetros:	genero:	caracter indicando o tipo do elemeto bsico de preen-
--		chimento do campo (exemplo: letra - gnero feminino - 'a', 
--		nmero - genro masculino - 'o').

-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		informa o tamanho do valor preenchido pelo usurio.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	tonumber(self.tamanhoValor) > 0 then
		return "Voc preencheu "..
		 numero_ou_artigo( numero_preenchido, genero ).."."
	else
		return ""
	end
end

--------------------------------------------------------------------------------
function CampoSimples:formaPreenchimentoCompleto()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural o corpo de uma mensagem completa 
--		sobre como o campo deve ser preenchido, com informaes sobre 
--		o tipo e as restries de limite, tamanho e valores a serem sa-
--		tisfeitas pelos valores digitados.

-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o corpo de uma mensagem completa sobre como o campo
--		deve ser preenchido.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  --self:imprime()
 	local retorno = self:formaPreenchimento()
	
	if 	self.restricoes then
		if 	self.restricoes.limite and self.restricoes.tamanho then
			retorno = retorno.." "..self:restricoesLimiteLn().." e "..
			self:restricoesTamanhoLn()
		elseif self.restricoes.limite then
			retorno = retorno.." "..self:restricoesLimiteLn()
		elseif self.restricoes.tamanho then
 	    if 	self.detalheTipoLn then
 		      retorno = gsub(retorno, self.detalheTipoLn, "")
 	    end
			retorno = retorno.." "..self:restricoesTamanhoLn()
		end
		
		if 	self.restricoes.valoresInvalidos then
			retorno = retorno..". "..self:restricoesValoresInvalidosLn()
		end
	end
	return retorno
end

CampoSimples.ordem = 'crescente'

-------------------------------------------------------------------------------- 
function CampoSimples:operadorLn( operador, genero ) 
--------------------------------------------------------------------------------
-- Descrio: 	Dado um operador de comparao de uma restrio de limite e o 
--		gnero do operando da direita da comparacao, retorna a expresso 
--		que descreve o operador, em linguagem natural.

-- Parmetros:	operador: string indicando o operador de comparao de uma res-
--		trio de limite: (pode ser '<', '<=', '>' ou '>='.

--		genero: caracter indicando o gnero do operando da direita da
--		comparao.

-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o operador de comparao de uma restrio de limite. 

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	not operador then
		error( 'O parmetro "operador"  nil.')
	end

	if 	operador == '<' then
		return "antes de" 
	elseif operador == '<=' then
		return participio("localizar", genero).." at" 
	elseif operador == '>' then
		return "depois de"
	elseif operador == '>=' then
	 	return participio("localizar", genero).." a partir de"
	else
		error( '"'..tostring(operador)..
		'" no  um valor vlido para o parmetro "operador".')
	end
end 

--------------------------------------------------------------------------------
function CampoSimples:operandoLn( operador )
--------------------------------------------------------------------------------
-- Descrio: 	Dado um operador de comparao de uma restrio de limite do 
--		campo, retorna o operando da direita da restrio em linguagem 
--		natural, quando h esta descrio, especificada pelo atributo 
--		'nome' da tabela que representa a restrio, ou o valor deste 
--		operando quando o atributo nome  nil. 

-- Parmetros:	operador: string indicando o operador de comparao de uma res-
--		trio de limite (pode ser '<', '<=', '>' ou '>='.

-- Retorno: 	Retorna uma string com a expresso, que	descreve o operando da 
--		direita da restrio em linguagem natural, quando h esta 
--		descrio, especificada pelo atributo 'nome' da tabela que 
--		representa a restrio, ou o valor deste operando quando o 
--		atributo nome  nil.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	not operador then
		error( 'O parmetro "operador"  nil.')
	end
  	--Identifica o operando da restrio a ser utilizado para a descrio em 
  	--linguagem natrual: pode ser um valor ou um descritor de um valor
	local operando = self.restricoes.limite[operador].nome
  	if 	not operando then
		operando = self.restricoes.limite[operador].valor
  	end
  	return operando
end

--------------------------------------------------------------------------------
function CampoSimples:restricaoLimiteLn( operador )
--------------------------------------------------------------------------------
-- Descrio: 	Dada um operador de uma restrio de limite a ser satisfeita 
--		por um campo, da forma <valor do campo> <operador> <operando>, 
--		exemplo:
--		
--		total de avaliaes <= 15 
--	
--		descreve o operador e o operando da direita em linguagem 
--		natural. 
--
-- Parmetros:	operador: string indicando o operador de comparao da 
--		restrio (pode ser '<', '<=', '>' ou '>='.
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve uma restrio de limite. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  if 	not operador then
	error("O parmetro 'operador'  nil.")
  end

  local genero_nome = genero( self.nome )

	
  --Identifica o operando da restrio a ser utilizado para a descrio em lin-
  --guagem natrual: pode ser um valor ou um descritor de um valor
  local operando = self:operandoLn( operador )
  
  return self:operadorLn( operador, genero_nome )..' "'..operando..'"'
end

--------------------------------------------------------------------------------
function CampoSimples:restricoesLimiteLn()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural todas as restries de limite a 
--		serem satisfeitas pelos valoRes digitados no campo simples.
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve todas as restries de limite a serem satisfeitas pelos 
--		valores digitados no campo simples.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local restricoes = self.restricoes.limite
	local limite_inferior, limite_superior, retorno
	--Montar a descrio do limite inferior se houver
	if 	restricoes['>'] then
		limite_inferior = self:restricaoLimiteLn('>')
	elseif 	restricoes['>='] then
		limite_inferior = self:restricaoLimiteLn('>=')
	end

	--Montar a descrio do limite superior se houver
	if 	restricoes['<'] then
		limite_superior = self:restricaoLimiteLn('<')
	elseif 	restricoes['<='] then
		limite_superior = self:restricaoLimiteLn('<=')
	end
		
	
	if 	limite_inferior and limite_superior then
		return limite_inferior.." e "..limite_superior
	elseif limite_inferior then
		return limite_inferior 
	else
		return limite_superior
	end
end

--------------------------------------------------------------------------------
function CampoSimples:restricoesTamanhoLn()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural todas as restries de tamanho a 
--		serem satisfeitas pelos valores digitados no campo simples.
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve todas as restries de tamanho a serem satisfeitas 
--		pelos valores digitados no campo simples.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local restricoes = self.restricoes.tamanho
	local restricao_ln
	local limite_inferior, limite_superior, retorno
	local operando_ultima_restricao
	------------------------------------------------------------------------	
	--Se o campo possui a restrio "==" ento, deve ter um tamanho exato
	------------------------------------------------------------------------
	if 	restricoes['=='] then
		operando_ultima_restricao = restricoes['==']
		restricao_ln = self.operadorRestricaoTamanhoLn['=='].." "..
		operando_ultima_restricao
	else
	------------------------------------------------------------------------		
	--Em caso contrrio, analisar os limites de tamanho do campo (restries
	--'>=' e '<='
	------------------------------------------------------------------------
		limite_inferior = restricoes['>='] 
		limite_superior	= restricoes['<=']
			
		if 	limite_inferior and limite_superior then
			-------------------------------------------------------
			--No mnimo X e no mximo Y
			-------------------------------------------------------
			restricao_ln =  self.operadorRestricaoTamanhoLn['>=']..
			" "..limite_inferior.." e "..
			self.operadorRestricaoTamanhoLn['<='].." "..
			limite_superior
			operando_ultima_restricao = restricoes['<=']
		elseif limite_inferior then
			-------------------------------------------------------
			--No mnimo X
			-------------------------------------------------------
			restricao_ln =  self.operadorRestricaoTamanhoLn['>=']..
			" "..limite_inferior
			operando_ultima_restricao = restricoes['>=']
		else
			-------------------------------------------------------
			--No mximo X
			-------------------------------------------------------
			restricao_ln =  self.operadorRestricaoTamanhoLn['<=']..
			" "..limite_superior
			operando_ultima_restricao = restricoes['<=']
		end
	end
	
	return "com "..restricao_ln.." "..
	unifica_numero(self:tipoElementoBasico(),
		operando_ultima_restricao )
end

--------------------------------------------------------------------------------
function CampoSimples:restricoesValoresInvalidosLn()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural o conjunto de valores que no so
--		permitidos para o preenchimento do campo. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o conjunto de valores que no so permitidos para o 
--		preenchimento do campo.
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local coordenacao = 
		self:formataCoordenacao(self.restricoes.valoresInvalidos)
	return "No so permitidos os seguintes valores: "..coordenacao
end

--------------------------------------------------------------------------------
-- Classe CampoAlfabtico
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoAlfabetico:formaPreenchimento()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural a forma de preenchimento relaciona
--		da ao tipo de valor que deve ser preenchido em um campo da clas-
--		se CampoAlfabetico. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve a forma de preenchimento relacionada ao tipo de valor 
--		que deve ser preenchido em um campo da classe CampoAlfabetico. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
       	return "somente com letras"
 
end

CampoAlfabetico.ordem = 'alfabtica'

--------------------------------------------------------------------------------
function CampoAlfabetico:tipoElementoBasico()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural o elemento bsico do valor que 
--		deve ser preenchido em um campo da classe CampoAlfabetico. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o elemento bsico do valor que deve ser preenchido em 
--		um campo da classe CampoAlfabetico. 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	return "letra"
end

--------------------------------------------------------------------------------
-- Classe CampoAlfanumrico
--------------------------------------------------------------------------------
CampoAlfanumerico.detalheTipoLn = "ao menos uma letra ou nmero"

--------------------------------------------------------------------------------
function CampoAlfanumerico:formaPreenchimento()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural a forma de preenchimento relaciona
--		da ao tipo de valor que deve ser preenchido em um campo da clas-
--		se CampoAlfanumerico. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve a forma de preenchimento relacionada ao tipo de valor 
--		que deve ser preenchido em um campo da classe CampoAlfanumerico. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
       	return "com "..self.detalheTipoLn 
end

--------------------------------------------------------------------------------
function CampoAlfanumerico:tipoElementoBasico()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural o elemento bsico do valor que 
--		deve ser preenchido em um campo da classe CampoAlfanumerico. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o elemento bsico do valor que deve ser preenchido em 
--		um campo da classe CampoAlfanumerico. 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	return "caracter"
end

--------------------------------------------------------------------------------
-- Classe CampoCaracter
--------------------------------------------------------------------------------
CampoCaracter.detalheTipoLn	= "ao menos uma letra"

--------------------------------------------------------------------------------
function CampoCaracter:formaPreenchimento()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural a forma de preenchimento relaciona
--		da ao tipo de valor que deve ser preenchido em um campo da clas-
--		se CampoCaracter. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve a forma de preenchimento relacionada ao tipo de valor 
--		que deve ser preenchido em um campo da classe CampoCaracter. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
    	return "com "..self.detalheTipoLn
end

--------------------------------------------------------------------------------
function CampoCaracter:tipoElementoBasico()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural o elemento bsico do valor que 
--		deve ser preenchido em um campo da classe CampoCaracter. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o elemento bsico do valor que deve ser preenchido em 
--		um campo da classe CampoCaracter. 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  	return "caracter" 
end

--------------------------------------------------------------------------------
--Classe CampoFormatado
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoFormatado:formaPreenchimento()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural a forma de preenchimento relaciona
--		da ao tipo de valor que deve ser preenchido em um campo da clas-
--		se CampoFormatado. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve a forma de preenchimento relacionada ao tipo de valor 
--		que deve ser preenchido em um campo da classe CampoFormatado. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
-- Alterado em: 03/11/1999
-- para adicionar o exemplo na mensagem
--------------------------------------------------------------------------------
        local forma =  'usando o formato "'..self:formatoInterface()..'"'
				if 	self.exemplo then
						forma = forma..'. Exemplo: '..self.exemplo	
				end
				return forma
end


--------------------------------------------------------------------------------
-- Classe CampoNumrico
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoNumerico:operadorLn( operador, genero ) 
--------------------------------------------------------------------------------
-- Descrio: 	Dado um operador de comparao de uma restrio de limite e o 
--		gnero do operando da direita da comparacao, retorna a expresso 
--		que descreve o operador, em linguagem natural.

-- Parmetros:	operador: string indicando o operador de comparao de uma res-
--		trio de limite: (pode ser '<', '<=', '>' ou '>='.

--		genero: caracter indicando o gnero do operando da direita da
--		comparao.

-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o operador de comparao de uma restrio de limite. 

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
 	if 	operador == '>' then 
 		return "maior do que "..genero
 	elseif 	operador == '<' then 
 		return "menor do que "..genero
	elseif 	operador == '>=' then
		return "maior ou igual "..conjuncao('a', genero)
	elseif  operador == '<=' then 
		return "menor ou igual "..conjuncao('a', genero)
	end 
end 

--------------------------------------------------------------------------------
function CampoNumerico:restricaoLimiteLn( operador )
--------------------------------------------------------------------------------
-- Descrio: 	Dada um operador de uma restrio de limite a ser satisfeita 
--		por um campo, da forma <valor do campo> <operador> <operando>, 
--		exemplo:
--		
--		total de avaliaes <= 15 
--	
--		descreve o operador e o operando da direita em linguagem 
--		natural. 
--
-- Parmetros:	operador: string indicando o operador de comparao da 
--		restrio (pode ser '<', '<=', '>' ou '>='.
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve uma restrio de limite. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  if 	not operador then
	error("O parmetro 'operador'  nil.")
  end

  --Identifica o operando da restrio a ser utilizado para a descrio em lin-
  --guagem natrual: pode ser um valor ou um descritor de um valor
  local operando = self:operandoLn( operador )
  
  local genero_descritor
  -------------------------------------------------------------------
  --Se o descritor do valor da restrio  um nmero, o gnero  nulo
  -------------------------------------------------------------------
  if 	strfind( operando, "^%d%d*$" ) or 
	strfind( operando, "^%d%d*%.%d%d*$" ) then
 	genero_descritor = ""
  else
  -----------------------------------------------------------------------------	
  --Caso contrrio, o gnero  recuperado atravs do dicionrio. Se o descritor 
  --do valor  uma locuo, o gnero da locuo  o genro do ncleo. 
  --Para simplificar, pegaremos o gnero da primeira palavra da locuo
  ------------------------------------------------------------------------------	
  	inicio = strfind( operando, " " )
	
	if 	inicio then 
		genero_descritor = genero(strsub(operando,1,inicio-1))
	else
 		genero_descritor = genero(operando)
	end  
  end
  
  return self:operadorLn( operador, genero_descritor ).." "..operando
end


--------------------------------------------------------------------------------
-- Classe CampoData
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoData:formaPreenchimento()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural a forma de preenchimento relaciona
--		da ao tipo de valor que deve ser preenchido em um campo da clas-
--		se CampoData. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve a forma de preenchimento relacionada ao tipo de valor 
--		que deve ser preenchido em um campo da classe CampoData. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
        if  self.interface == 'texto' then
            return 'com uma data no formato "dd/mm/aaaa"'
        else
            return 'com uma data'        
        end
end

-------------------------------------------------------------------------------- 
function CampoData:operadorLn( operador, genero ) 
--------------------------------------------------------------------------------
-- Descrio: 	Dado um operador de comparao de uma restrio de limite e o 
--		gnero do operando da direita da comparacao, retorna a expresso 
--		que descreve o operador, em linguagem natural.

-- Parmetros:	operador: string indicando o operador de comparao de uma res-
--		trio de limite: (pode ser '<', '<=', '>' ou '>='.

--		genero: caracter indicando o gnero do operando da direita da
--		comparao.

-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o operador de comparao de uma restrio de limite. 

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	operador == '<' then
		return "anterior "..conjuncao('a', genero) 
	elseif operador == '<=' then
		return "anterior ou igual "..conjuncao('a', genero) 
	elseif operador == '>' then
		return "posterior "..conjuncao('a', genero)
	elseif operador == '>=' then
	 	return "posterior ou igual "..conjuncao('a', genero)
	end
end 

--------------------------------------------------------------------------------
function CampoData:operandoLn( operador )
--------------------------------------------------------------------------------
-- Descrio: 	Dado um operador de comparao de uma restrio de limite do 
--		campo Data, retorna o operando da direita da restrio em 
--		linguagem natural, quando h esta descrio, especificada pelo 
--		atributo 'nome' da tabela que representa a restrio, ou o valor 
--		deste operando quando o atributo nome  nil. 

-- Parmetros:	operador: string indicando o operador de comparao de uma res-
--		trio de limite (pode ser '<', '<=', '>' ou '>='.

-- Retorno: 	Retorna uma string com a expresso, que	descreve o operando da 
--		direita da restrio em linguagem natural, quando h esta 
--		descrio, especificada pelo atributo 'nome' da tabela que 
--		representa a restrio, ou o valor deste operando quando o 
--		atributo nome  nil.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	--BUG: No consigo chamar um mtodo da superclasse que tenha o mesmo nome!!!
	--local operando = self.parent:operandoLn( operador )
	--Se o operando  um valor, ele deve aparecer no formato "dd/mm/aaaa"
	if 	not operador then
		error( 'O parmetro "operador"  nil.')
	end
  	--Identifica o operando da restrio a ser utilizado para a descrio em 
  	--linguagem natrual: pode ser um valor ou um descritor de um valor
	local operando = self.restricoes.limite[operador].nome
  if 	not operando then
		  operando = self.restricoes.limite[operador].valor
 	end
--------------------------------------------------------------------------------

  	if 	strfind( operando, "^%d%d%d%d%d%d%d%d$" ) then
  		operando = self.formata()
  	end
	return operando
end

--------------------------------------------------------------------------------
-- Classe CampoInteiro
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoInteiro:formaPreenchimento()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural a forma de preenchimento relaciona
--		da ao tipo de valor que deve ser preenchido em um campo da clas-
--		se CampoInteiro. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve a forma de preenchimento relacionada ao tipo de valor 
--		que deve ser preenchido em um campo da classe CampoInteiro. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
        return "com um nmero inteiro"  --[{positivo, negativo}]
end

--------------------------------------------------------------------------------
function CampoInteiro:tipoElementoBasico()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural o elemento bsico do valor que 
--		deve ser preenchido em um campo da classe CampoInteiro. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o elemento bsico do valor que deve ser preenchido em 
--		um campo da classe CampoInteiro. 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return "nmero" 
end	

--------------------------------------------------------------------------------
-- Classe CampoReal
--------------------------------------------------------------------------------
CampoReal.detalheTipoLn		= 'usando "," ou "." como separador'  

--------------------------------------------------------------------------------
function CampoReal:formaPreenchimento()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural a forma de preenchimento relaciona
--		da ao tipo de valor que deve ser preenchido em um campo da clas-
--		se CampoReal. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve a forma de preenchimento relacionada ao tipo de valor 
--		que deve ser preenchido em um campo da classe CampoReal. 
--
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
        return 'com um nmero real '..self.detalheTipoLn --[{positivo, negativo}]
end

--------------------------------------------------------------------------------
function CampoReal:tipoElementoBasico()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural o elemento bsico do valor que 
--		deve ser preenchido em um campo da classe CampoReal. 
--
-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o elemento bsico do valor que deve ser preenchido em 
--		um campo da classe CampoReal. 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return "nmero" 
end	

--------------------------------------------------------------------------------
--Classe CampoEstruturado
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoEstruturado:coordenaNomeCampos( sufixo_tabela )
--------------------------------------------------------------------------------
-- Descrio: 	Este mtodo  utilizado para a gerao de mensagens globais que
--		fazem referncia aos componentes do campo estruturado onde ocor-
--		reram erros. Dado o sufixo da tabela que armazena os nomes dos
--		componentes associados a um determinado tipo de erro 
--		(componentes obrigatrios vazios ou componentes invlidos), 
--		retorna uma coordenao destes nomes.

-- Parmetros:	sufixo_tabela: string que indica o sufixo da tabela cujos nomes 
--		de campos devem ser coordenados.
--		sufixo		tabela
--		'Vazios' 	self.camposVazios
--		'Invalidos' 	self.camposInvalidos

-- Retorno: 	Retorna uma string com a coordenao dos nomes dos componentes 
--		armazenados na tabela especificada. 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local tabela = self['campos'..sufixo_tabela]
	local num_itens = self['cont'..sufixo_tabela]
	local i = 1
	local retorno 
	--print( num_itens )
	retorno = tabela[1].nome
	while 	i < num_itens-1 do
		--print( i )
		i = i + 1	
		retorno = retorno..", "..tabela[i].nome
	end
		
	if 	num_itens > 1 then
		retorno = retorno.." e "..tabela[num_itens].nome
	end
	return retorno
end

--------------------------------------------------------------------------------
-- Classe CampoComposto
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoComposto:coordenaNomesSubconjunto()
--------------------------------------------------------------------------------
-- Descrio: 	Este mtodo  utilizado para a gerao de mensagens que indicam
--		que as restries de tamanho do campo composto no foram sa-
--		tisfeitas. Gera uma coordenao com os nomes dos campos que
--		fazem parte do subconjunto de componentes relevantes para a ve-
--		rificao das restries de tamanho do campo (especificados no 
--		membro self.restricoes.subconjunto).

-- Retorno: 	Retorna uma coordenao com os nomes dos campos que fazem parte 
--		do subconjunto de componentes relevantes para a verificao das 
--		restries de tamanho do campo (especificados no membro 
--		self.restricoes.subconjunto). OBS: esta coordenao  obtida, 
--		substituindo-se os identificadores dos componentes contidos na 
--		string self.restricoes.subconjunto pelos seus nomes. 
 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	--Formata a coordenao para que haja sempre um e somente um espao depois de
	--cada ','

	local coordenacao = self:formataCoordenacao( self.restricoes.subconjunto )
	
	--Para cada componente
	local id, atributos = next( self.componentes, nil )
	while( id ) do
		--Se ele est contido no subconjunto 
		--Substitui o seu identificador, pelo seu nome
		if 	strfind( coordenacao, id ) then
				coordenacao = gsub( coordenacao, id, atributos.nome )
		end	
		id, atributos = next( self.componentes, id )
	end
	
	return coordenacao	
end

--------------------------------------------------------------------------------
function CampoComposto:objetosVerboImperativo(coordenacao, genero, quantidade)
--------------------------------------------------------------------------------
-- Descrio: 	Este mtodo  utilizado para a gerao de mensagens do tipo ao
--		corretiva que fazem referncia a componentes do campo composto.
--		Nestas mensagens o verbo principal est no imperativo e a
--		coordenao dos componentes  o ncleo de um objeto direto, 
--		em contraste com o a mensagem de diagnstico, onde o verbo est 
--		na passiva e a coordenao  o ncleo do complemento nominal. 
--		Exemplo:AC: 	Prencha os campos: campo1 e campo2
--						X
--			D:	 obrigatrio o preenchimento dos campos: 
--				campo1 e campo2	
--		Assim, este mtdodo gera o objeto direto que contm a 
--		coordenao, a ser concatenado com o verbo imperativo.

-- Parmetros:	coordenacao:sting contendo uma coordenao de componentes do  
--		campo composto.
--		genero: caracter indicando o genero do membro 
--		self.nomeComponente. (a - feminino, o- masculino)
--		quantidade:inteiro indicando o nmero de componentes na coorde
--		coordenao.

-- Retorno: 	Retorna o objeto direto que contm uma coordenao de nomes de 
--		componentes do campo composto. 
 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	quantidade == 1 then
		return genero..' '..self.nomeComponente.." "..
		coordenacao
	else
		return 	genero..'s '..plural(self.nomeComponente)..": "..
		coordenacao 
	end
end

--------------------------------------------------------------------------------
function CampoComposto:formaPreenchimentoCompleto()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural o corpo de uma mensagem completa 
--		sobre como o campo deve ser preenchido, com informaes sobre 
--		o conjunto de componentes que deve ser preenchido e sobre a for
--		ma de preenchimento de cada componente.

-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o corpo de uma mensagem completa sobre como o campo
--		deve ser preenchido.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local retorno = 'da seguinte forma:<BR>\n'
	local id, componente = next(self.componentes,nil)
	
	while 	id do
		retorno = retorno..componente.nome..": "..
		componente:formaPreenchimentoCompleto()..'.<BR>\n'	
		id, componente = next(self.componentes,id)
	end
	return retorno
end

--------------------------------------------------------------------------------
function CampoComposto:preparaCoordenacaoCamposInvalidos( configuracao )
--------------------------------------------------------------------------------
-- Descrio: 	A partir do membro self.camposInvalidos, que contm informaes
--		para a gerao das mensagens de erros dos componentes que foram 
--		preenchidos de forma invlida, cria uma tabela de submensagens
--		que agrupam os campos que possuem mensagens de erro idnticas. 
--		Exemplo:
--		Dadas as seguintes mensagens de componentes:

--		msg do campo data: 'com uma data no formato "dd/mm/aaaa"'
--		msg do campo hora: 'com um valor maior do que 0'
--		msg do campo minuto: 'com um valor maior do que 0'

--		A tabela criada seria:

--		{ 	"o campo data com uma data no formato "dd/mm/aaaa",
--			"os campos hora e minuto com um valor maior do que 0"
--		}

-- Retorno:	retorna uma tabela de strings contendo submensagens que agrupam 
--		os campos que possuem mensagens de erro idnticas e o nmero de 
--		itens da tabela.

-- Autora: 	Gilda Dahis
-- Data:	07/06/1998
-- Alterado por: 
--------------------------------------------------------------------------------
	local inicio_msg
	if 	configuracao == 'C' then
		--O pedao da mensagem a ser substitudo  
		inicio_msg = "Preencha o campo "
	elseif	configuracao == 'D' then
		inicio_msg = "O campo deve ser preenchido "
	else
		error( tostring( configuracao )..
		' no  um valor vlido para o parmetro configurao.')
	end
	
	cgi.configuracao = configuracao
	chdir(CAMINHOS[strlower(cgilua.script_vdir)])
	dofile( 'configura_msgs.lua' )
	chdir(cgilua.script_pdir)

	--Inicializa a lista de parametros para a geracao de formas de 
	--preenchimento com o primeiro campo invlido
	local indice, parametros, incluiu
	
 	local campo_invalido = self.camposInvalidos[1]
	local i 
	local campo_aux
	
	local coordenadas = 
	{ { 
			objeto = campo_invalido.componente,
			campos = campo_invalido.nome,
			numCampos = 1, 
			erro   = campo_invalido.erro,
			parametros = campo_invalido.parametros 
	} }
	
	--Para cada campo invlido, a partir do prximo, inluir seu
	--nome associado a uma forma de preenchimento na tabela. 
	--Caso sua forma seja indita,  criada uma entrada para ela 	
	
	local campos = self.camposInvalidos
	i = 2
	while 	campos[i] do
		--Para cada forma de preenchimento ja includa, compar-la com
		--a do campo corrente
		
		incluiu = nil
		indice = 1
		while 	coordenadas[indice] and not incluiu do
			campo_aux = coordenadas[indice]
			if 	campo_aux.objeto:mesmaMsg(campo_aux, 
				self.componentes[i], campos[i]) then
				coordenadas[indice].campos = campo_aux.campos..
				", "..campos[i].nome 
				coordenadas[indice].numCampos = coordenadas[indice].numCampos+1
				incluiu = 1
			else
				indice = indice + 1
			end
		end
		
		if 	not incluiu then
			coordenadas[indice] = 
			{ 	
				objeto	= campos[i].componente,
				campos  = campos[i].nome, 
				numCampos = 1,
				erro 	= campos[i].erro,
				parametros = campos[i].parametros  }
			--print( '<BR>', indice, coordenadas[indice].campos )		
		end
		i = i + 1
	end
	
	--Se a configurao  de diagnstico, a primeira coordenada  
	--especial, ela fixa o verbo 'deve ser preenchido'
	if 	configuracao == 'D' then
		local genero_componente = genero( self.nomeComponente )
		local num_campos = coordenadas[1].numCampos
	
		local numero_sujeito = numero( num_campos )

		coordenadas[1].campos = 
		primeira_maiuscula(unifica_numero(genero_componente, 
		num_campos)).." "..
		unifica_numero(self.nomeComponente, 
		num_campos).." "..
		coordenadas[1].campos.." "..
		conjugacao('dever', '3'..numero_sujeito, genero_componente,
		'presente', 'ativa').." "..
		conjugacao('preencher', '3'..numero_sujeito, 
		genero_componente, 'infinitivo impessoal', 'passiva', 'ser')
	end

	local msg_tipo
	indice = 1
	while 	coordenadas[indice] do
		--print( "<BR>", indice, 'campos: ', coordenadas[ indice ].campos )
		campo_aux = coordenadas[ indice ]
		msg_tipo = gsub(campo_aux.objeto:msgErro( campo_aux.erro, campo_aux.parametros),
		inicio_msg, "" )
		msg_tipo = strsub( msg_tipo, 1,-2)
		coordenadas[ indice ] = self:formataCoordenacao(coordenadas[ indice ].campos)..
		" "..msg_tipo
		--print( '<BR>coordenadas['..indice..']: ', coordenadas[ indice ] )
		indice = indice + 1
	end	
	--error( cgilua.tostring( coordenadas ) )
	return coordenadas, indice-1
end

--------------------------------------------------------------------------------
--Classe CampoVetor
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoVetor:coordenaNomesSubconjunto()
--------------------------------------------------------------------------------
-- Descrio: 	Este mtodo  utilizado para a gerao de mensagens que indicam
--		que as restries de tamanho do campo vetor no foram sa-
--		tisfeitas. Gera uma coordenao com os nomes dos campos que
--		fazem parte do subconjunto de elementos relevantes para a ve-
--		rificao das restries de tamanho do campo (especificados no 
--		membro self.restricoes.subconjunto).

-- Retorno: 	Retorna uma coordenao com os nomes dos campos que fazem parte 
--		do subconjunto de componentes relevantes para a verificao das 
--		restries de tamanho do campo (especificados no membro 
--		self.restricoes.subconjunto). OBS: esta coordenao  obtida, 
--		substituindo-se os identificadores dos elementos contidos na 
--		string self.restricoes.subconjunto pelos nmeros ordinais que 
--		representam seus ndices. 
 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local genero_elemento = genero(self.nomeComponente)
	--Substitui a expresso 'nomeComponenteI' por I-sim(o/a)
	return
	gsub(self:formataCoordenacao(self.restricoes.subconjunto),
	"([,]?%s*)"..self.nomeComponente.."(%d%d*)(%s*[,]?)", 
	"%1%2"..ordinal(genero_elemento).."%3")
end

--------------------------------------------------------------------------------
function CampoVetor:formaPreenchimentoCompleto()
--------------------------------------------------------------------------------
-- Descrio: 	Descreve em linguagem natural o corpo de uma mensagem completa 
--		sobre como o campo deve ser preenchido, com informaes sobre 
--		o conjunto de elementos que deve ser preenchido e sobre a for
--		ma de preenchimento dos elementos.

-- Retorno: 	Retorna uma string com a expresso em linguagem natural, que 
--		descreve o corpo de uma mensagem completa sobre como o campo
--		deve ser preenchido.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	return "<BR>Cada "..self.nomeComponente.." deve ser preenchid"..
	genero(self.nomeComponente)..self.componentes:formaPreenchimentoCompleto().."." 	
end


--------------------------------------------------------------------------------
function CampoVetor:objetosVerboImperativo(coordenacao, genero, quantidade)
--------------------------------------------------------------------------------
-- Descrio: 	Este mtodo  utilizado para a gerao de mensagens do tipo ao
--		corretiva que fazem referncia a elementos do campo vetor.
--		Nestas mensagens o verbo principal est no imperativo e a
--		coordenao dos componentes  o ncleo de um objeto direto, 
--		em contraste com o a mensagem de diagnstico, onde o verbo est 
--		na passiva e a coordenao  o ncleo do complemento nominal. 
--		Exemplo:AC: 	Prencha a 1a, a 2a, a 3a e a 7a fase.
--						X
--			D:	 obrigatrio o preenchimento das fases: 
--				1a, 2a, 3a e 7a.	
--		Assim, este mtdodo gera o objeto direto que contm a 
--		coordenao, a ser concatenado com o verbo imperativo.

-- Parmetros:	coordenacao:sting contendo uma coordenao de componentes do  
--		campo composto.
--		genero: caracter indicando o genero do membro 
--		self.nomeComponente. (a - feminino, o- masculino)
--		quantidade:inteiro indicando o nmero de componentes na coorde
--		coordenao.

-- Retorno: 	Retorna o objeto direto que contm uma coordenao de ndices
--		de elementos do vetor. 
 
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
		return gsub(coordenacao,"(%d%d*)", "a %1").. 
	' '..self.nomeComponente
end