------------------------- funcoes_linguisticas.lua -----------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	funcoes_linguisticas.lua
-- Descrio:  	Biblioteca com funes lingsticas para a gerao de texto em 
--		portugus. Est em desenvolvimento, contendo apenas as funes 
--		utilizadas pelos mtodos geradores de erro do VALIDAFORM. 
--		Algumas funes so simples mapeamentos e por isso foram 
--		implementadas como tabelas, exemplo:
--		artigo_indefinido = { o = "um", a = "uma" }.
		 
-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes:
--     Quem    |   Quando   |   Motivo
-- ------------|------------|---------------------------------------------------
--   
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
--Carregamento do dicionrio de palavras, que contm
--informaes morfolgicas sobre palavras em portugus
--------------------------------------------------------------------------------
dofile( 'dicionario.lua' )

-------------------------------------------------------------------------------
artigo_indefinido = { o = "um", a = "uma" }
-------------------------------------------------------------------------------
-- Tabela que mapeia um caracter representando um gnero (o - masculino, 
-- a - feminino) a uma string representando um artigo indefinido.
-------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function conjugacao( verbo, pessoa, genero, tempo, voz, verbo_auxiliar, tempo_principal )
--------------------------------------------------------------------------------
-- Descrio: 	Dado um verbo e informaes para a sua conjugao, retorna
--		a conjugao adequada.
-- Parmetros:	verbo: string indicando o verbo a ser conjugado.
--		pessoa:string indicando a pessoa para a conjugao.
--		genero:string indicando o genro da pessoa (necessrio quando a 
--		conjugao  de voz ativa ou o tempo  particpio.
--		tempo: string indicando o tempo para a conjugao.
--		voz: string indicando a voz para a conjugao ('ativa'/'passiva').
--		verbo_auxiliar: string indicando o verbo auxiliar, caso a conju-
--		gao seja na voz passiva.
--    tempo_principal: tempo do verbo auxiliar, por enqto pode ser particpio ou 
--    gerndio

-- Retorno: 	Retorna uma string contendo a conjugao de um verbo de acordo
--		com os parmetros para conjugao: pessoa, genero da pessoa, 
--		tempo, voz e verbo auxiliar e incluso do gerndio.	
	
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
-- Atualizado em: 16/02/1999 para incluir o parmetro tempo auxiliar
--------------------------------------------------------------------------------
	if 	voz == 'passiva' then
	  
		if  not tempo_principal then
		    tempo_principal = 'particpio'
		end
		return conjugacao( verbo_auxiliar, pessoa, genero, tempo, 'ativa' ).." "..
		conjugacao( verbo, pessoa, genero, tempo_principal, 'ativa' )
	elseif 	voz == 'ativa' then
		---------------------------------------------------------------
		if 	tempo == 'infinitivo impessoal' then
		---------------------------------------------------------------
			return verbo
		---------------------------------------------------------------
		elseif tempo == 'particpio' then
		---------------------------------------------------------------
			local retorno =  participio( verbo, genero )
			--Se a pessoa est no plural
			if 	strfind(pessoa, 'p$')  then 
				retorno = retorno.."s"
			end
			return retorno
		---------------------------------------------------------------
		elseif tempo == 'gerndio' then
		---------------------------------------------------------------
			return gerundio(verbo)
		---------------------------------------------------------------
		elseif tempo == 'presente' then
		---------------------------------------------------------------
			return presente( verbo, pessoa )
		---------------------------------------------------------------
		elseif tempo == 'preterito_perfeito' then
		---------------------------------------------------------------
			return preterito_perfeito( verbo, pessoa )
		end
	else
		error( "Parmetro 'voz' invlido." )  
	end
end

--conjuncao_a = { nulo = "a", a = "", o = "ao" }

-------------------------------------------------------------------------------
function conjuncao( preposicao, artigo )
--------------------------------------------------------------------------------
-- Descrio: 	Dada uma preposicao e um artigo, retorna a conjuno dos dois.

-- Parmetros:	
--		preposicao: 	string indicando uma preposicao.
--		artigo:		string indicando um gnero. Pode ser ''.

-- Retorno: 	Retorna a conjuno de uma preposio com um artigo. Caso algum
--		parmetro seja invlido, finaliza a execuo e exibe uma mensa- 
--		gem de erro.
		
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	artigo == '' then
		return preposicao
	end
	-------------------------------
	if 	preposicao == 'a' then
	-------------------------------
		if artigo == 'a' then
			return ''
		elseif artigo == 'o' then
			return 'ao'
		else
			error('Artigo invlido para esta preposio.')
		end	
	--------------------------------
	elseif preposicao == 'de' then
	--------------------------------
		if artigo == 'a' or artigo == 'o' or artigo == 'um' or artigo == 'uma' then
			return 'd'..artigo
		else
			error('Artigo invlido.')
		end	
	--------------------------------
	elseif	preposicao == 'em' then
	--------------------------------
		if artigo == 'a' or artigo == 'o' or artigo == 'um' or artigo == 'uma' then
			return 'n'..artigo
		else
			error('Artigo invlido.')
		end	
	else
		error('Preposio invlida.')
	end
end

--------------------------------------------------------------------------------
function feminino( palavra )
--------------------------------------------------------------------------------
-- Descrio: 	Dada uma palavra, retorna a mesma no gnero feminino.
-- Parmetros:	
--		palavra: string indicando a palavra a ser transformada para
--		o feminino.

-- Retorno: 	Retorna uma string contendo o feminino da palavra.		
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if not palavra then
		error( "O parmetro 'palavra'  nulo.")
	end

	--print(strlower(palavra))
	
	local retorno = dicionario[strlower(palavra)]  
	if 	not retorno then
		error('A palavra '..palavra..'no consta do dicionrio.')	
	else	
		--Verificar se a palavra  uma excesso a regra para a deterina
		--o do feminino
		if 	retorno.morfologia.excecoes and  
			retorno.morfologia.excecoes.feminino then
			return retorno.morfologia.excecoes.feminino
		--Caso no seja, aplica-se a regra geral
		else
			return strsub(palavra, 1, -2).."a" 
		end
	end
end

--------------------------------------------------------------------------------
function genero( palavra )
--------------------------------------------------------------------------------
-- Descrio: 	Dada uma palavra, retorna seu gnero, atravs de uma consulta ao
--		dicionrio de palavras.
-- Parmetros:	
--		palavra: string indicando a palavra a ter o gnero identificado.

-- Retorno: 	Retorna um caracter com o identificador do gnero da palavra
--		('o'- masculino e 'a'- feminino). 
		
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if not palavra then
		error( "O parmetro 'palavra'  nulo.")
	end

  -----------------------------------------------------------------------------	
  --Se a palavra  uma locuo, o gnero da locuo  o genro do ncleo. 
  --Para simplificar, pegaremos o gnero da primeira palavra da locuo
  ------------------------------------------------------------------------------	
  local inicio = strfind( palavra, " " )
	
	if 	inicio then 
		palavra = strsub(palavra,1,inicio-1)
	end  
		
	local retorno = dicionario[strlower(palavra)]

	if 	not retorno then
		error('A palavra "'..tostring(strlower(palavra))..
		'"no est no dicionrio.')	
	end
	
	retorno = retorno.morfologia

	if 	not retorno then
		error('No foi definida a morfologia da palavra "'..tostring(strlower(palavra))..
		'"no dicionrio.')	
	end
	
	retorno = retorno.genero 
	if 	not retorno then
		error('No foi definido o gnero da palavra "'..
		tostring(strlower(palavra))..'"')	
	else	
		return retorno 
	end
end

--------------------------------------------------------------------------------
function gerundio( verbo )
--------------------------------------------------------------------------------
-- Descrio: 	Dado um verbo, retorna a conjugao do verbo no 
--		gerndio.
-- Parmetros:	verbo: string indicando o verbo a ser conjugado.

-- Retorno: 	Retorna uma string contendo a conjugao de um verbo no gerndio.
	
-- Autora: 	Gilda Dahis
-- Data: 	16/02/2000
--------------------------------------------------------------------------------
	if 	not dicionario[verbo] then
		error( 'O verbo "'..tostring( verbo )..
		'" no consta do dicionrio.')
	end
	
	if 	dicionario[verbo].classe ~= 'verbo' then
		error( 'A classe de "'..tostring( verbo )..'" no dicionrio  "'
		..tostring(dicionario[verbo].classe)..'".')
	end
	
	if 	dicionario[verbo].excecoes then
		if 	dicionario[verbo].excecoes.participio then
			return dicionario[verbo].excecoes.participio
		end
	end
	
	local inicio, fim, radical, vogal_tematica = strfind(verbo,"^(%a*)(%a)%r$")
	--print( radical, final )
	return radical..vogal_tematica..'ndo'
end

--------------------------------------------------------------------------------
function masculino( palavra )
--------------------------------------------------------------------------------
-- Descrio: 	Dada uma palavra, retorna a mesma no gnero masculino.
-- Parmetros:	
--		palavra: string indicando a palavra a ser transformada para
--		o masculino.

-- Retorno: 	Retorna uma string contendo o masculino da palavra.		
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if not palavra then
		error( "O parmetro 'palavra'  nulo.")
	end

	--print(strlower(palavra))
	
	local retorno = dicionario[strlower(palavra)]  
	if 	not retorno then
		error('A palavra '..palavra..'no consta do dicionrio.')	
	else	
		--Verificar se a palavra  uma excesso a regra para a deterina
		--o do feminino
		if 	retorno.morfologia.excecoes and  
			retorno.morfologia.excecoes.masculino then
			return retorno.morfologia.excecoes.masculino
		--Caso no seja, aplica-se a regra geral
		else
			return strsub(palavra, 1,-2).."o" 
		end
	end
end

--------------------------------------------------------------------------------
function numero( quantidade )
--------------------------------------------------------------------------------
-- Descrio: 	Dado uma quantidade retorna um caracter indicando o seu nmero 
--		em termos morfolcicos: 's' - singular, 'p' - plural.
-- Parmetros:	
--		quantidade: inteiro represntando uma quantidade.

-- Retorno: 	Retorna um caracter indicando o nmero de uma quantidade em
--		termos morfolcicos: 's' - singular, 'p' - plural.		
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	not tonumber( quantidade ) then
		error( 'O parmetro valor deve ser um nmero inteiro.')
	end
	if 	quantidade > 1 then
		return 'p'
	else
		return 's'
	end
end

--------------------------------------------------------------------------------
function participio( verbo, genero )
--------------------------------------------------------------------------------
-- Descrio: 	Dado um verbo e um genero, retorna a conjugao do verbo no 
--		particpio (para pessoas no singular) e com o gnero especifi-
--		cado.
-- Parmetros:	verbo: string indicando o verbo a ser conjugado.
--		genero:string indicando o genro da pessoa.

-- Retorno: 	Retorna uma string contendo a conjugao de um verbo no partic-
--		pio (para pessoas no singular) de acordo com o gnero especifi-
--		cado.
	
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	not dicionario[verbo] then
		error( 'O verbo "'..tostring( verbo )..
		'" no consta do dicionrio.')
	end
	
	if 	dicionario[verbo].classe ~= 'verbo' then
		error( 'A classe de "'..tostring( verbo )..'" no dicionrio  "'
		..tostring(dicionario[verbo].classe)..'".')
	end
	
	if 	dicionario[verbo].excecoes then
		if 	dicionario[verbo].excecoes.participio then
			return dicionario[verbo].excecoes.participio
		end
	end
	
	local inicio, fim, radical, final = strfind(verbo,"^(%a*)(%a%r)$")
	--print( radical, final )

	--Verbos terminados em 'ar'  
	if  	final == 'ar' then
		return radical..'ad'..genero
	elseif final == 'er' or final == 'ir' then
		return radical..'id'..genero
	elseif final == 'or' then
		return radical..'ost'..genero
	end
end

--------------------------------------------------------------------------------
function plural( palavra )
--------------------------------------------------------------------------------
-- Descrio: 	Dada uma palavra, retorna seu plural. Se a palavra for uma ex-
--		cesso, retorna o plural especificado no dicionrio, caso con-
--		trrio, calcula o plural atravs da aplicao de regras gerais.

-- Parmetros:	
--		palavra: string indicando a palavra a ter o plural retornado.

-- Retorno: 	Retorna o plural de uma palavra dada.
		
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if not palavra then
		error( "O parmetro 'palavra'  nulo.")
	end

	local retorno = dicionario[strlower(palavra)]  
	if 	not retorno then
		error('A palavra "'..palavra..'" no consta do dicionrio.')	
	else	
		--Verificar se a palavra  uma excesso a regra para a deterina
		--o do plural
		if 	retorno.morfologia.excecoes and  
			retorno.morfologia.excecoes.plural then
			return retorno.morfologia.excecoes.plural
		--Caso no seja, aplica-se a regra geral
		else
			local inicio, fim, radical
			-------------------------------------------------------
			--Palavras terminadas em "l", o plural termina com "is"
			-------------------------------------------------------
			inicio, fim, radical = strfind( palavra, "(%w*)l$" )
			if 	radical then
				return radical.."is"
			-------------------------------------------------------
			--Palavras terminadas em "m", o plural termina com "ns"
			-------------------------------------------------------
			else
				inicio, fim, radical = strfind( palavra, "(%w*)m$" )
				if 	radical then
					return radical.."ns"
			--------------------------------------------------------
			--Palavras terminadas em "r", o plural termina com "res"
			--------------------------------------------------------
			  else
				  inicio, fim, radical = strfind( palavra, "(%w*)r$" )
				  if 	radical then
					    return radical.."res"

			-------------------------------------------------------
			--Outras palavras, o plural termina com "s"
			-------------------------------------------------------
				  else
					  return palavra.."s"
				  end
			  end	
	
			-------------------------------------------------------
			--FALTA O = ES | ES | OS !!!!
			-------------------------------------------------------
		  end
	  end
  end
end

--------------------------------------------------------------------------------
function presente( verbo, pessoa )
--------------------------------------------------------------------------------
-- Descrio: 	Dado um verbo e uma pessoa, retorna a conjugao do verbo para
--		esta pessoa no presente.
-- Parmetros:	verbo: string indicando o verbo a ser conjugado.
--		pessoa:string indicando a pessoa para a conjugao.

-- Retorno: 	Retorna uma string contendo a conjugao de um verbo no 
--		presente para uma dada pessoa.
	
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	not dicionario[verbo] then
		error( 'O verbo "'..tostring( verbo )..'" no consta do dicionrio.')
	end
	
	--if 	dicionario[verbo].classe ~= 'verbo' then
	--	error( 'A classe de "'..tostring( verbo )..'" no dicionrio  "'..
	--	tostring(dicionario[verbo].classe)..'".')
	--end

	if 	dicionario[verbo].excecoes then
		if 	dicionario[verbo].excecoes.presente then
			if 	dicionario[verbo].excecoes.presente[pessoa] then
				return dicionario[verbo].excecoes.presente[pessoa]
			end
		end
	end
	
	local inicio, fim, radical, final = strfind(verbo,"^(%a*)(%a%r)$")
	--print( radical, final )
	--Verbos terminados em 'ar'  
	if  	final == 'ar' or final  == 'er' or final  == 'ir' then
		if 	strsub(pessoa,1,1) == '3' then
			retorno = strsub(verbo,1,-2)
			if 	final == 'ir' then
				retorno =  gsub(retorno,  "%i$", "e")
			end
			
			if 	pessoa == '3p' then
				retorno = retorno..'m'  
			end
		end
	else --if final == 'or' then
		if 	strsub(pessoa,1,1) == '3' then
			retorno = radical..'e'
			if 	pessoa == '3p' then
				retorno = retorno..'m'  
			end
		end
	end
	
	return retorno
end


--------------------------------------------------------------------------------
function preterito_perfeito( verbo, pessoa )
--------------------------------------------------------------------------------
-- Descrio: 	Dado um verbo e uma pessoa, retorna a conjugao do verbo para
--		esta pessoa no pretrito perfeito.
-- Parmetros:	verbo: string indicando o verbo a ser conjugado.
--		pessoa:string indicando a pessoa para a conjugao.

-- Retorno: 	Retorna uma string contendo a conjugao de um verbo no 
--		presente para uma dada pessoa.
	
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	not dicionario[verbo] then
		error( 'O verbo "'..tostring( verbo )..'" no consta do dicionrio.')
	end
	
	if 	dicionario[verbo].classe ~= 'verbo' then
		error( 'A classe de "'..tostring( verbo )..'" no dicionrio  "'..
		tostring(dicionario[verbo].classe)..'".')
	end
	
	if 	dicionario[verbo].excecoes then
		if 	dicionario[verbo].excecoes.preterito_perfeito then
			if 	dicionario[verbo].excecoes.preterito_perfeito[pessoa] then
				return dicionario[verbo].excecoes.preterito_perfeito[pessoa]
			end
		end
	else
	  local inicio, fim, radical, final = strfind(verbo,"^(%a*)(%a%r)$")
	  --print( radical, final )
	  --Verbos terminados em 'er'  
	  if  final == 'er' then
		    if 	strsub(pessoa,1,1) == '3' then
	          if  pessoa == '3s' then
	              return radical.."eu"
	          else
	              return radical.."eram"  
	          end
	      end
		    error( 'A funo ainda no foi implementada para o verbo '..verbo )
	  end
	end
end

--------------------------------------------------------------------------------
function unifica_genero( palavra, genero )
--------------------------------------------------------------------------------
-- Descrio: 	Dada uma palavra e um gnero, retorna a unificao da pala-
--		vra com o genro. Exemplo: 
--		palavra = 'mesmo', gnero => retorno = 'mesma'. 

-- Parmetros:	
--		palavra: 	string indicando uma palavra.
--		genero:		caracter indicando um gnero ('o'- masculino e 
--		'a'- feminino)

-- Retorno: 	Retorna a a unificao da palavra com o nmero representado pela 
--		quantidade (veja exemplo acima, na descrio). Caso algum par-
--		metro seja invlido, finaliza a execuo e exibe uma mensagem de 
--		erro.
		
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	not palavra then
		error('O parmetro palavra  nil.')
	end

	if 	not genero then
		error('O parmetro genero  nil.')
	end

	if 	genero == 'a' then
		return feminino(palavra)
	elseif genero == 'o' then
		return  masculino(palavra)	
	end 
end


-------------------------------------------------------------------------------
function unifica_numero( palavra, quantidade )
--------------------------------------------------------------------------------
-- Descrio: 	Dada uma palavra e uma quantidade, retorna a unificao da pala-
--		vra com o nmero representado pela quantidade. Exemplo: 
--		palavra = 'casa', quantidade = 5 => retorno = 'casas'. Se quan-
--		tidade fosse 1, retornaria 'casa'.

-- Parmetros:	
--		palavra: 	string indicando uma palavra.
--		quantidade:	inteiro indicando uma quantidade.

-- Retorno: 	Retorna a a unificao da palavra com o nmero representado pela 
--		quantidade (veja exemplo acima, na descrio). Caso algum par-
--		metro seja invlido, finaliza a execuo e exibe uma mensagem de 
--		erro.
		
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	if 	not palavra then
		error('O parmetro palavra  nil.')
	end

	if 	not quantidade then
		error('O parmetro quantidade  nil.')
	end

	if 	quantidade > 1 then 
		return plural( palavra )
	else
		return palavra
	end
end

