------------------------- msg_acao_corretiva.lua ----------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	msg_acao_corretiva.lua
-- Descrio:  	Contm Mtodos das subclasses da classe Campo, geradores de 
--		mensagens de erro do tipo 'Ao Corretiva.' A gerao das 
--		mensagens  feita utilizando-se a tcnica de templates.
--		Os templates so textos prontos, com partes fixas e dinmicas.
--		Cada funo deste mdulo est associada a um tipo de erro e 
--		retorna uma string com a respectiva mensagem de erro.

-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes:
--     Quem    |   Quando   |   Motivo
-- ------------|------------|---------------------------------------------------
--   
--------------------------------------------------------------------------------

--Quando estiver debugando, retire os comentrios das 2 linhas a seguir
--cgilua.htmlheader()
--$debug

--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Limitados pela
--Interface
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoLimitadoInterface:msgPreencherCampo()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de falta de	preenchimento de 
--		um campo limitado pela interface obrigatrio.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  --print( self )
  return "Selecione uma opo."
end

--------------------------------------------------------------------------------
function CampoRadio:msgPreencherCampo()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de falta de	preenchimento de 
--		um campo limitado pela interface obrigatrio.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  --print( self )
  return "Marque uma das opes."
end

--------------------------------------------------------------------------------
function CampoCheckbox:msgPreencherCampo()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de falta de	preenchimento de 
--		um campo limitado pela interface obrigatrio.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  --print( self )
  return "Marque ao menos uma das opes."
end


--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Simples
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoSimples:msgAcertarTipo()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		simples com um valor com o tipo invlido.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return "Preencha o campo "..self:formaPreenchimento().."."
end

--------------------------------------------------------------------------------
function CampoSimples:msgPreencherCampo()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de falta de	preenchimento de 
--		um campo ilimitado pela interface simples obrigatrio.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
 return "Preencha o campo."
end

--------------------------------------------------------------------------------
function CampoSimples:msgRespeitarLimite( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		simples com um valor que no est dentro do intervalo de valores 
--		permitido para o campo.
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: String que identifica o tipo da restrio quanto ao 
--		intervalo de valores permitido, cuja violao causou o erro.
--		----------------------------------------------------------------
--		Operadores de restries quanto ao intervalo permitido
--		----------------------------------------------------------------
--		'<' - o valor digitado deve ser menor do que valor especificado
--		'<=' - o valor digitado deve ser menor ou igual a um valor espe-
--		cificado
--		'>' - o valor digitado deve ser maior do que valor especificado
--		'>=' - o valor digitado deve ser maior ou igual a um valor espe-
--		cificado

-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
 return "Preencha o campo com "..artigo_indefinido[ genero(self.nome)].." "..
	self.nome.." que esteja "..self:restricaoLimiteLn(parametros.operador)..
	" em ordem "..self.ordem.."."
end

--------------------------------------------------------------------------------
function CampoSimples:msgSatisfazerRestricaoTamanho( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		simples com um valor que no satisfaz a uma restrio de 
--		tamanho.
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: string que identifica o operador da restrio quanto 
--		ao tamanho do campo, cuja violao causou o erro.

--		Operadores de restries quanto ao tamanho do valor
--		'<=', '<' - o valor digitado est limitado por um tamanho mximo
--		'=='      - o valor digitado deve ser de um tamanho fixo
--		'>=', '>' - o valor digitado est limitado por um tamanho mnimo

-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  local operando = self.restricoes.tamanho[parametros.operador]	
  return "Preencha o campo com "..
  self.operadorRestricaoTamanhoLn[ parametros.operador ].." "..operando.." "..
  unifica_numero( self:tipoElementoBasico(), operando ).."."
end

--------------------------------------------------------------------------------
function CampoNumerico:msgRespeitarLimite( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		simples com um valor que no est dentro do intervalo de valores 
--		permitido para o campo.
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: String que identifica o tipo da restrio quanto ao 
--		intervalo de valores permitido, cuja violao causou o erro.
--		----------------------------------------------------------------
--		Operadores de restries quanto ao intervalo permitido
--		----------------------------------------------------------------
--		'<' - o valor digitado deve ser menor do que valor especificado
--		'<=' - o valor digitado deve ser menor ou igual a um valor espe-
--		cificado
--		'>' - o valor digitado deve ser maior do que valor especificado
--		'>=' - o valor digitado deve ser maior ou igual a um valor espe-
--		cificado

-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
 local msg_tipo = self:formaPreenchimento()
 if 	self.detalheTipoLn then
 	msg_tipo = gsub(msg_tipo, self.detalheTipoLn, "")
 end
	
 return "Preencha o campo "..msg_tipo.." "..
  	self:restricaoLimiteLn( parametros.operador ).."."
end



--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Data
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoData:msgCompletarData()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		data com um valor incompleto. (Falta ou o ms ou o ano da data,
--		 chamada somente quando a interface da data  de lista. 
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	return "Complete a data." 	
end

--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Estruturados
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoEstruturado:msgSatisfazerRestricaoTamanho( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de uma quan
--		tidade inadequada de componentes.  
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: string que identifica o operador da restrio quanto 
--		ao tamanho do campo, cuja violao causou o erro.

--		Operadores de restries quanto ao tamanho do valor
--		'<=', '<' - o valor digitado est limitado por um tamanho mximo
--		'=='      - o valor digitado deve ser de um tamanho fixo
--		'>=', '>' - o valor digitado est limitado por um tamanho mnimo


-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local genero_componente = genero(self.nomeComponente)
  	local operando = self.restricoes.tamanho[parametros.operador]	
	local quantidade, objetos

	--Os componentes obrigatrios so indeterminados
	if 	self.restricoes.subconjunto == '#indeterminado' then
		--O objeto  o nome genrico dos componentes exemplo: 'campos', 
		--'fases', 'itens', etc, unificado com o operando da restrio 
		--(se for 1  singular. 
		objetos = unifica_numero(self.nomeComponente, operando)
		
		--Se a restrio de tamanho  '== nmero de componentes'
		---------------------------------------------------------------
		--Msg: "Preencha todos os campos."
		---------------------------------------------------------------
		if 	parametros.operador == '==' and 
				operando == self.numComponentes then
 			
			--Ser usada a expresso 'todos os', unificada com o 
			--gnero de self.nomeComponente
			quantidade = 'tod'..genero_componente..'s '..
			genero_componente..'s'
		--Seno 
		---------------------------------------------------------------
		--Msg: "Preencha no mximo/no mnimo/exatamente 5 campos."
		---------------------------------------------------------------
		else	
			--Transformar a restrio para linguagem natural
			quantidade = 
			self.operadorRestricaoTamanhoLn[ parametros.operador ].." "..
			numero_ou_artigo( operando, genero_componente )
		end
	--Os componentes obrigatrios so determinados
	else
		---------------------------------------------------------------
		--Msg: "Preencha os campos: ...."
		---------------------------------------------------------------
		--Se a restrio de tamanho  '== nmero de componentes', na 
		--verdade so todos os componentes obrigatrios, pois no caso 
		--de subconjunto determinado, self.numComponentes, contm somen
		--te o nmero de componentes obrigatrios
		if 	(parametros.operador == '==' or parametros.operador == '>=' 
				or parametros.operador == '>') and 
				operando == self.numComponentes then
			
			--Ser usada a expresso 'obrigatoriamente', unificada com o 
			--gnero de self.nomeComponente
			quantidade = 'obrigatoriamente'
			local coordenacao = self:coordenaNomesSubconjunto()
			objetos = self:objetosVerboImperativo( coordenacao, 
			genero_componente, self.numComponentes)
		--Seno 
		---------------------------------------------------------------
		--Msg: "Preencha no mximo/no mnimo/exatamente 5 dos campos: 
		-- ..."
		---------------------------------------------------------------
		else	
			--Transformar a restrio para linguagem natural
			quantidade = 
			self.operadorRestricaoTamanhoLn[ parametros.operador ].." "..
			numero_ou_artigo( operando, genero_componente )
			--Obter a coordena
			objetos = "d"..genero_componente..'s '..
			self.nomeComponente..'s: '..self:coordenaNomesSubconjunto()
		end
	end

	return "Preencha "..
	quantidade.." "..objetos..'.'
end

--------------------------------------------------------------------------------
function CampoEstruturado:msgSatisfazerRestricaoTamanhoCorrigirComponentes( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento invlido de 
--		um campo estruturado, onde ocorreram tanto preenchimentos inv-
--		lidos de elementos como preenchimento de uma quantidade 
--		inadequada de componentes. 
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: string que identifica o operador da restrio quanto 
--		ao tamanho do campo, cuja violao causou o erro.

--		Operadores de restries quanto ao tamanho do valor
--		'<=', '<' - o valor digitado est limitado por um tamanho mximo
--		'=='      - o valor digitado deve ser de um tamanho fixo
--		'>=', '>' - o valor digitado est limitado por um tamanho mnimo
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return self:msgSatisfazerRestricaoTamanho( parametros ).." "..
  	self:msgCorrigirComponentes()		
end

--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Compostos
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoComposto:msgCorrigirComponentes()
--------------------------------------------------------------------------------
-- Descrio: 	Primitiva geradora da mensagem relacionada ao erro de preenchi-
--		mento incorreto de componentes de um campo composto.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local genero_componente = genero(self.nomeComponente)
	local itens_coordenacao, num_itens 
	local coordenacao
	------------------------------------------------------------------------
	if 	self.tipoMsg == 'global_local'  then
	------------------------------------------------------------------------
		coordenacao = self:coordenaNomeCampos( 'Invalidos' )
		return "Corrija "..
		self:objetosVerboImperativo( coordenacao, genero_componente, 
		self.contInvalidos)..
		", seguindo as instrues junto "..
		unifica_numero(conjuncao( 'a', genero_componente), self.contInvalidos ).." "..
		unifica_numero( unifica_genero('mesmo', genero_componente ), self.contInvalidos ).."."
	end
	
	------------------------------------------------------------------------
	--self.tipoMsg == 'global'  
	------------------------------------------------------------------------
  	itens_coordenacao, num_itens = self:preparaCoordenacaoCamposInvalidos('C')
  	coordenacao = coordena( itens_coordenacao, num_itens, "e" )
	
  	return "Preencha "..coordenacao.."."
  	
  	--self:objetosVerboImperativo( coordenacao, genero_componente, 
		--num_itens).."."		
end

--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Vetor
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoVetor:msgCorrigirComponentes()
--------------------------------------------------------------------------------
-- Descrio: 	Primitiva geradora da mensagem relacionada ao erro de preenchi-
--		mento incorreto de componentes de um campo vetor.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local genero_componente = genero(self.nomeComponente)
	
	local coordenacao = self:coordenaNomeCampos('Invalidos')
	
	------------------------------------------------------------------------
	if 	self.tipoMsg == 'global_local'  then
	------------------------------------------------------------------------
		return "Corrija "..self:objetosVerboImperativo( coordenacao, 
		genero_componentes, self.contInvalidos )..
		", seguindo as instrues junto "..
		unifica_numero(conjuncao( 'a', genero_componente), 
		self.contInvalidos).." "..
		unifica_numero( unifica_genero('mesmo', genero_componente ), 
		self.contInvalidos  ).."."
	else
	------------------------------------------------------------------------
	--self.tipoMsg == 'global'  
	------------------------------------------------------------------------
  		--return "Preencha "..self:objetosVerboImperativo( coordenacao,
  		--genero_componentes, self.contInvalidos )..' '..
  		--self.componentes:formaPreenchimentoCompleto().."."
  		
  		local 	pronome_obliquo
  		if 	self.contInvalidos > 1 then
  			pronome_obliquo = genero_componente.."s"	
  		else
  			pronome_obliquo =  genero_componente
  		end							
  		return "Corrija "..self:objetosVerboImperativo( coordenacao,
  		genero_componentes, self.contInvalidos )..' '..
  		" preenchendo-"..pronome_obliquo.." "..
  		self.componentes:formaPreenchimentoCompleto().."."
	end
end