------------------------- msg_diagnostico.lua ----------------------------
--
-- PUC-Rio
-- Departamento de Informtica
--
-- Projeto:	VALIDAFORM
-- Arquivo: 	msg_diagnostico.lua
-- Descrio:  	Contm Mtodos das subclasses da classe Campo, geradores de 
--		mensagens de erro do tipo 'Diagnstico.' A gerao das men-
--		sagens  feita utilizando-se a tcnica de templates.
--		Os templates so textos prontos, com partes fixas e dinmicas.
--		Cada funo deste mdulo retorna uma string com a mensagem de 
--		erro gerada.

-- Data:	21/04/1999
-- Autor:      	Gilda Dahis
--
-- 
-- Alteraes:
--     Quem    |   Quando   |   Motivo
-- ------------|------------|---------------------------------------------------
--   
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Limitados pela
--Interface
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function CampoLimitadoInterface:msgCampoObrigatorioVazio()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de falta de	preenchimento de 
--		um campo limitado pela interface obrigatrio.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return "Voc deve selecionar obrigatoriamente uma opo."
end

--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Simples
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
function CampoSimples:msgCampoObrigatorioVazio()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de falta de preenchimento de 
--		um campo ilimitado pela interface simples obrigatrio.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	return " obrigatrio o preenchimento deste campo."
end	

--------------------------------------------------------------------------------
function CampoSimples:msgChaveJaCadastrada()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um 
--		campo 'chave' com um valor j cadastrado no BD.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  local genero_nome
  local inicio = strfind( self.nome, " ")
  if inicio then
      --print( inicio )
     genero_nome =  genero(strsub(self.nome, 1, inicio-1))
  else
    genero_nome = genero(self.nome)
  end  
  return primeira_maiuscula( self.nome ).." j cadastrad"..
  	genero_nome.."."
end

--------------------------------------------------------------------------------
function CampoSimples:msgLimiteUltrapassado( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		simples com um valor que no est dentro do intervalo de valores 
--		permitido para o campo.
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: String que identifica o tipo da restrio quanto ao 
--		intervalo de valores permitido, cuja violao causou o erro.
--		----------------------------------------------------------------
--		Operadores de restries quanto ao intervalo permitido
--		----------------------------------------------------------------
--		'<' - o valor digitado deve ser menor do que valor especificado
--		'<=' - o valor digitado deve ser menor ou igual a um valor espe-
--		cificado
--		'>' - o valor digitado deve ser maior do que valor especificado
--		'>=' - o valor digitado deve ser maior ou igual a um valor espe-
--		cificado

-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
 --return "O campo deve ser preenchido com "..artigo_indefinido[ genero(self.nome)].." "..
 --	self.nome.." que esteja "..self:restricaoLimiteLn(parametros.operador)..
 --	" em ordem "..self.ordem.."."

  local genero_nome
  local inicio = strfind( self.nome, " ")
  if inicio then
      --print( inicio )
     genero_nome =  genero(strsub(self.nome, 1, inicio-1))
  else
    genero_nome = genero(self.nome)
  end  
  return primeira_maiuscula( genero_nome )..' '..self.nome.." deve estar "..
	self:restricaoLimiteLn(parametros.operador)..
 	" em ordem "..self.ordem.."."
end


--------------------------------------------------------------------------------
function CampoNumerico:msgLimiteUltrapassado( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		simples com um valor que no est dentro do intervalo de valores 
--		permitido para o campo.
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: String que identifica o tipo da restrio quanto ao 
--		intervalo de valores permitido, cuja violao causou o erro.
--		----------------------------------------------------------------
--		Operadores de restries quanto ao intervalo permitido
--		----------------------------------------------------------------
--		'<' - o valor digitado deve ser menor do que valor especificado
--		'<=' - o valor digitado deve ser menor ou igual a um valor espe-
--		cificado
--		'>' - o valor digitado deve ser maior do que valor especificado
--		'>=' - o valor digitado deve ser maior ou igual a um valor espe-
--		cificado

-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  --local operando = self.restricoes.limite[parametros.operador].nome
  local genero_nome
  --if 	not operando then
  --	operando = self.restricoes.limite[parametros.operador].valor
  --end
  local inicio = strfind( self.nome, " ")
  if inicio then
      --print( inicio )
     genero_nome =  genero(strsub(self.nome, 1, inicio-1))
  else
    genero_nome = genero(self.nome)
  end  
  return primeira_maiuscula( genero_nome )..' '..	self.nome.." deve ser "..
  	self:restricaoLimiteLn( parametros.operador ).."."
end


-----------------------------------------------------------------------------------
function CampoSimples:msgRestricaoTamanhoInsatisfeita( parametros )
-----------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		simples com um valor que no satisfaz a uma restrio de tamanho.
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: string que identifica a restrio quanto ao tamanho
--		do campo, cuja violao causou o erro.
 
--		Operadores de restries quanto ao tamanho do valor
--		'<=' - o valor digitado est limitado por um tamanho mximo
--		'='  - o valor digitado deve ser de um tamanho fixo
--		'>=' - o valor digitado est limitado por um tamanho mnimo

-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  local operando = self.restricoes.tamanho[parametros.operador]
  local msg =  "O campo deve ter "..
	self.operadorRestricaoTamanhoLn[ parametros.operador ].." "..
  	operando.." "..unifica_numero( self.tipoElementoBasico(), operando )
	
	--Se existe algum detalhe sobre o tipo, a mensagem de tamanho deve 
	--cont-lo
	if 	self.detalheTipoLn then 
		msg = msg.." ("..self.detalheTipoLn..")"	
	end
	msg = msg..". Voc digitou "..self.tamanhoValor.."."
	return msg
end

--------------------------------------------------------------------------------
function CampoSimples:msgTipoInvalido()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		simples com um valor com o tipo invlido.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return "O campo deve ser preenchido "..self:formaPreenchimento().."." 
end

-------------------------------------------------------------------------------
function CampoSimples:msgValorInvalido()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		com um valor do tipo correto, porm que no faz parte do conjun-
--		to de valores aceitos para o preenchimento do campo. OBS: Para 
--		este tipo de erro, no h mensagem com configurao 'C'.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return 	'"'..self.valor..
  		'" no  um valor permitido para o preenchimento do campo.'
end

--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Data
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function CampoData:msgDataIncompleta()
--------------------------------------------------------------------------------
-- Descrio: 	Primitiva geradora da mensagem relacionada ao erro de preenchi-
--		mento de um campo data com um valor incompleto. (Falta ou o ms 
--		ou o ano da data, chamada somente quando a interface do campo  
--		do tipo 'lista').
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	return "A data est incompleta." 	
end

--------------------------------------------------------------------------------
function CampoData:msgDiaFevereiroAcimaLimite( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		data com um valor de um dia em fevereiro, maior que o dia mximo 
--		de fevereiro no ano da data. OBS: Para este tipo de erro, no h 
--		mensagem com configurao 'C'.
-- Parmetros:	
--		numero_dias: inteiro indicando o nmero de dias de fevereiro no
--		ano da data.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return "Neste ano, Fevereiro tem "..parametros.numero_dias.." dias."
end

--------------------------------------------------------------------------------
function CampoData:msgDiaInvalido()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a da mensagem relacionada ao erro de preenchimento de um 
--		campo data com um valor de um dia fora dos limites de 1 e 31. 
--		OBS: Para este tipo de erro no h mensagem com configuracao 
--		'C'.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return "Dia no pode ser menor do que 1, nem maior do que 31."
end

--------------------------------------------------------------------------------
function CampoData:msgDiaMaiorQue30()
--------------------------------------------------------------------------------
-- Descrio: 	Primitiva geradora da mensagem relacionada ao erro de preenchi-
--		mento de um campo data com um valor de um dia maior do que 30, 
--		quando o ms da data s tem 30 dias. OBS: Para este tipo de erro
--		no h mensagem com configuracao 'C'.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return "Este ms s tem 30 dias."
end

--------------------------------------------------------------------------------
function CampoData:msgMesInvalido()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento de um campo 
--		data com um valor de um mes fora dos limites de 1 e 12. OBS: 
--		Para este tipo de erro no h mensagem com configuracao 'C'.
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return "Ms no pode ser menor do que 1, nem maior do que 12."
end

--------------------------------------------------------------------------------
--Mtodos para a gerao de mensagens associadas a campos Estruturados
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function CampoComposto:msgComponentesInvalidos()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento incorreto 
--		de componentes de um campo estruturado. 
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local genero_componente = genero(self.nomeComponente)
	local itens_coordenacao, num_itens 
	local coordenacao
	------------------------------------------------------------------------
	if 	self.tipoMsg == 'global_local'  then
	------------------------------------------------------------------------
		coordenacao = self:coordenaNomeCampos( 'Invalidos' )
		
		local verbo = conjugacao( 'preencher', '3'..numero(self.contInvalidos), 
		genero_componente, 'preterito_perfeito', 'passiva', 'ser' )

		return primeira_maiuscula(unifica_numero(genero_componente, 
			self.contInvalidos)).." "..
			unifica_numero(self.nomeComponente, 
			self.contInvalidos).." "..coordenacao.." "..verbo..
			" incorretamente. Corrija- "..
			unifica_numero(genero_componente, self.contInvalidos)..
			", seguindo as instrues junto "..
			unifica_numero(conjuncao( 'a', genero_componente), self.contInvalidos ).." "..
			unifica_numero( unifica_genero('mesmo', genero_componente ), self.contInvalidos ).."."
	end
	
	------------------------------------------------------------------------
	--self.tipoMsg == 'global'  
	------------------------------------------------------------------------
  	itens_coordenacao, num_itens = self:preparaCoordenacaoCamposInvalidos( 'D' )

	coordenacao = coordena( itens_coordenacao, num_itens, "e" )
	
  	return coordenacao.."."		
end


--------------------------------------------------------------------------------
function CampoVetor:msgComponentesInvalidos()
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento incorreto 
--		de componentes de um campo estruturado. 
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local genero_componente = genero(self.nomeComponente)
	local numero_componentes = numero( self.contInvalidos )
	local retorno = self:coordenaNomeCampos('Invalidos')

	retorno = self:objetosVerboImperativo( retorno, genero_componente, 
		self.contInvalidos)

	
	retorno = primeira_maiuscula(retorno).." "..
		conjugacao('dever', '3'..numero_componentes, genero_componente,
		'presente', 'ativa').." "..
		conjugacao('preencher', '3'..numero_componentes, 
		genero_componente, 'infinitivo impessoal', 'passiva', 'ser')
	
	------------------------------------------------------------------------
	if 	self.tipoMsg == 'global_local'  then
	------------------------------------------------------------------------
		return retorno..", conforme as instrues junto "..
		unifica_numero(conjuncao( 'a', genero_componente), 
		self.contInvalidos).." "..
		unifica_numero( unifica_genero('mesmo', genero_componente ), 
		self.contInvalidos  ).."."
	else
	------------------------------------------------------------------------
	--self.tipoMsg == 'global'  
	------------------------------------------------------------------------
		return retorno.." "..
		self.componentes:formaPreenchimentoCompleto()
	end
end

--------------------------------------------------------------------------------
function CampoEstruturado:msgRestricaoTamanhoInsatisfeita( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Primitiva geradora da mensagem relacionada ao erro de preenchi-
--		mento incorreto de componentes de um campo estruturado.
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: string que identifica o operador da restrio quanto 
--		ao tamanho do campo, cuja violao causou o erro.

--		Operadores de restries quanto ao tamanho do valor
--		'<=', '<' - o valor digitado est limitado por um tamanho mximo
--		'=='      - o valor digitado deve ser de um tamanho fixo
--		'>=', '>' - o valor digitado est limitado por um tamanho mnimo

-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.
-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
	local genero_componente = genero(self.nomeComponente)
 	local operando = self.restricoes.tamanho[parametros.operador]	
	local quantidade, objetos

	--Os componentes obrigatrios so indeterminados
	if 	self.restricoes.subconjunto == '#indeterminado' then
		--O objeto  o nome genrico dos componentes exemplo: 'campos', 
		--'fases', 'itens', etc, unificado com o operando da restrio 
		--(se for 1  singular. 
		objetos = unifica_numero(self.nomeComponente, operando)
		
		--Se a restrio de tamanho  '== nmero de componentes'
		---------------------------------------------------------------
		--Msg: " obrigatrio o preenchimento de todos os campos."
		---------------------------------------------------------------
		if 	parametros.operador == '==' and 
			operando == self.numComponentes then
			
			--Ser usada a expresso 'todos os', unificada com o 
			--gnero de self.nomeComponente
			quantidade = 'de tod'..genero_componente..'s '..
			genero_componente..'s'
		--Seno 
		---------------------------------------------------------------
		--Msg: " obrigatrio o preenchimento de no mnimo/exatamente 
		--5 campos."
		---------------------------------------------------------------
		else	
			--Transformar a restrio para linguagem natural
			quantidade = 'de '..
			self.operadorRestricaoTamanhoLn[ parametros.operador ].." "..
			numero_ou_artigo( operando, genero_componente )			
		end
	--Os componentes obrigatrios so determinados
	else
		---------------------------------------------------------------
		--Msg: " obrigatrio o preenchimento dos campos: ...."
		---------------------------------------------------------------
		--Se a restrio de tamanho  '== nmero de componentes', na 
		--verdade so todos os componentes obrigatrios, pois no caso 
		--de subconjunto determinado, self.numComponentes, contm somen
		--te o nmero de componentes obrigatrios
		
		--Obter a coordenao dos objetos
		--print( 'self.componentes:', self.numComponentes )
		if 	self.numComponentes == 1 then
			objetos = "d"..genero_componente.." "..self.nomeComponente.." "
		else	
			objetos = "d"..genero_componente.."s "..plural(self.nomeComponente)..': '
		end	
			
		objetos = objetos..self:coordenaNomesSubconjunto()

		if 	(parametros.operador == '==' or parametros.operador == '>=' 
				or parametros.operador == '>') and 
			operando == self.numComponentes then
			quantidade = ''
		--Seno 
		---------------------------------------------------------------
		--Msg: " obrigatrio o preenchimento de no mnimo/exatamente 
		--5 dos campos: ..."
		---------------------------------------------------------------
		else	
			--Transformar a restrio para linguagem natural
			quantidade = "de "..
			self.operadorRestricaoTamanhoLn[ parametros.operador ].." "..
			numero_ou_artigo( operando, genero_componente )
		end
	end

	---------------------------------------------------------------
	--Msg: "No devem ser preenchidos mais de 5 campos/dos campos:
	--..."
	---------------------------------------------------------------
	--Se a o operador da restrio  '<=' ento o template  diferente
	if 	parametros.operador == '<=' or parametros.operador == '<' then
		return "No "..conjugacao('dever', '3'..numero(operando), genero_componente,
				'presente', 'ativa').." "..
		conjugacao('preencher', '3'..numero(operando), genero_componente,
		'infinitivo impessoal', 'passiva', 'ser').." mais de "..
		numero_ou_artigo( operando, genero_componente ).." "..objetos.."."	
	else
		return " obrigatrio o preenchimento "..
		quantidade.." "..objetos..'.'
	end
end

--------------------------------------------------------------------------------
function 
CampoEstruturado:msgRestricaoTamanhoInsatisfeitaComponentesInvalidos( parametros )
--------------------------------------------------------------------------------
-- Descrio: 	Gera a mensagem relacionada ao erro de preenchimento invlido de 
--		um campo estruturado, onde ocorreram tanto preenchimentos inv-
--		lidos de elementos como preenchimento de uma quantidade 
--		inadequada de componentes. 
-- Parmetros:	
--		parametros: tabela com parmetros para a gerao da mensagem.
--		----------------------------------------------------------------
--		-- campos da tabela parmetros
		----------------------------------------------------------------
--		operador: string que identifica o operador da restrio quanto 
--		ao tamanho do campo, cuja violao causou o erro.

--		Operadores de restries quanto ao tamanho do valor
--		'<=', '<' - o valor digitado est limitado por um tamanho mximo
--		'=='      - o valor digitado deve ser de um tamanho fixo
--		'>=', '>' - o valor digitado est limitado por um tamanho mnimo
-- Retorno: 	Retorna uma string com a mensagem relacionada ao erro.

-- Autora: 	Gilda Dahis
-- Data: 	06/01/1998
--------------------------------------------------------------------------------
  return self:msgRestricaoTamanhoInsatisfeita( parametros ).." "..
  	self:msgComponentesInvalidos()		
end